/******************************************************************************
 * A x e l                                                                   *
 ******************************************************************************
 * ObjectManager                                                             *
 * 2007-04-13                                                                *
 * Julien Wintz                                                              *
 ******************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis                   *
 ******************************************************************************
 * Comments :                                                                *
 *****************************************************************************/

# include <QAxel/ObjectManager.h>

namespace QAxel {

/****************************************************************************/
/*! Creates the manager. Does very little currently.
*/
/****************************************************************************/
ObjectManager::ObjectManager () 
{

}

/****************************************************************************/
/*! Destroys the instance. Clears the list of all QAxelObjects.
*/
/****************************************************************************/
ObjectManager::~ObjectManager () 
{

}

/***************************************************************************** 
 * Define the static member.
 ****************************************************************************/
ObjectManager * ObjectManager::m_instance = 0;

/****************************************************************************/
/*! Returns the sole instance of the class. This is the way you should access
    this class, no other way is ok. This method will create the sole instance
	(transparently) if it does not already exist.
*/
/****************************************************************************/
ObjectManager * ObjectManager::instance(void) 
{
    if(!ObjectManager::m_instance) {
	m_instance = new ObjectManager ();
	Q_ASSERT_X(m_instance != 0,
		   "Object Manager",
		   "The instance has not been succesfully created") ;
    }

    return m_instance ;
}

void ObjectManager::addObject(QAxelObject * object)
{
    m_items << object ;
    emit objectAdded(object) ;
}

void ObjectManager::removeObject(QAxelObject * object)
{
    m_items.removeAll(object) ;
    emit objectRemoved(object) ;
}

/****************************************************************************/
/*! Attempts to find an \link QAxelObject QAxelObject \endlink from the 
  manager based on a \a name. This is the human-readable name assigned to the 
  QAxelObject. If such an QAxelObject is found it is returned, otherwise \c 0 
  is returned.

  \arg const QString & name - name to search for.
*/
/****************************************************************************/
QAxelObject * ObjectManager::findByName (const QString & name) 
{
    QListIterator<QAxelObject*> iter (m_items) ;

    while(iter.hasNext()) {
	QAxelObject * tmp = iter.next() ;
	if(tmp->name() == name)
	    return tmp ;
    }

    return 0 ;
}

/****************************************************************************/
/*! Attempts to find an \link QAxelObject QAxelObject \endlink from the 
  manager based on a \a yype. This is the classification assigned to the 
  QAxelObject. If such an QAxelObject is found it is returned, otherwise \c 0 
  is returned.

  \arg const QString & type - type to search for.
*/
/****************************************************************************/
QAxelObject * ObjectManager::findByType (const QString & type) 
{
    QListIterator <QAxelObject *> iter(m_items) ;

    while(iter.hasNext()) {
	QAxelObject * tmp = iter.next() ;
	if(tmp->type() == type)
	    return tmp ;
    }

    return 0 ;
}

QList<QAxelObject *> ObjectManager::objects(void)
{
    return m_items ;
}

} ; // namespace QAxel ;
