/*****************************************************************************
 * A x e l
 *****************************************************************************
 * PluginManager
 * 2006-05-16
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006-2007 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments : Does it really works ?
 ****************************************************************************/

# include <QWidget>
# include <QString>
# include <QStringList>
# include <QDir>
# include <QLibrary>
# include <QPluginLoader>

# include <QAxel/QInitializablePluginInterface.h>
# include <QAxel/PluginManager.h>

# include <QGui/color.h>
# include <QGui/PluginDialog.h>
# include <QGui/MainWindow.h>

using namespace std ;

PluginManager * PluginManager::instance(void)
{
    if (!f_instance) {
	f_instance = new PluginManager();
    }
    return f_instance;
}

bool PluginManager::loadPlugin(QObject* plugin)
{
    using namespace QAxel;
    if(plugin) {
	QPluginInterface * iPlugin = qobject_cast<QAxel::QPluginInterface *>(plugin);
	if (iPlugin) {
	    f_plugins.insert(iPlugin->name(), plugin) ;
	    QInitializablePluginInterface * iInitPlugin = qobject_cast<QAxel::QInitializablePluginInterface *>(plugin);
	    if(iInitPlugin)
		iInitPlugin->init(mainwindow);
	    return true ;
	}
    }
    return false ;
}

void PluginManager::loadPlugins(QString applicationPath, MainWindow * mw)
{
    mainwindow = mw ;
    pluginsDir = new QDir(applicationPath) ;

# if defined(Q_OS_WIN)
    if (pluginsDir->dirName().toLower() == "debug" || pluginsDir->dirName().toLower() == "release")
	pluginsDir->cdUp();
# elif defined(Q_OS_MAC)
    if (pluginsDir->dirName() == "MacOS") {
# if defined(GENERATE_BUNDLE)
	pluginsDir->cdUp();
	pluginsDir->cd("Plugins");
# else
	pluginsDir->cdUp();
	pluginsDir->cdUp();
	pluginsDir->cdUp();
	pluginsDir->cdUp();
	pluginsDir->cd("plugins") ;
# endif
    }
# elif defined(Q_OS_UNIX)
    pluginsDir->cdUp() ;
    pluginsDir->cd("plugins");
# endif
  
    foreach (QString fileName, pluginsDir->entryList(QDir::Files)) {
	if( ! QLibrary::isLibrary(fileName) ) continue;
	QPluginLoader * loader = NULL ;
	QString name = pluginsDir->absoluteFilePath(fileName) ;
	if(f_loaders.contains(name)) {
	    loader = f_loaders[name] ;
	} else {
	    loader = new QPluginLoader(name) ;
	    f_loaders.insert(name, loader) ;
	}

	cout << "Loading " << FG_YELLOW << fileName.toAscii().constData() << NOCOLOR << "... " << flush ;
    
	QObject * plugin = loader->instance() ;
	if(plugin && loadPlugin(plugin)) {
	    cout << "[" << FG_GREEN << "success" << NOCOLOR << "]" << endl ;
	    pluginFileNames << fileName ;
	} else {
	    cout << "[" << FG_RED << "failure" << NOCOLOR << "]" << endl ;
	    if(!plugin) {
		cerr << "  " << FG_MAGENTA
		     << "(" << loader->errorString().toAscii().constData() << ")"
		     << NOCOLOR << endl;
	    }
	}
	if(!loader->isLoaded())
	    delete loader;
    }
}

/**
 * Unloads the named plugin. If the plugin cannot be unloaded, the method
 * returns false but the plugin is still removed from the active list
 * and not used for further processing (This is most relevant on OSX
 * where it seems we can't ever unload a plugin).
 * \param plugin Name of the plugin to unload
 * \return Plugin unloaded result.
 */ 
bool PluginManager::unloadPlugin(const QString& plugin)
{
    if(!f_plugins.contains(plugin)) {
	cout << "Plugin " << plugin.toAscii().constData() << " wasn't found when trying to unload" << endl ;
	return false;
    }

    cout << "attempting to disconnect " << plugin.toAscii().constData() << endl ;
    f_plugins[plugin]->disconnect() ;

    QPluginLoader * loader = f_loaders[plugin];
    bool done = false ;
    if(loader->unload()) {
	//if we're done with the plugin completely and it's unloaded
	// we can delete the loader;
	done = true ;
	delete loader ;
	f_loaders.remove(plugin) ;
    }
    //if the plugin was not unloaded, we remove it from the active
    // list, but keep the loader so we can obtain future instances
    f_plugins.remove(plugin) ;
    return done ;
}

void PluginManager::aboutPlugins(QWidget * parent) 
{
    PluginDialog dialog(pluginsDir->path(), pluginFileNames, parent);
    dialog.exec();
}

QObject * PluginManager::plugin(QString pluginName)
{
    if(f_plugins.keys().contains(pluginName))
	return f_plugins[pluginName] ;
    return NULL ;
}

QPluginLoader * PluginManager::loader(QString pluginName)
{
    if(f_loaders.keys().contains(pluginName))
	return f_loaders[pluginName] ;
    return NULL ;
}

PluginManager * PluginManager::f_instance = NULL ;

