/*****************************************************************************
 * A x e l
 *****************************************************************************
 * Plugin Manager
 * 2006-04-23
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2007 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :         
 *
 ****************************************************************************/

# ifndef PLUGINMANAGER_H
# define PLUGINMANAGER_H

# include <QObject>
# include <QWidget>
# include <QString>
# include <QStringList>
# include <QMap>
# include <QDir>
# include <QPluginLoader>

class MainWindow ;

/****************************************************************************/
/*!
  \class PluginManager PluginManager.h
  \brief The class PluginManager provides a way to load and acces Axel plugins.

  Usage:
  <code>
  QCoreInterface * iCore = dynamic_cast<QCoreInterface *>(PluginManager::instance()->plugin("Core")) ;
  </code>
*/
/****************************************************************************/

class PluginManager
{
public :
    static PluginManager * instance(void) ;
    
    bool loadPlugin(QObject* plugin) ;
    void loadPlugins(QString applicationPath, MainWindow * mw) ;
    void aboutPlugins(QWidget * parent) ;
    bool unloadPlugin(const QString& plugin) ;
    
    QObject * plugin(QString pluginName) ;
    QPluginLoader * loader(QString pluginName) ;
    
private :
    static PluginManager * f_instance ;
    
    QDir * pluginsDir ;
    QStringList pluginFileNames ;

    MainWindow * mainwindow ;
    
    QMap<QString, QObject *> f_plugins ;          /*! Name of the plugin -> plugin interface eg. Synaps -> SynapsInterface */
    QMap<QString, QPluginLoader *> f_loaders ;    /*! File of the plugin -> plugin loader    eg. libQSynaps.dylib -> QPluginLoader */
} ;

# endif
