/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QAxelObject
 * 2006-01-25
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************/

# ifndef QAXELOBJECT_H
# define QAXELOBJECT_H

# include <QObject>
# include <QGLViewer/qglviewer.h>

class QColor ;
class QString ;

using namespace qglviewer ;

namespace QAxel {

/****************************************************************************/
/*!
  \class QAxelObject QAxelObject.h
  \brief The class QAxelObject provides ...

  More ...
*/
/****************************************************************************/
class QAxelObject : public QObject 
{
    Q_OBJECT 
    Q_PROPERTY(QString type READ type WRITE setType)
    Q_PROPERTY(QString name READ name WRITE setName)
    Q_PROPERTY(Mode mode READ mode WRITE setMode)
    Q_ENUMS(Mode)
    Q_OVERRIDE(QString objectName SCRIPTABLE true)
	
public:
             QAxelObject(QAxelObject * parent = 0) ;
    virtual ~QAxelObject(void) ;

    virtual void draw(void) = 0 ;	
    virtual void drawWithNames(void) = 0 ;
	
    ManipulatedFrame * frame(void) ;
    
    void setName(QString name) { setObjectName(name) ; }
    void setType(QString type) { type_ = type ; }
    
    QString name(void) { return objectName() ; }
    QString type(void) { return type_ ; }
    
    QString position(void) ;
    QString orientation(void) ; 
    
    QColor * color(void) ;
    QColor * selectionColor(void) ;
    QColor * grabbingColor(void) ;
    void setColor(QColor color) ;
    void setObjectColor(QColor * color) ;
    void setRGB(int r, int g, int b) ;
    void setRGBA(int r, int g, int b, int a) ;  
    static void setSelectionColor(QColor * color) ;
    static void setGrabbingColor(QColor * color) ;

    void glColor(void) ;
    void glColorAlpha(void) ;
    void glObjectColor(void) ;
    void glObjectColorAlpha(void) ;
    void glSelectionColor(void) ;
    void glSelectionColorAlpha(void) ;
    void glGrabbingColor(void) ; 
    
    void hide() { hidden_ = true ; }
    bool hidden(void) { return hidden_ ; }
    void setHidden(bool hidden) { hidden_ = hidden ; }
    
    void blacklist(void) { blacklisted_ = true ; }
    bool blacklisted(void) { return blacklisted_ ; }
    void setBlackListed(bool b) { blacklisted_ = b ; }
    
    enum Mode { None, Edit, Extrude, Grab } ;
    Mode mode() const { return mode_ ; }
    void setMode(Mode mode) { mode_ = mode ; }
    
public slots:
    void select(void) ;
    void deselect(void) ;
    bool selected(void) ;
    
protected:
    QString name_ ;
    QString type_ ;
    Mode mode_ ;
    
    ManipulatedFrame * frame_ ;
    bool selected_ ;
    bool hidden_ ;
    bool blacklisted_ ;
    
    QColor * color_ ;
    static QColor * selectionColor_ ;
    static QColor * grabbingColor_ ;
    
public:
    QAxelObject * parent(void) ;
    void setParent(QAxelObject * parent) ;

    virtual void addChild(QAxelObject * child) ;
    virtual void addChild(QAxelObject * child, const unsigned int index) ;
    virtual QAxelObject * removeChild(QAxelObject * child) ;
    virtual QAxelObject * findChild(QAxelObject * child) ;
    
    void removeAllChildren(const bool deleteAll = true) ;

protected:
    QAxelObject * m_parent ;
    QList<QAxelObject *> m_children ;
} ;
    
} ; // namespace QAxel

# endif
