/*****************************************************************************
 * Axel
 *****************************************************************************
 * Curves in Axel
 * 2007-05-22
 * Ashish Reddy
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# ifndef QBSPLINECURVE_H
# define QBSPLINECURVE_H

# include <QAxel/QParametricCurve.h>

namespace QAxel {

class QPoint3f ;

/****************************************************************************/
/*!
  \class QBSplineCurve QCurve.h
  \brief The class QBSplineCurve provides ...

  More ...
*/
/****************************************************************************/
class QBSplineCurve : public QParametricCurve
{
    Q_OBJECT
	
public :
     QBSplineCurve(void) ;
     QBSplineCurve(QBoundingBox * boundingbox) ;
    ~QBSplineCurve(void) {}

    void draw(void) ;
    void drawWithNames(void) ;

public slots:
    QPoint3f * eval(float u) ;
    float umin(void) { return knots[0] ; }
    float umax(void) { return knots[number+order-1] ; }

public slots:
    int getDimension(void);
    int getNumber(void);
    int getOrder(void);

    double * getKnots(void);

    void compute(void) ;
    void insert(int pos, QPoint3f * p) ;
    void push_point(QPoint3f * p) ;
    void push_front(QPoint3f * p) ;
    void computeKnots(void) ;
	
public:
    int order ; 		//!< Order of curve
    int number ;		//!< Number of vertices
    double * knots ;		//!< Pointer to the knot vector
    
    // Use points instead
    // double * ecoef ;		//!< Pointer to the array containing non rational vertices (in.idim)
    // *********************			       
    
    double * rcoef ;		//!< Pointer to the array of rational vertices and weights (in.(idim+1))

    int ikind ;			//!< Type of the curve
                                //!< 1 : Polynomial BSpline curve
                                //!< 2 : Rational BSpline curve (nurbs)
                                //!< 3 : Polynomial Bezier curve
                                //!< 4 : Rational Bezier curve
    int idim ;			//!< Dimension of the space in which the curve lies

    int icopy ;			//!< Indicates wether the arrays of the curve are allocated and copied or
                                //!< referenced by creation of the curve
                                //!< O : Pointer set to input arrays. The arrays are not delete by Sisl freeCurve()
                                //!< 1 : Array allocated and copied. The arrays are delete by Sisl freeCurve()
                                //!< 2 : Pointer set to input arrays, but are to be treated as copied. The arrays are
                                //!< deleted by Sisl freeCurve()

    int cuopen ;	        //!< Open/Closed/Periodic flags.
                                //!< -1 : Closed curve with periodic (cyclic) parameterization and overlapping end 
                                //!< vertices
                                //!< 0 : Closed curve with k-tuple end knots and coinciding start/end vertices
                                //!< 1 : Open curve (default)
    
    QList<QPoint3f *> points ;
	
private:
    static int id_ ;
} ;

} ; // namespace QAxel ;

# endif // QBSPLINECURVE_H
