/*****************************************************************************
 * Axel
 *****************************************************************************
 * QBoundingBox
 * 2006-01-18
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# ifndef QBOUNDINGBOX_H
# define QBOUNDINGBOX_H

# include <QAxel/QAxelObject.h>

namespace QAxel {

class QPoint3f ;

/****************************************************************************/
/*!
  \class QBoundingBox QBoundingBox.h
  \brief The class QBoundingBox provides ...

  More ...
*/
/****************************************************************************/
class QBoundingBox : public QAxelObject 
{
    Q_OBJECT
			
public:
    QBoundingBox(void) ;
    QBoundingBox(float xmin, float xmax, float ymin=0.0, float ymax=0.0, float zmin=0.0, float zmax=0.0) ;
	
    float xmin(void) { return frame()->position().x + xmin_ ; }
    float xmax(void) { return frame()->position().x + xmax_ ; }
    float ymin(void) { return frame()->position().y + ymin_ ; }
    float ymax(void) { return frame()->position().y + ymax_ ; }
    float zmin(void) { return frame()->position().z + zmin_ ; }
    float zmax(void) { return frame()->position().z + zmax_ ; }
	
    void setXMin(float x) { xmin_ = x ; }
    void setXMax(float x) { xmax_ = x ; }
    void setYMin(float y) { ymin_ = y ; }
    void setYMax(float y) { ymax_ = y ; }
    void setZMin(float z) { zmin_ = z ; }
    void setZMax(float z) { zmax_ = z ; }
	
    float left   (void) { return frame()->position().x + xmin_ ; }
    float right  (void) { return frame()->position().x + xmax_ ; }
    float bottom (void) { return frame()->position().y + ymin_ ; }
    float top    (void) { return frame()->position().y + ymax_ ; }
    float back   (void) { return frame()->position().z + zmin_ ; }
    float front  (void) { return frame()->position().z + zmax_ ; }
	
    void setLeft   (float x) { xmin_ = x ; }
    void setRight  (float x) { xmax_ = x ; }
    void setBottom (float y) { ymin_ = y ; }
    void setTop    (float y) { ymax_ = y ; }
    void setBack   (float z) { zmin_ = z ; }
    void setFront  (float z) { zmax_ = z ; }
	
    float width  (void) { return xmax_-xmin_ ; }
    float height (void) { return ymax_-ymin_ ; }
    float depth  (void) { return zmax_-zmin_ ; }

    float middleX(void) { return (xmax_+xmin_)/2 ; }
    float middleY(void) { return (ymax_+ymin_)/2 ; }

    void draw(void) ;
    void drawWithNames(void) ;
    QString toString(void) ; 
    QString toDomain2D(void) ;
    QString toDomain3D(void) ;

    float abs(float a) ;

    float size(void) ;
    QPoint3f * center(void) ;

    bool isNull(void) ;
    bool isEmpty(void) ;
    bool isValid(void) ;

    bool contains(float x, float y, bool strict) ;
    bool contains(float x, float y, float z, bool strict) ;
    bool contains(QPoint3f * p, bool strict) ;
    bool contains(QBoundingBox * b, bool strict) ;
    bool intersects(QBoundingBox * b, bool strict = true) ;
    bool unites(QBoundingBox * b, bool strict = true) ;

    QList<QPoint3f *> corners(void) ;

    QBoundingBox * normalized(void) ;
    QBoundingBox * intersect(QBoundingBox * b) ;
    QBoundingBox * intersected(QBoundingBox * b) ;
    QBoundingBox * unite(QBoundingBox * b) ;
    QBoundingBox * united(QBoundingBox * b) ;
    
    friend std::ostream& operator <<(std::ostream& stream, QBoundingBox & b) ;
    friend std::ostream& operator <<(std::ostream& stream, QBoundingBox * b) ;
    
public:
    static QBoundingBox * intersect(QBoundingBox * b1, QBoundingBox * b2) ;
    static QBoundingBox * unite    (QBoundingBox * b1, QBoundingBox * b2) ;

    static bool intersects(QBoundingBox * b1, QBoundingBox * b2) ;
    static bool unites    (QBoundingBox * b1, QBoundingBox * b2) ;

protected:	
    static int id_ ;
    float xmin_, xmax_ ;
    float ymin_, ymax_ ;
    float zmin_, zmax_ ;
} ;

} ; // namespace QAxel

# endif
