# include <QAxel/QConstraintDistance.h>

namespace QAxel 
{
GLUquadricObj *QConstraintDistance::quadObj = NULL;
int QConstraintDistance::id_ = 0 ;

QConstraintDistance::QConstraintDistance(QAxelObject *o1, QAxelObject *o2, float value)
{
	_objects.append(o1);
	_objects.append(o2);
	_value=value;
	
	if(!quadObj)
		quadObj = gluNewQuadric();
	
	QString number;
	QString str = "distpp(" +o1->name()+ "," +o2->name()+ "," + "k"+ number.setNum(id_) + ")" ;
	setProperty("type", "ConstraintDistance");
	setProperty("name", str) ;
	id_++ ;
	setObjectColor(new QColor(255,51,112));
}

QConstraintDistance::~QConstraintDistance()
{
}

float QConstraintDistance::getValue() const
{
	return _value;
}

void QConstraintDistance::setValue(float f)
{
	_value=f;
}

void QConstraintDistance::draw()
{
	if(hidden()) return ;
	float offset = 0.05;
	float coneLength = 0.01;
	float coneHeight = 0.05;
	
	Vec v1 = _objects[0]->frame()->position();
	Vec v2 = _objects[1]->frame()->position();
	Vec v =  v2 - v1;
	double length = v.norm();
	
	Vec x=v, y, z;
	Vec front(0,0,1);
	Vec right(1,0,0);
	
	if(x!=front)
		y = front^x;
	else
		y = right^x;
	z = x^y;
	
	x.normalize();
	y.normalize();
	z.normalize();
	
	GLdouble m[4][4];
	m[0][0] = -z.x;
	m[0][1] = -z.y;
	m[0][2] = -z.z;
	m[0][3] = 0;
	m[1][0] = y.x;
	m[1][1] = y.y;
	m[1][2] = y.z;
	m[1][3] = 0;
	m[2][0] = x.x;
	m[2][1] = x.y;
	m[2][2] = x.z;
	m[2][3] = 0;
	m[3][0] = v1.x;
	m[3][1] = v1.y;
	m[3][2] = v1.z;
	m[3][3] = 1;

	frame_->setFromMatrix(m);
	
	glPushMatrix();
		glMultMatrixd(frame_->matrix());
		glColor();
		// cone 1: <|---
		glPushMatrix();
		glTranslatef(0.0,0.0,offset+coneHeight);
			glRotatef(-180,0,1,0);
			gluQuadricDrawStyle(quadObj, GLU_FILL);
			gluQuadricNormals(quadObj, GLU_SMOOTH);
			gluCylinder(quadObj, coneLength, 0.0, coneHeight, 10, 20);
		glPopMatrix();
		
// 		// cone 2: ---|>
		glPushMatrix();
		glTranslatef(0.0,0.0,length-(coneHeight+offset));
			gluQuadricDrawStyle(quadObj, GLU_FILL);
			gluQuadricNormals(quadObj, GLU_SMOOTH);
			gluCylinder(quadObj, coneLength, 0.0, coneHeight, 10, 20);
		glPopMatrix();
		
		glTranslatef(0.0,0.0,offset+coneHeight);
		gluCylinder(quadObj, coneLength/2, coneLength/2, length-2*(coneHeight+offset), 10, 20);
	glPopMatrix();

}

void QConstraintDistance::drawWithNames()
{
// 	No manipulation for the moment
	glPushName((GLuint)this);
	draw();
	glPopName();
}

float QConstraintDistance::computeDistance()
{
	Vec v1 = _objects[0]->frame()->position();
	Vec v2 = _objects[1]->frame()->position();
	Vec v =  v2 - v1;
	return v.norm();
}

} // namespace QAxel
