/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QCube
 * 2006-11-06
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# include <QAxel/QCube.h>

namespace QAxel {

int QCube::id_ = 0 ;

QCube::QCube(float x, float y, float z, float s) 
{
	QString str;
	setProperty("type", "Cube");
	setProperty("name", "cube_"+str.setNum(id_)) ;
	id_++ ;

	frame_->setPosition(x,y,z);
	c_size = s ;
	
	setObjectColor(new QColor(55,128,55,0)) ;
}

QCube::~QCube(void) 
{
	
}

void QCube::setx(float x)
{
	Vec v = frame_->position();
	frame_->setPosition(x,v.y,v.z);
}

void QCube::sety(float y)
{
	Vec v = frame_->position();
	frame_->setPosition(v.x,y,v.z);
}

void QCube::setz(float z)
{
	Vec v = frame_->position();
	frame_->setPosition(v.x,v.y,z);
}

void QCube::setSize(float s) 
{
	c_size = s ;
}

void QCube::draw(void)
{	
	if(hidden()) return ;
	glPushMatrix() ;
		glMultMatrixd(frame_->matrix()) ;
		glColor() ;
		glBegin(GL_QUADS);
	    	// Top
		    glNormal3d(0,1,0);
		    glVertex3d(-c_size/2,c_size/2,c_size/2);
		    glVertex3d(c_size/2,c_size/2,c_size/2);
		    glVertex3d(c_size/2,c_size/2,-c_size/2);
		    glVertex3d(-c_size/2,c_size/2,-c_size/2);
		    // Back
		    glNormal3d(0,0,1);
		    glVertex3d(-c_size/2,c_size/2,c_size/2);
		    glVertex3d(-c_size/2,-c_size/2,c_size/2);
		    glVertex3d(c_size/2,-c_size/2,c_size/2);
		    glVertex3d(c_size/2,c_size/2,c_size/2);
		    // Front
		    glNormal3d(0,0,-1);
		    glVertex3d(-c_size/2,c_size/2,-c_size/2);
		    glVertex3d(c_size/2,c_size/2,-c_size/2);
		    glVertex3d(c_size/2,-c_size/2,-c_size/2);
		    glVertex3d(-c_size/2,-c_size/2,-c_size/2);
		    // Left
		    glNormal3d(1,0,0);
		    glVertex3d(c_size/2,c_size/2,-c_size/2);
		    glVertex3d(c_size/2,c_size/2,c_size/2);
		    glVertex3d(c_size/2,-c_size/2,c_size/2);
		    glVertex3d(c_size/2,-c_size/2,-c_size/2);
		    // Right
		    glNormal3d(-1,0,0);
		    glVertex3d(-c_size/2,c_size/2,c_size/2);
		    glVertex3d(-c_size/2,c_size/2,-c_size/2);
		    glVertex3d(-c_size/2,-c_size/2,0-c_size/2);
		    glVertex3d(-c_size/2,-c_size/2,c_size/2);
		    // Bottom
		    glNormal3d(0,-1,0);
		    glVertex3d(-c_size/2,-c_size/2,c_size/2);
		    glVertex3d(-c_size/2,-c_size/2,-c_size/2);
		    glVertex3d(c_size/2,-c_size/2,-c_size/2);
		    glVertex3d(c_size/2,-c_size/2,c_size/2);
		glEnd();
	glPopMatrix() ;
}

void QCube::drawWithNames(void)
{
	glPushName((GLuint)this) ;
	draw();
	glPopName() ;
}

} ;
