/*****************************************************************************
 * Axel
 *****************************************************************************
 * QCylinder
 * 2006-09-14
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# include <QAxel/QPoint3f.h>
# include <QAxel/QCylinder.h>

namespace QAxel {

int QCylinder::id_ = 0 ;
GLUquadricObj * QCylinder::quadObj = NULL ;

QCylinder::QCylinder(QPoint3f * from, QPoint3f * to, float radius) 
{
  	QString str ;
	setProperty("type", "Cylinder") ;
	setProperty("name", "cylinder_"+str.setNum(id_)) ;
	id_++ ;
	setObjectColor(new QColor(255,0,0)) ;
	
	this->from_ = from ;
	this->to_ = to ;
	this->radius_ = radius ;
	
	if(!quadObj) quadObj = gluNewQuadric() ;
}

QCylinder::~QCylinder(void) {}

QPoint3f* QCylinder::from(void)
{
    return from_;
}

QPoint3f* QCylinder::to(void)
{
    return to_;
}

float QCylinder::s(void)
{
    return radius_;
}

void QCylinder::draw(void) 
{     
	if(hidden()) return ;
	glPushMatrix() ;
		glMultMatrixd(frame_->matrix()) ;
		glColor() ;
		Vec v1 = from_->frame()->position();
		Vec v2 = to_->frame()->position();
		drawCylinder(v1, v2, radius_, 32) ;
	glPopMatrix() ;
}

void QCylinder::drawCylinder(const Vec& from, const Vec& to, float radius, int nbSubdivisions)
{
  glPushMatrix() ;
  glTranslatef(from[0],from[1],from[2]) ;
  glMultMatrixd(Quaternion(Vec(0,0,1), to-from).matrix()) ;
  drawCylinder((to-from).norm(), radius, nbSubdivisions) ;
  glPopMatrix() ;
}

void QCylinder::drawCylinder(float length, float radius, int nbSubdivisions)
{
  static GLUquadric* quadric = gluNewQuadric();

  if (radius < 0.0) radius = 0.05 * length;

  gluCylinder(quadric, radius, radius, length, nbSubdivisions, 1);
  gluSphere(quadric, radius, 16, 16) ;
  glTranslatef(0.0, 0.0, length) ;
  gluSphere(quadric, radius, 16, 16) ;
  
}

void QCylinder::drawWithNames(void) 
{	  
	glPushName((GLuint)this) ;
	draw();
	glPopName() ;
}

} ; // namespace QAxel
