/*****************************************************************************
 * Axel
 *****************************************************************************
 * QImplicitCurve4D.h
 * 2007-05-22
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# ifndef QIMPLICITCURVE4D_H
# define QIMPLICITCURVE4D_H

# include <QAxel/QCurve.h>
# include <QData/QBoundingBox4D.h>

namespace QAxel {

class QPoint3f ;
class QMesh ;
/****************************************************************************/
/*!
  \class QImpliciteCurve4D QCurve.h
  \brief The class QImplicitCurve4D is an abstract base class for implicit curves.

  More ...
*/
/****************************************************************************/
class QImplicitCurve4D : public QCurve
{
    Q_OBJECT

public:
    QImplicitCurve4D(QString f, QString g, QString h, QData::QBoundingBox4D * bb = new QData::QBoundingBox4D(), char * var = "s t u v") ;
    QImplicitCurve4D(QString f, QString g, QString h, 
		     double smin=0, double smax=1, 
		     double tmin=0, double tmax=1, 
		     double umin=0, double umax=1, 
		     double vmin=0, double vmax=1,
		     char * var = "s t u v");
    
    void draw(void) {}
    void drawWithNames(void) {}

    void compute(void) ;

public slots:
	QString f(void) { return c_f ; }
	QString g(void) { return c_g ; }
	QString h(void) { return c_h ; }
	char * eqf(void){ return c_f.toLatin1().data() ; }
	char * eqg(void){ return c_g.toLatin1().data() ; }
	char * eqh(void){ return c_h.toLatin1().data() ; }
	char * var(void){ return c_var ; }
	
	float smin(void) { return c_smin ; }
	float smax(void) { return c_smax ; }
	float tmin(void) { return c_tmin ; }
	float tmax(void) { return c_tmax ; }
	float umin(void) { return c_umin ; }
	float umax(void) { return c_umax ; }
	float vmin(void) { return c_vmin ; }
	float vmax(void) { return c_vmax ; }
	
	QData::QBoundingBox4D * bb(void) { return new QData::QBoundingBox4D(c_smin, c_smax, c_tmin, c_tmax, c_umin, c_umax, c_vmin, c_vmax) ; }
	
//	bool roots(float xmin, float xmax, float ymin, float ymax) ;
//	int nbRoots(float xmin, float xmax, float ymin, float ymax) ;

private:
	QString c_f ;
	QString c_g ;
	QString c_h ;
	char * c_var ;
	
	double c_smin ;
	double c_smax ;
	double c_tmin ;
	double c_tmax ;
	double c_umin ;
	double c_umax ;
	double c_vmin ;
	double c_vmax ;
} ;

} ;

# endif
