/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QMesh
 * 2006-03-25
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :         
 *
 ****************************************************************************/

# ifndef QMESH_H
# define QMESH_H

# include <QList>
# include <QAxel/QAxelObject.h>
# include <QGLViewer/qglviewer.h>

using namespace qglviewer ;
using namespace std ;

namespace QAxel {

class QDisplayList ;
class QVertex ;
class QEdge ;
class QFace ;

/****************************************************************************/
/*!
  \class QMesh QMesh.h
  \brief The class QMesh provides ...

  More ...
*/
/****************************************************************************/
class QMesh : public QAxelObject 
{
    Q_OBJECT
    Q_PROPERTY(MeshType meshType READ meshType WRITE setMeshType)
    Q_PROPERTY(EditionMode editionMode READ editionMode WRITE setEditionMode)
    Q_ENUMS(MeshType)
    Q_ENUMS(EditionMode)
	
public:
    QMesh(void) ;          
    QMesh(QMesh * m) ;
    QMesh(int nvertices, int nfaces) ;
	
    void draw(void) ;
    void drawWithNames(void) ;
    void drawWithoutColor(void) ;
    void clear(void) ;
    
    enum MeshType { Curve, Surface } ;
    enum EditionMode { Vertex, Edge, Face } ;
					    
public slots:
    void push_vertex(QVertex * v) { vertices_.push_back(v) ; }
    void push_edge(QEdge * e) { edges_.push_back(e) ; }
    void push_face(QFace * f) { faces_.push_back(f) ; }    
    
    void setNumberOfVertices(int n) ;
    void setNumberOfEdges(int n) ;
    void setNumberOfFaces(int n) ;
    
    QList<QVertex *>& vertices(void) ;
    QList<QEdge *>& edges(void) ;
    QList<QFace *>& faces(void) ;
        
    double * getVertex(int index) ;
    void setVertexNormal(int index, double nx, double ny, double nz) ;
    void computeNormals(void) ;
    
    void setMeshType(MeshType meshType) { meshType_ = meshType ; }
    void setEditionMode(EditionMode editionMode) { editionMode_ = editionMode ; }                          
    MeshType meshType() const { return meshType_ ; }
    EditionMode editionMode() const { return editionMode_ ; }
    
    void translate(float x, float y, float z) ;
    QAxelObject * toObject(void) { 
	return (QAxelObject *)this ;
    }
	
public:
    int nindexes_ ;
    int nvertices_ ;
    GLuint * glindexes_ ;
    float * glvertices_ ;
    float * glnormals_ ;
    
    QList<QVertex *> vertices_ ;  
    QList<QEdge *> edges_ ;
    QList<QFace *> faces_ ;
    
    friend class QVertex ;
    friend class QEdge ;
    friend class QFace ;

public:
    bool drawEdges ;
    QDisplayList * edgesList ;
    
private :
    static int id_ ;
    MeshType meshType_ ;
    EditionMode editionMode_ ;
} ;

} ; // namespace QAxel

# endif
