/*****************************************************************************
 * Axel
 *****************************************************************************
 * QPoint3f
 * 2006-03-24
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 * Point's coordinates are the coordinates of its frame ! This way they are
 * always update with no problem ....	
 ****************************************************************************/

# ifndef QPOINT3F_H
# define QPOINT3F_H

# include <QList>

# include <QAxel/QAxelObject.h>

namespace QAxel {

/****************************************************************************/
/*!
  \class QPoint3f QPoint3f.h
  \brief The class QPoint3f provides ...

  More ...
*/
/****************************************************************************/
class QPoint3f : public QAxelObject 
{
    Q_OBJECT

public:
    QPoint3f(float x, float y, float z = 0.0) ;
    ~QPoint3f(void) ;
    
    void draw(void) ; 
    void drawWithNames(void) ;

    QString toString(void) ; 
    
    friend std::ostream& operator <<(std::ostream& stream, QPoint3f & r) ;
    friend std::ostream& operator <<(std::ostream& stream, QPoint3f * r) ;
    
    bool equals(QPoint3f * p) ;
    bool equals(QPoint3f * p, double tolerance) ;

public slots:
    float x(void);
    float y(void);
    float z(void);
    void setx(float x);
    void sety(float y);
    void setz(float z);
    void setPosition(float x, float y, float z) ;
    void setPosition(QPoint3f *) ;
    void translate(void) ;
    
    float size(void) ;
    void setSize(float size) ;

/* The following functions are provided because QSA 1.2.1 does handle super methods. */
    void select(void) { ((QAxelObject *)this)->select() ; }
    QAxelObject * toObject(void) { return (QAxelObject *)this ; }
    
private :
    float p_size ;
    static int id_ ;
} ;

} ; // namespace QAxel

# endif
