/*****************************************************************************
 * Axel
 *****************************************************************************
 * QPointSet
 * 2007-05-16
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# include <QAxel/QPointSet.h>


namespace QAxel {

int QPointSet::id_ = 0 ;

QPointSet::QPointSet(int size, bool colors) : m_size(size), m_colors(colors), nindexes(0), nvertices(0), ncolors(0)
{
  if(m_colors) {
    glindexes  = new int[m_size] ;
    glvertices = new float[m_size*3] ;
    glcolors   = new float[m_size*3] ;
  } else {
    glindexes  = new int[m_size] ;
    glvertices = new float[m_size*3] ;
  }
  
  QString str ;
  setProperty("type", "PointSet") ;
  setProperty("name", "pset_"+str.setNum(id_)) ;
  setObjectName("pset_"+str.setNum(id_)) ;
  id_++ ;
  
  setObjectColor(new QColor(Qt::white)) ;
}

int QPointSet::size(void)
{
    return m_size;
}

float* QPointSet::vertices(void)
{
    return glvertices;
}

float* QPointSet::colors(void)
{
    return glcolors;
}

  
void QPointSet::draw(void)
{
  // if(nindexes != m_size) return ;
  glColor() ;
  glEnableClientState(GL_VERTEX_ARRAY) ;
  if(m_colors) glEnableClientState(GL_COLOR_ARRAY) ;
  glVertexPointer(3, GL_FLOAT, 0, glvertices) ;
  if(m_colors) glColorPointer(3, GL_FLOAT, 0, glcolors) ;
  glDrawArrays(GL_POINTS, 0, nindexes) ;
  glDisableClientState(GL_VERTEX_ARRAY) ;
  glDisableClientState(GL_COLOR_ARRAY) ;
}

void QPointSet::drawWithNames(void)
{
  glPushName((GLuint)this) ;
  draw();
  glPopName() ;
}
  
void QPointSet::push(float x, float y, float z)
{
  glvertices[3*nvertices+0] = x ;
  glvertices[3*nvertices+1] = y ;
  glvertices[3*nvertices+2] = z ;
  nvertices++ ;
  
  glindexes[nindexes] = nindexes ;
  nindexes++ ;
}
  
void QPointSet::push(float x, float y, float z, int r, int g, int b)
{
  glvertices[3*nvertices+0] = x ;
  glvertices[3*nvertices+1] = y ;
  glvertices[3*nvertices+2] = z ;
  nvertices++ ;
  
  glcolors[3*ncolors+0] = (float)r/255.0 ;
  glcolors[3*ncolors+1] = (float)g/255.0 ;
  glcolors[3*ncolors+2] = (float)b/255.0 ;
  ncolors++ ;
  
  glindexes[nindexes] = nindexes ;
  nindexes++ ;
}

void QPointSet::push(float x, float y, float z, float r, float g, float b)
{
  glvertices[3*nvertices+0] = x ;
  glvertices[3*nvertices+1] = y ;
  glvertices[3*nvertices+2] = z ;
  nvertices++ ;
  
  glcolors[3*ncolors+0] = r ;
  glcolors[3*ncolors+1] = g ;
  glcolors[3*ncolors+2] = b ;
  ncolors++ ;
  
  glindexes[nindexes] = nindexes ;
  nindexes++ ;
}

void QPointSet::translate(float x, float y, float z)
{
  for(int i=0 ; i < nvertices ; i++) {
    glvertices[3*i+0]+=x ;
    glvertices[3*i+1]+=y ;
    glvertices[3*i+2]+=z ;
  }
}

} ; // namespace QAxel
