/*****************************************************************************
 * Axel
 *****************************************************************************
 * QPointSet
 * 2007-05-16
 * Ashish Reddy
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# ifndef QPOINTSET_H
# define QPOINTSET_H

# include <QList>

# include <QAxel/QAxelObject.h>
# include <QAxel/QPoint3f.h>

namespace QAxel {

/************************************
 * @class QPointSet
 * @brief A fastly drawable point set
 * Since QObject cannot inherit template
 * classe, we make composition instead.
 ************************************/  
class QPointSet : public QAxelObject
{
    Q_OBJECT
    Q_OVERRIDE(QString objectName SCRIPTABLE true)
	
public:
    QPointSet(int size, bool colors = false) ;
    
    int size(void);
    float* vertices(void);
    float* colors(void);
    void draw(void) ;
    void drawWithNames(void) ;
    
    void push(float x, float y, float z) ;
    void push(float x, float y, float z, int r  , int g  , int b) ;
    void push(float x, float y, float z, float r, float g, float b) ;
								    
public slots:
    void translate(float x, float y, float z) ;
    QAxelObject * toObject(void) { return (QAxelObject *)this ; }
    
private:
    int m_size ;
    bool m_colors ;
    
private:
    int nindexes ;
    int nvertices ;
    int ncolors ;
    int * glindexes ;
    float * glvertices ;
    float * glcolors ;
    
private:
    static int id_ ;
} ;

} ; // namespace QAxel

# endif
