/****************************************************************************
 * A x e l
 *****************************************************************************
 * QRationalSurface
 * 2007-05-22
 * Julien Wintz
 * Ashish Reddy
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# include <QMessageBox>

# include <QAxel/QBoundingBox.h>
# include <QAxel/QPoint3f.h>
# include <QAxel/QCurve.h>
# include <QAxel/QDisplayList.h>
# include <QAxel/PluginManager.h>

# include <QSisl/QSislInterface.h>
# include <QCore/QCoreInterface.h>

namespace QAxel {
        
QRationalCurve::QRationalCurve(QString px, QString py, QString pz, QString pw, double tmin, double tmax, QBoundingBox * boundingbox) : QParametricCurve(boundingbox)
{
    QString str ;
    setProperty("type", "RationalCurve") ;
    setProperty("name", "rcurve_"+str.setNum(id_)) ;
    
    this->px_ = px ;
    this->py_ = py ;
    this->pz_ = pz ;
    this->pw_ = pw ;
    
    this->tmin_ = tmin ;
    this->tmax_ = tmax ;
    
    this->compute() ;
}

void QRationalCurve::compute(void) 
{
    QCoreInterface * iCore = dynamic_cast<QCoreInterface *>(PluginManager::instance()->plugin("Core")) ;
    if(iCore) {
        iCore->create(this) ;
    } else {
        QMessageBox::information(0, "Axel","QRationalCurve Computation : QCore plugin is not loaded","&Ok",0) ;
    }
}

QPoint3f * QRationalCurve::eval(float u)
{
    QCoreInterface * iCore = dynamic_cast<QCoreInterface *>(PluginManager::instance()->plugin("Core")) ;
    if(iCore) return iCore->eval(this, u) ;
    else      return NULL ;
}

void QRationalCurve::draw(void) 
{
    if(hidden()) return ;
    
    QCoreInterface * iCore = dynamic_cast<QCoreInterface *>(PluginManager::instance()->plugin("Core")) ;
    if(iCore) {
	glPushMatrix() ;
	glMultMatrixd(frame_->matrix()) ;
	glColor() ;
	iCore->draw(this) ;
	glPopMatrix() ;
    } else {
	QMessageBox::information(0, "Axel","QRationalCurve Computation : QCore plugin is not loaded","&Ok",0) ;
    }
}

void QRationalCurve::drawWithNames(void) 
{
    glPushName((GLuint)this);
    draw();
    glPopName();
}

int QRationalCurve::id_ = 0 ;

} ; // namespace QAxel

