/****************************************************************************
 * A x e l
 *****************************************************************************
 * QRationalSurface.cpp
 * 2007-05-22
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# include <QMessageBox>

# include <QAxel/QPoint3f.h>
# include <QAxel/QBoundingBox.h>
# include <QAxel/QSingularity.h>
# include <QAxel/QSurface.h>
# include <QAxel/QMesh.h>
# include <QAxel/PluginManager.h>

# include <QSisl/QSislInterface.h>
# include <QSynaps/QSynapsInterface.h>
# include <QCore/QCoreInterface.h>

using namespace std ;

namespace QAxel {

/*****************************************************************************
 * QRationalSurface
 *****************************************************************************/

int QRationalSurface::id_ = 0 ;

QRationalSurface::QRationalSurface(QString px, QString py, QString pz, QString pw,
				   double umin, double umax,
				   double vmin, double vmax,
				   double grid_u, double grid_v,
				   QBoundingBox * boundingbox) : QParametricSurface(boundingbox)
{
    // Selection matter 
    QString str ;
    setProperty("type", "RationalSurface") ;
    setProperty("name", "RationalSurface"+str.setNum(id_)) ;
    id_++ ;
    
    this->grid_u = grid_u ;
    this->grid_v = grid_v ;
    
    umin_ = umin ; umax_ = umax ;
    vmin_ = vmin ; vmax_ = vmax ;
    
    this->px_ = px ;
    this->py_ = py ;
    this->pz_ = pz ;
    this->pw_ = pw ;
    
    this->compute() ;
}

void QRationalSurface::compute(void) 
{
    QCoreInterface * iCore = dynamic_cast<QCoreInterface *>(PluginManager::instance()->plugin("Core")) ;
    if(iCore) iCore->create(this) ;
    else QMessageBox::information(0, "Axel","QRationalSurface computation : QCore plugin is not loaded","&Ok",0) ;
}

void QRationalSurface::draw(void) 
{
    QCoreInterface * iCore = dynamic_cast<QCoreInterface *>(PluginManager::instance()->plugin("Core")) ;
    if(iCore) {
	if(hidden()) return ;
	glPushMatrix() ;
	glMultMatrixd(frame_->matrix()) ;
	glColor() ;
	iCore->draw(this) ;
	glPopMatrix() ;
    } else {
	QMessageBox::information(0, "Axel","QRationalCurve Computation : QCore plugin is not loaded","&Ok",0) ;
    }
}

void QRationalSurface::drawWithNames(void) 
{
    glPushName((GLuint)this);
    draw();
    glPopName();
}

QPoint3f * QRationalSurface::eval(float u, float v) 
{
    QCoreInterface * iCore = dynamic_cast<QCoreInterface *>(PluginManager::instance()->plugin("Core")) ;
    if(iCore) return iCore->eval(this, u, v) ;
    else      return NULL ;
}

} ;
