/*****************************************************************************
 * A x e l
 *****************************************************************************
 * Tool Manager
 * 2007-06-07
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2007 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :         
 *
 ****************************************************************************/

# include <QAxel/ToolManager.h>

ToolManager * ToolManager::m_instance = NULL ;

ToolManager * ToolManager::instance(void)
{
    if (!m_instance) {
	m_instance = new ToolManager ;
    }
    return m_instance;
}

QPoint3f * ToolManager::intersection(LineSegment_t s1, LineSegment_t s2)
{
    float Ax = s1.first->x() ;  float Ay = s1.first->y() ;
    float Bx = s1.second->x() ; float By = s1.second->y() ;
    float Cx = s2.first->x() ;  float Cy = s2.first->y() ;
    float Dx = s2.second->x() ; float Dy = s2.second->y() ;
    float X ; float Y ;

    float  distAB, theCos, theSin, newX, ABpos ;

    //  Fail if either line is undefined.
    if (Ax==Bx && Ay==By || Cx==Dx && Cy==Dy) return false ;

    //  (1) Translate the system so that point A is on the origin.
    Bx-=Ax; By-=Ay;
    Cx-=Ax; Cy-=Ay;
    Dx-=Ax; Dy-=Ay;

    //  Discover the length of segment A-B.
    distAB=sqrt(Bx*Bx+By*By);

    //  (2) Rotate the system so that point B is on the positive X axis.
    theCos=Bx/distAB;
    theSin=By/distAB;
    newX=Cx*theCos+Cy*theSin;
    Cy  =Cy*theCos-Cx*theSin; Cx=newX;
    newX=Dx*theCos+Dy*theSin;
    Dy  =Dy*theCos-Dx*theSin; Dx=newX;

    //  Fail if the lines are parallel.
    if (Cy==Dy) return NULL ;

    //  (3) Discover the position of the intersection point along line A-B.
    ABpos=Dx+(Cx-Dx)*Dy/(Dy-Cy);

    //  (4) Apply the discovered position to line A-B in the original coordinate system.
    X=Ax+ABpos*theCos;
    Y=Ay+ABpos*theSin;

    //  Success.
    return new QPoint3f(X, Y, 0.0) ; 
}

