/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QCorePlugin
 * 2006-11-06
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :         
 ****************************************************************************/

# include <QtGui>

# include <QAxel/QAxelObject.h>
# include <QAxel/QShape.h>
# include <QAxel/QPoint3f.h>
# include <QAxel/QVertex.h>
# include <QAxel/QEdge.h>
# include <QAxel/QFace.h>
# include <QAxel/QMesh.h>

# include <QCore/QCorePlugin.h>

# include <synaps/shape/curves/SISLBSplineCurve.h>
# include <synaps/shape/curves/RationalCurve.h>
# include <synaps/shape/surfaces/SISLBSplineSurface.h>
# include <synaps/shape/surfaces/RationalSurface.h>
# include <synaps/shape/wrappers/WShapeFactory.h>

# include <synaps/shape/interfaces/IGLGood.h>
# include <synaps/shape/interfaces/IRationalCurve.h>
# include <synaps/shape/interfaces/IRationalSurface.h>
# include <synaps/shape/interfaces/IBSplineCurve.h>
# include <synaps/shape/interfaces/IBSplineSurface.h>
# include <synaps/shape/interfaces/IParametricSurface.h>

# include <QGLViewer/qglviewer.h>

typedef type::gentlist< 
    // MPolImplicitSurface,
    RationalCurve<double,3>, 
    RationalSurface<double>,
    SISLBSplineCurve,
    SISL::BSPS
>::T TypeList ;

using namespace std ;
using namespace qglviewer ;

void gldraw(IGLGood * glg)
{
    if(!glg->vertices) return ;
    
    glEnableClientState(GL_VERTEX_ARRAY) ;
    glVertexPointer(3, GL_DOUBLE, 0, glg->vertices) ;
    
    if(glg->normals) { 
	glEnableClientState(GL_NORMAL_ARRAY) ;   
	glNormalPointer(GL_DOUBLE, 0, glg->normals) ; 
    }
    
    switch ( glg->type ) {
    case IGLGood::E_VERTEX:
	glDisable(GL_LIGHTING) ;
	glDrawElements(GL_POINTS, glg->nbi, GL_UNSIGNED_INT, glg->indexes) ;
	glEnable(GL_LIGHTING) ;
	break;
	
    case IGLGood::E_LINE:
	glDisable(GL_LIGHTING) ;
	glDrawElements(GL_LINES, glg->nbi, GL_UNSIGNED_INT, glg->indexes) ;
	glEnable(GL_LIGHTING) ;
	break;
	
    case IGLGood::E_TRIANGLE:
	glDrawElements(GL_TRIANGLES, glg->nbi, GL_UNSIGNED_INT, glg->indexes) ;
	break;
	
    case IGLGood::E_QUAD:
	glDrawElements(GL_QUADS, glg->nbi, GL_UNSIGNED_INT, glg->indexes) ;
	break;
    }
    
    glDisableClientState(GL_VERTEX_ARRAY) ;
    
    if(glg->normals) glDisableClientState(GL_NORMAL_ARRAY) ;
}

QString QCorePlugin::name() const
{
    return "Core" ;
}

QStringList QCorePlugin::features(void) const
{
    return QStringList() << "" ;
}

void QCorePlugin::init(MainWindow * window)
{
    factory = new WShapeFactory<TypeList> ;
}

// void QCorePlugin::create(QImplicitCurve * c)
// {
// }
// 
// void QCorePlugin::create(QImplicitSurface * s)
// {
// 	IShape * shape = factory->interface( "SYNAPS::IImplicitSurface", 0 ) ;
// 	IImplicitSurface * is = dynamic_cast<IImplicitSurface *>(shape) ;
// 	map.insert(c, shape) ;
// }


void QCorePlugin::create(QRationalCurve * c)
{
    IShape * shape = factory->interface( "SYNAPS::IRationalCurve", 0 ) ;
    IRationalCurve * rc = dynamic_cast<IRationalCurve *>(shape) ;
    rc->setEquations(
        c->pw().toAscii().constData(),
        c->px().toAscii().constData(),
        c->py().toAscii().constData(),
        c->pz().toAscii().constData()
	) ;
    rc->setRange(c->tmin(), c->tmax()) ;
    map.insert(c, shape) ;
}

void QCorePlugin::create(QRationalSurface * s)
{
    IShape * shape = factory->interface( "SYNAPS::IRationalSurface", 0 ) ;
    IRationalSurface * rs = dynamic_cast<IRationalSurface *>(shape) ;
    rs->setEquations(
        s->pw().toAscii().constData(),
        s->px().toAscii().constData(),
        s->py().toAscii().constData(),
        s->pz().toAscii().constData(),
		"s t"
    ) ;
    rs->setRange(s->umin(), s->umax(), s->vmin(), s->vmax()) ;
    IParametricSurface * ps = dynamic_cast<IParametricSurface *>(shape) ;
    ps->setGrid(s->gridU(),s->gridV()) ;
    //ps->setGrid(20,20) ;
    map.insert(s, shape) ;
}

void QCorePlugin::create(QBSplineCurve * c)
{
    IShape * shape = factory->interface( "SYNAPS::IBSplineCurve", 0 ) ;
    IBSplineCurve * bc = dynamic_cast<IBSplineCurve *>(shape) ;
    double points[3*c->points.size()] ;
    for(int i = 0 ; i<c->points.size() ; i++) {
	points[3*i+0] = c->points[i]->x() ;
	points[3*i+1] = c->points[i]->y() ;
	points[3*i+2] = c->points[i]->z() ;
    }
    bc->setDefinition(c->number, c->order, c->knots, points) ;
    map.insert(c, shape) ;
}

void QCorePlugin::create(QBSplineSurface * s)
{
    IShape * shape = factory->interface( "SYNAPS::IBSplineSurface", 0 ) ;
    IBSplineSurface * bs = dynamic_cast<IBSplineSurface *>(shape) ;
    double points[s->points.size()*3] ;
    for(int i = 0 ; i<s->points.size() ; i++) {
	points[3*i+0] = s->points[i]->x() ;
	points[3*i+1] = s->points[i]->y() ;
	points[3*i+2] = s->points[i]->z() ;
    }
    bs->setDefinition(s->number1, s->number2,
		      s->order1, s->order2,
		      s->knots1, s->knots2,
		      points 
	) ;
    map.insert(s, shape) ;
}

QPoint3f * QCorePlugin::eval(QAxel::QRationalCurve * c, float u)
{
    IShape * shape = map.value((QAxelObject *)c) ;
    IRationalCurve * rc = dynamic_cast<IRationalCurve *>(shape) ;
    double p[3] ; rc->eval(p, u) ;
    return new QPoint3f(p[0], p[1], p[2]) ;
}

QPoint3f * QCorePlugin::eval(QAxel::QRationalSurface * s, float u, float v)
{
    IShape * shape = map.value((QAxelObject *)s) ;
    IRationalSurface * rs = dynamic_cast<IRationalSurface *>(shape) ;
    double p[3] ; rs->eval(p, u, v) ;
    return new QPoint3f(p[0], p[1], p[2]) ;
}

QPoint3f * QCorePlugin::eval(QAxel::QBSplineCurve * c, float u)
{
    IShape * shape = map.value((QAxelObject *)c) ;
    IBSplineCurve * bc = dynamic_cast<IBSplineCurve *>(shape) ;
    double p[3] ; bc->eval(p, u) ;
    return new QPoint3f(p[0], p[1], p[2]) ;
}

QPoint3f * QCorePlugin::eval(QAxel::QBSplineSurface * s, float u, float v)
{
    IShape * shape = map.value((QAxelObject *)s) ;
    IBSplineSurface * bs = dynamic_cast<IBSplineSurface *>(shape) ;
    double p[3] ; bs->eval(p, u, v) ;
    return new QPoint3f(p[0], p[1], p[2]) ;
}

QShape * QCorePlugin::intersect(QAxelObject * s1, QAxelObject * s2) 
{
    IParametricSurface * ps1 = dynamic_cast<IParametricSurface *>(map[s1]) ;
    IParametricSurface * ps2 = dynamic_cast<IParametricSurface *>(map[s2]) ;
    IShape * shape = SYNAPS::intersect(ps1, ps2) ;
    QShape * qshape = new QShape ;
    qshape->setRGB(255, 0, 0) ;
    map.insert(qshape, shape) ;
    return qshape ;
}

QShape * QCorePlugin::autointersect(QAxelObject * s) 
{
    IParametricSurface * ps = dynamic_cast<IParametricSurface *>(map[s]) ;
    IShape * shape = SYNAPS::selfintersection(ps) ;
    QShape * qshape = new QShape ;
    qshape->setRGB(255, 0, 0) ;
    map.insert(qshape, shape) ;
    return qshape ;
}

void QCorePlugin::draw(QAxelObject * object) 
{
    gldraw(map[object]->mesh()) ;
}

void QCorePlugin::getGlData(QBSplineSurface * s, int * nbi, int * indices, int * nbv, double * vertices)
{
    IParametricSurface * ps = dynamic_cast<IParametricSurface *>(map[s]) ;
    IGLGood * glg = ps->mesh() ;
    *nbi = glg->nbi ; indices = new int[glg->nbi] ;
    for(int i = 0 ; i < glg->nbi ; i++) *(indices+i) = glg->indexes[i] ;
    *nbv = glg->nbv ; vertices = new double[3*glg->nbv] ;
    for(int i = 0 ; i < glg->nbv ; i++) for(int j = 0 ; j < 3 ; j++) *(vertices+3*i+j) = glg->vertices[3*i+j] ;
}

Q_EXPORT_PLUGIN(QCorePlugin)
