/*****************************************************************************
 * Axel
 *****************************************************************************
 * QCell4D
 * 2007-05-02
 * Stephane CHAU
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * TODO : What if two objects intersect on the border of the cell ?
 ****************************************************************************/

# include <QData/QCell4D.h>

namespace QData {

QCell4D::QCell4D(void) : QBoundingBox4D()
{	
	QString str ;
	setProperty("type", "Cell") ;
	setProperty("name", "cell"+str.setNum(id_)) ;
	id_++ ;	
}

QCell4D::QCell4D(QBoundingBox4D * bb) : QBoundingBox4D(bb->smin(), bb->smax(), bb->tmin(), bb->tmax(), bb->umin(), bb->umax(), bb->vmin(), bb->vmax())
{
	QString str ;
	setProperty("type", "Cell") ;
	setProperty("name", "cell"+str.setNum(id_)) ;
	id_++ ;
}

QCell4D::QCell4D(float smin, float smax, float tmin, float tmax, float umin, float umax, float vmin, float vmax) : QBoundingBox4D(smin, smax, tmin, tmax, umin, umax, vmin, vmax)
{
	QString str ;
	setProperty("type", "Cell") ;
	setProperty("name", "cell"+str.setNum(id_)) ;
	id_++ ;
}

QList<QPoint4f *> QCell4D::allIntersections(void)
{
	QList<QPoint4f *> l;
	l << sMinIntersections() << sMaxIntersections() 
		<< tMinIntersections() << tMaxIntersections() 
		<< uMinIntersections() << uMaxIntersections() 
		<< vMinIntersections() << vMaxIntersections();
	return l;
}        


QList<QCell4D *> QCell4D::subdivide4D(QPoint4f * center)
{
	QList<QCell4D *> l ;
	QCell4D * LLLL = new QCell4D(b_smin, center->s(), b_tmin, center->t(), b_umin, center->u(), b_vmin, center->v()) ; l << LLLL ; // 0 in binary notation => l[0]
	QCell4D * LLLR = new QCell4D(b_smin, center->s(), b_tmin, center->t(), b_umin, center->u(), center->v(), b_vmax) ; l << LLLR ; // 1 in binary notation => l[1]
	QCell4D * LLRL = new QCell4D(b_smin, center->s(), b_tmin, center->t(), center->u(), b_umax, b_vmin, center->v()) ; l << LLRL ; // 2 in binary notation => l[2]
	QCell4D * LLRR = new QCell4D(b_smin, center->s(), b_tmin, center->t(), center->u(), b_umax, center->v(), b_vmax) ; l << LLRR ; // 3 in binary notation => l[3]
	QCell4D * LRLL = new QCell4D(b_smin, center->s(), center->t(), b_tmax, b_umin, center->u(), b_vmin, center->v()) ; l << LRLL ; // 4 in binary notation => l[4]
	QCell4D * LRLR = new QCell4D(b_smin, center->s(), center->t(), b_tmax, b_umin, center->u(), center->v(), b_vmax) ; l << LRLR ; // 5 in binary notation => l[5]
	QCell4D * LRRL = new QCell4D(b_smin, center->s(), center->t(), b_tmax, center->u(), b_umax, b_vmin, center->v()) ; l << LRRL ; // 6 in binary notation => l[6]
	QCell4D * LRRR = new QCell4D(b_smin, center->s(), center->t(), b_tmax, center->u(), b_umax, center->v(), b_vmax) ; l << LRRR ; // 7 in binary notation => l[7]
	QCell4D * RLLL = new QCell4D(center->s(), b_smax, b_tmin, center->t(), b_umin, center->u(), b_vmin, center->v()) ; l << RLLL ; // 8 in binary notation => l[8]
	QCell4D * RLLR = new QCell4D(center->s(), b_smax, b_tmin, center->t(), b_umin, center->u(), center->v(), b_vmax) ; l << RLLR ; // 9 in binary notation => l[9]
	QCell4D * RLRL = new QCell4D(center->s(), b_smax, b_tmin, center->t(), center->u(), b_umax, b_vmin, center->v()) ; l << RLRL ; // 10 in binary notation => l[10]
	QCell4D * RLRR = new QCell4D(center->s(), b_smax, b_tmin, center->t(), center->u(), b_umax, center->v(), b_vmax) ; l << RLRR ; // 11 in binary notation => l[11]
	QCell4D * RRLL = new QCell4D(center->s(), b_smax, center->t(), b_tmax, b_umin, center->u(), b_vmin, center->v()) ; l << RRLL ; // 12 in binary notation => l[12]
	QCell4D * RRLR = new QCell4D(center->s(), b_smax, center->t(), b_tmax, b_umin, center->u(), center->v(), b_vmax) ; l << RRLR ; // 13 in binary notation => l[13]
	QCell4D * RRRL = new QCell4D(center->s(), b_smax, center->t(), b_tmax, center->u(), b_umax, b_vmin, center->v()) ; l << RRRL ; // 14 in binary notation => l[14]
	QCell4D * RRRR = new QCell4D(center->s(), b_smax, center->t(), b_tmax, center->u(), b_umax, center->v(), b_vmax) ; l << RRRR ; // 15 in binary notation => l[15]

	// std::cout << __func__ << " : b_smin = " << b_smin << "; center->s() = " << center->s() << "; b_smax = " << b_smax << std::endl;
	// std::cout << __func__ << " : b_tmin = " << b_tmin << "; center->t() = " << center->t() << "; b_tmax = " << b_tmax << std::endl;
	// std::cout << __func__ << " : b_umin = " << b_umin << "; center->u() = " << center->u() << "; b_umax = " << b_umax << std::endl;
	// std::cout << __func__ << " : b_vmin = " << b_vmin << "; center->v() = " << center->v() << "; b_vmax = " << b_vmax << std::endl;

	int code;// determine which cell we have to manipulate (e.g 0 corresponds to LLLL and 15 corresponds to RRRR).

	code = 0 ; // if p is in smin_intersections, we always have p->s() <= center->s()
	foreach(QPoint4f * p, smin_intersections) {
		// if (p->s() > center->s()) code += 8 ;
		if (p->t() > center->t()) code += 4 ;
		if (p->u() > center->u()) code += 2 ;
		if (p->v() > center->v()) code += 1 ;
		l[code]->pushSMinIntersection(p); code = 0;
	}

	code = 8 ; // if p is in smax_intersections, we always have p->s() > center->s()
	foreach(QPoint4f * p, smax_intersections) {
		// if (p->s() > center->s()) code += 8 ;
		if (p->t() > center->t()) code += 4 ;
		if (p->u() > center->u()) code += 2 ;
		if (p->v() > center->v()) code += 1 ;
		l[code]->pushSMaxIntersection(p); code = 8;
	}

	code = 0 ; //  if p is in tmin_intersections, we always have p->t() <= center->t()
	foreach(QPoint4f * p, tmin_intersections) {
		if (p->s() > center->s()) code += 8 ;
		// if (p->t() > center->t()) code += 4 ;
		if (p->u() > center->u()) code += 2 ;
		if (p->v() > center->v()) code += 1 ;
		l[code]->pushTMinIntersection(p); code = 0;
	}

	code = 4 ; // if p is in tmax_intersections, we always have p->t() > center->t()
	foreach(QPoint4f * p, tmax_intersections) {
		if (p->s() > center->s()) code += 8 ;
		// if (p->t() > center->t()) code += 4 ;
		if (p->u() > center->u()) code += 2 ;
		if (p->v() > center->v()) code += 1 ;
		l[code]->pushTMaxIntersection(p);  code = 4;
	}

	code = 0 ; //  if p is in umin_intersections, we always have p->u() <= center->u()
	foreach(QPoint4f * p, umin_intersections) {
		if (p->s() > center->s()) code += 8 ;
		if (p->t() > center->t()) code += 4 ;
		// if (p->u() > center->u()) code += 2 ;
		if (p->v() > center->v()) code += 1 ;
		l[code]->pushUMinIntersection(p);  code = 0;
	}

	code = 2 ; // if p is in umax_intersections, we always have p->u() > center->u()
	foreach(QPoint4f * p, umax_intersections) {
		if (p->s() > center->s()) code += 8 ;
		if (p->t() > center->t()) code += 4 ;
		// if (p->u() > center->u()) code += 2 ;
		if (p->v() > center->v()) code += 1 ;
		l[code]->pushUMaxIntersection(p); code = 2;
	}

	code = 0 ; //  if p is in vmin_intersections, we always have p->v() <= center->v()
	foreach(QPoint4f * p, vmin_intersections) {
		if (p->s() > center->s()) code += 8 ;
		if (p->t() > center->t()) code += 4 ;
		if (p->u() > center->u()) code += 2 ;
		// if (p->v() > center->v()) code += 1 ;
		l[code]->pushVMinIntersection(p); code = 0;
	}

	code = 1 ; // if p is in vmax_intersections, we always have p->v() > center->v()
	foreach(QPoint4f * p, vmax_intersections) {
		if (p->s() > center->s()) code += 8 ;
		if (p->t() > center->t()) code += 4 ;
		if (p->u() > center->u()) code += 2 ;
		// if (p->v() > center->v()) code += 1 ;
		l[code]->pushVMaxIntersection(p); code = 1;
	}

	return l ;
}


float QCell4D::abs(float a) { return (a<0)?-a:a ; }


QCell4D * QCell4D::intersection(QCell4D * c1, QCell4D * c2)
{
	// Is the intersection non empty ?
	if(c1->smin() < c1->smax() && c1->smax() <= c2->smin() && c2->smin() < c2->smax()) return NULL ;
	if(c2->smin() < c2->smax() && c2->smax() <= c1->smin() && c1->smin() < c1->smax()) return NULL ;

	if(c1->tmin() < c1->tmax() && c1->tmax() <= c2->tmin() && c2->tmin() < c2->tmax()) return NULL ;
	if(c2->tmin() < c2->tmax() && c2->tmax() <= c1->tmin() && c1->tmin() < c1->tmax()) return NULL ;

	if(c1->umin() < c1->umax() && c1->umax() <= c2->umin() && c2->umin() < c2->umax()) return NULL ;
	if(c2->umin() < c2->umax() && c2->umax() <= c1->umin() && c1->umin() < c1->umax()) return NULL ;

	if(c1->vmin() < c1->vmax() && c1->vmax() <= c2->vmin() && c2->vmin() < c2->vmax()) return NULL ;
	if(c2->vmin() < c2->vmax() && c2->vmax() <= c1->vmin() && c1->vmin() < c1->vmax()) return NULL ;
	
	// Compute cell size
	float smin = (c1->smin()<c2->smin()) ? c1->smin() : c2->smin() ;
	float smax = (c1->smax()<c2->smax()) ? c1->smax() : c2->smax() ;
	float tmin = (c1->tmin()<c2->tmin()) ? c1->tmin() : c2->tmin() ;
	float tmax = (c1->tmax()<c2->tmax()) ? c1->tmax() : c2->tmax() ;
	float umin = (c1->umin()<c2->umin()) ? c1->umin() : c2->umin() ;
	float umax = (c1->umax()<c2->umax()) ? c1->umax() : c2->umax() ;
	float vmin = (c1->vmin()<c2->vmin()) ? c1->vmin() : c2->vmin() ;
	float vmax = (c1->vmax()<c2->vmax()) ? c1->vmax() : c2->vmax() ;

	QCell4D * c = new QCell4D(smin, smax, tmin, tmax, umin, umax, vmin, vmax) ;
	
	// Inherit intersections of objects with the border of the cell
	if(c1->sMinIntersections().size() > 0 && c1->sMinIntersections().first()->s() == smin) c->setSMinIntersections(c1->sMinIntersections()) ;
	if(c2->sMinIntersections().size() > 0 && c2->sMinIntersections().first()->s() == smin) c->setSMinIntersections(c2->sMinIntersections()) ;
	
	if(c1->sMaxIntersections().size() > 0 && c1->sMaxIntersections().first()->s() == smax) c->setSMaxIntersections(c1->sMaxIntersections()) ;
	if(c2->sMaxIntersections().size() > 0 && c2->sMaxIntersections().first()->s() == smax) c->setSMaxIntersections(c2->sMaxIntersections()) ;
	
	if(c1->tMinIntersections().size() > 0 && c1->tMinIntersections().first()->t() == tmin) c->setTMinIntersections(c1->tMinIntersections()) ;
	if(c2->tMinIntersections().size() > 0 && c2->tMinIntersections().first()->t() == tmin) c->setTMinIntersections(c2->tMinIntersections()) ;
	
	if(c1->tMaxIntersections().size() > 0 && c1->tMaxIntersections().first()->t() == tmax) c->setTMaxIntersections(c1->tMaxIntersections()) ;
	if(c2->tMaxIntersections().size() > 0 && c2->tMaxIntersections().first()->t() == tmax) c->setTMaxIntersections(c2->tMaxIntersections()) ;

	if(c1->uMinIntersections().size() > 0 && c1->uMinIntersections().first()->u() == umin) c->setUMinIntersections(c1->uMinIntersections()) ;
	if(c2->uMinIntersections().size() > 0 && c2->uMinIntersections().first()->u() == umin) c->setUMinIntersections(c2->uMinIntersections()) ;
	
	if(c1->uMaxIntersections().size() > 0 && c1->uMaxIntersections().first()->u() == umax) c->setUMaxIntersections(c1->uMaxIntersections()) ;
	if(c2->uMaxIntersections().size() > 0 && c2->uMaxIntersections().first()->u() == umax) c->setUMaxIntersections(c2->uMaxIntersections()) ;
	
	if(c1->vMinIntersections().size() > 0 && c1->vMinIntersections().first()->v() == vmin) c->setVMinIntersections(c1->vMinIntersections()) ;
	if(c2->vMinIntersections().size() > 0 && c2->vMinIntersections().first()->v() == vmin) c->setVMinIntersections(c2->vMinIntersections()) ;
	
	if(c1->vMaxIntersections().size() > 0 && c1->vMaxIntersections().first()->t() == vmax) c->setVMaxIntersections(c1->vMaxIntersections()) ;
	if(c2->vMaxIntersections().size() > 0 && c2->vMaxIntersections().first()->t() == vmax) c->setVMaxIntersections(c2->vMaxIntersections()) ;

	// Inherit singularities of objects
	foreach(QPoint4f * sp, c1->singularPoints()) 
		if (sp->s() >= smin && sp->s() <= smax && sp->t() >= tmin && sp->t() <= tmax && sp->u() >= umin && sp->u() <= umax && sp->v() >= vmin && sp->v() <= vmax) 
			c->pushSingularPoint(sp) ;
			
	foreach(QPoint4f * sp, c2->singularPoints()) 
		if (sp->s() >= smin && sp->s() <= smax && sp->t() >= tmin && sp->t() <= tmax && sp->u() >= umin && sp->u() <= umax && sp->v() >= vmin && sp->v() <= vmax) 
			c->pushSingularPoint(sp) ;
			
	return c ;
}

int QCell4D::id_ = 0 ;

} ; // namespace QData
