/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QGraph
 * 2006-07-20
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# include <QData/QGraphNode.h>
# include <QData/QGraphEdge.h>
# include <QData/QGraph.h>

namespace QData {

QGraph::QGraph(void)
{
	
}

void QGraph::addNode(QGraphNode *node)
{
    nodeList << node ;
    emit changed() ;
}

void QGraph::addEdge(QGraphEdge *edge)
{
    edgeList << edge ;
    emit changed() ;
}

void QGraph::takeNode(QGraphNode * node)
{
    foreach(QGraphEdge * e, node->adjacentEdges()) 
	takeEdge(e) ;
    nodeList.removeAll(node) ;
    emit changed() ;
}

void QGraph::takeEdge(QGraphEdge * edge) 
{
    foreach(QGraphNode * n, edge->adjacentNodes())
	n->takeEdge(edge) ;
    edgeList.removeAll(edge) ;
    emit changed() ;
}

QList<QGraphNode *> QGraph::nodes(void) 
{
    return nodeList ;
}

QList<QGraphEdge *> QGraph::edges(void) 
{
    return edgeList ;
}

QList<QGraphNode *> QGraph::rootNodes(void) 
{
    QList<QGraphNode *> roots ;
    foreach(QGraphNode * n, nodeList)
	if(n->inNodesList().size() == 0)
	    roots << n ;
    return roots ;
}

void QGraph::update(void) 
{ 
    emit changed() ; 
}

void QGraph::clear(void)
{
    nodeList.clear() ;
    edgeList.clear() ;
}

QGraph::GraphType QGraph::type(void)
{
    return g_type ;
}

void QGraph::setType(GraphType type)
{
    g_type = type ;
}

void QGraph::resetColors(void)
{
    foreach(QGraphNode * n, nodeList)
	n->resetColors() ;

    foreach(QGraphNode * n, rootNodes())
	n->setColors(Qt::magenta, Qt::darkMagenta) ;

    update() ;
}

} ; // namespace QData

