/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QGraph
 * 2006-07-20
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# ifndef QGRAPH_H
# define QGRAPH_H

# include <QObject>
# include <QList>

namespace QData {

class QGraphNode ;
class QGraphEdge ;

/****************************************************************************/
/*!
  \class QGraph QGraph.h
  \brief The class QGraph provides a base class for graphs such as quadtrees,
  octrees, influence graphs etc.

  It can be used as a model for the QGraphWidget. In such case, its type has
  to be specified so that slots can be determined.
*/
/****************************************************************************/
class QGraph : public QObject
{	
    Q_OBJECT
    Q_ENUMS(GraphType)
    
public:
    QGraph(void) ;
    
    QList<QGraphNode *> nodes(void) ;
    QList<QGraphEdge *> edges(void) ;
    QList<QGraphNode *> rootNodes(void) ;

    enum GraphType { AugmentedInfluenceGraph, Quadtree, Octree } ;

signals:
    void changed(void) ;					

public slots:	
    void addNode(QGraphNode * node) ;
    void addEdge(QGraphEdge * edge) ;
    
    void takeNode(QGraphNode * node) ;
    void takeEdge(QGraphEdge * edge) ;
    
    void update(void) ;
    void clear(void) ;

    GraphType type(void) ;
    void setType(GraphType type) ;

public slots:
    void resetColors(void) ;
    
protected:
    QList<QGraphNode *> nodeList ;
    QList<QGraphEdge *> edgeList ;
    GraphType g_type ;
} ;

} ;

# endif // QGRAPH_H
