/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QGraphNode
 * 2006-10-04
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# include <QList>
# include <QData/QCell.h>
# include <QData/QRegion3f.h>
# include <QData/QGraphNode.h>
# include <QData/QGraphEdge.h>

namespace QData {

QGraphNode::QGraphNode(QString label, QRegion3f * region) 
{
    this->label_ = label ;
    this->region_ = region ;
    this->cell_ = NULL ;
    this->n_type = Graph ;
    this->lightColor_ = Qt::yellow ;
    this->darkColor_ = Qt::darkYellow ;
}
    
QGraphNode::QGraphNode(QString label, QCell * cell) 
{
    this->label_ = label ;
    this->region_ = NULL ;
    this->cell_ = cell ;
    this->n_type = Tree ;
    lightColor_ = Qt::yellow ;
    darkColor_ = Qt::darkYellow ;
}


QString QGraphNode::label(void) 
{
    return label_ ;
}

void QGraphNode::setColors(QColor lightColor, QColor darkColor) 
{
    this->lightColor_ = lightColor ;
    this->darkColor_ = darkColor ;
}

void QGraphNode::setLightColor(QColor lightColor)
{
    this->lightColor_ = lightColor ;
}

void QGraphNode::setDarkColor(QColor darkColor) 
{
    this->darkColor_ = darkColor ;
}

void QGraphNode::setAlpha(int alpha)
{
    this->lightColor_.setAlpha(alpha) ;
    this->darkColor_.setAlpha(alpha) ;
}

QColor QGraphNode::lightColor(void) 
{ 
    return lightColor_ ; 
}

QColor QGraphNode::darkColor(void) 
{ 
    return darkColor_ ; 
}

void QGraphNode::resetColors(void)
{
    this->lightColor_ = Qt::yellow ;
    this->darkColor_  = Qt::darkYellow ;
}

void QGraphNode::addEdge(QGraphEdge *edge)
{
    edgeList << edge ;
}

void QGraphNode::takeEdge(QGraphEdge * edge) 
{
    edgeList.removeAll(edge) ;
}

QList<QGraphEdge *> QGraphNode::edges() const
{
    return edgeList;
}

QList<QGraphEdge *> QGraphNode::adjacentEdges() const
{
    return edgeList;
}

void QGraphNode::setPosition(float x, float y) 
{
    xpos = x ;
    ypos = y ;
}

QList<QGraphNode *> QGraphNode::parents(void)
{
    return inNodesList() ;
}

QList<QGraphNode *> QGraphNode::children(void)
{
    return outNodesList() ;
}

QList<QGraphNode *> QGraphNode::inNodesList(void) 
{
    QList<QGraphNode *> inNodes ;
    foreach(QGraphEdge * e, edgeList)
	if(e->destNode() == this) 
	    inNodes << e->destNode() ;
    return inNodes ;
}

QList<QGraphNode *> QGraphNode::outNodesList(void)
{
    QList<QGraphNode *> outNodes ;
    foreach(QGraphEdge * e, edgeList)
	if(e->sourceNode() == this) 
	    outNodes << e->destNode() ;
    return outNodes ;
}

bool QGraphNode::isLeaf(void)
{
    bool leaf = true ;
    foreach(QGraphEdge * e, edgeList)
	if(e->sourceNode() == this) 
	    leaf = false ;
    return leaf ;
}

QRegion3f * QGraphNode::region(void)
{
    return region_ ;
}

QCell * QGraphNode::cell(void)
{
    if(!region_) return cell_ ;
    return region_->cell() ;
}

} ; // namesapce QData
