/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QTreeNode4D
 * 2007-05-09
 * Stephane CHAU
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

/*!***************************************************************************
  This is the constructor for the node, doing some stuff of initialization.
*****************************************************************************/
template <class Object, class Cell> QTreeNode4D<Object, Cell>::QTreeNode4D(NODE_TYPE nodeType, QTreeNode4D<Object, Cell> * theParent, Cell &theCell)
{  
    cell = theCell;
    type = nodeType; 
	parent = theParent;

	LLLLchild = NULL;
	LLLRchild = NULL;
	LLRLchild = NULL;
	LLRRchild = NULL;
	LRLLchild = NULL;
	LRLRchild = NULL;
	LRRLchild = NULL;
	LRRRchild = NULL;
	RLLLchild = NULL;
	RLLRchild = NULL;
	RLRLchild = NULL;
	RLRRchild = NULL;
	RRLLchild = NULL;
	RRLRchild = NULL;
	RRRLchild = NULL;
	RRRRchild = NULL;

    depth = theParent->depth+1;
	
	switch( nodeType )
	{
		case LLLL : theParent->SetLLLLchild(this);
		break;
		case LLLR : theParent->SetLLLRchild(this);
		break;
		case LLRL : theParent->SetLLRLchild(this);
		break;
		case LLRR : theParent->SetLLRRchild(this);
		break;
		case LRLL : theParent->SetLRLLchild(this);
		break;
		case LRLR : theParent->SetLRLRchild(this);
		break;
		case LRRL : theParent->SetLRRLchild(this);
		break;
		case LRRR : theParent->SetLRRRchild(this);
		break;
		case RLLL : theParent->SetRLLLchild(this);
		break;
		case RLLR : theParent->SetRLLRchild(this);
		break;
		case RLRL : theParent->SetRLRLchild(this);
		break;
		case RLRR : theParent->SetRLRRchild(this);
		break;
		case RRLL : theParent->SetRRLLchild(this);
		break;
		case RRLR : theParent->SetRRLRchild(this);
		break;
		case RRRL : theParent->SetRRRLchild(this);
		break;
		case RRRR : theParent->SetRRRRchild(this);
		break;
		default: std::cerr << "Error : the node's type isn't appropriate \n";
		break;
	}
}


template <class Object, class Cell> QTreeNode4D<Object, Cell>::QTreeNode4D(int nodeType, QTreeNode4D<Object, Cell> * theParent, Cell & theCell)
{  
    cell = theCell;
	parent = theParent;

	LLLLchild = NULL;
	LLLRchild = NULL;
	LLRLchild = NULL;
	LLRRchild = NULL;
	LRLLchild = NULL;
	LRLRchild = NULL;
	LRRLchild = NULL;
	LRRRchild = NULL;
	RLLLchild = NULL;
	RLLRchild = NULL;
	RLRLchild = NULL;
	RLRRchild = NULL;
	RRLLchild = NULL;
	RRLRchild = NULL;
	RRRLchild = NULL;
	RRRRchild = NULL;

    depth = theParent->depth+1;
	
	switch( nodeType )
	{
		case 0: theParent->SetLLLLchild(this); type = LLLL; 
		break;
		case 1: theParent->SetLLLRchild(this); type = LLLR;
		break;
		case 2: theParent->SetLLRLchild(this); type = LLRL;
		break;
		case 3: theParent->SetLLRRchild(this); type = LLRR;
		break;
		case 4: theParent->SetLRLLchild(this); type = LRLL;
		break;
		case 5: theParent->SetLRLRchild(this); type = LRLR;
		break;
		case 6: theParent->SetLRRLchild(this); type = LRRL;
		break;
		case 7: theParent->SetLRRRchild(this); type = LRRR;
		break;
		case 8: theParent->SetRLLLchild(this); type = RLLL;
		break;
		case 9: theParent->SetRLLRchild(this); type = RLLR;
		break;
		case 10: theParent->SetRLRLchild(this); type = RLRL;
		break;
		case 11: theParent->SetRLRRchild(this); type = RLRR;
		break;
		case 12: theParent->SetRRLLchild(this); type = RRLL;
		break;
		case 13: theParent->SetRRLRchild(this); type = RRLR;
		break;
		case 14: theParent->SetRRRLchild(this); type = RRRL;
		break;
		case 15: theParent->SetRRRRchild(this); type = RRRR;
		break;
		default: std::cerr << "Error : the node's type isn't appropriate \n";
		break;
	}
}

// {
// 	switch (nodeType)
// 	{
// 		case 0: QTreeNode4D<Object, Cell>(LLLL, parent, cell);
// 		break;
// 		case 1: QTreeNode4D<Object, Cell>(LLLR, parent, cell);
// 		break;
// 		case 2: QTreeNode4D<Object, Cell>(LLRL, parent, cell);
// 		break;
// 		case 3: QTreeNode4D<Object, Cell>(LLRR, parent, cell);
// 		break;
// 		case 4: QTreeNode4D<Object, Cell>(LRLL, parent, cell);
// 		break;
// 		case 5: QTreeNode4D<Object, Cell>(LRLR, parent, cell);
// 		break;
// 		case 6: QTreeNode4D<Object, Cell>(LRRL, parent, cell);
// 		break;
// 		case 7: QTreeNode4D<Object, Cell>(LRRR, parent, cell);
// 		break;
// 		case 8: QTreeNode4D<Object, Cell>(RLLL, parent, cell);
// 		break;
// 		case 9: QTreeNode4D<Object, Cell>(RLLR, parent, cell);
// 		break;
// 		case 10: QTreeNode4D<Object, Cell>(RLRL, parent, cell);
// 		break;
// 		case 11: QTreeNode4D<Object, Cell>(RLRR, parent, cell);
// 		break;
// 		case 12: QTreeNode4D<Object, Cell>(RRLL, parent, cell);
// 		break;
// 		case 13: QTreeNode4D<Object, Cell>(RRLR, parent, cell);
// 		break;
// 		case 14: QTreeNode4D<Object, Cell>(RRRL, parent, cell);
// 		break;
// 		case 15: QTreeNode4D<Object, Cell>(RRRR, parent, cell);
// 		break;
// 		default: qCritical("Problem in the constructor of QTreeNode4D with an integer") ;
// 		break;
// 	}
// }
// no->LLLLchild = new Node_t(LLLL, no, cells[0]);	
// }
// // template <class Object, class Cell> QTreeNode4D<Object, Cell>::QTreeNode4D(QTreeNode4D<Object, Cell> * left, QTreeNode4D<Object, Cell> * right, Cell cell)
// // {
// //     this->parent = NULL ;
// //     this->cell   = cell ;
// // 
// //     left->type  = F_NW ; left->parent =  this ; F_NWchild = left ;
// //     right->type = F_NE ; right->parent = this ; F_NEchild = right ;
// // 
// //     F_NWchild = left  ; F_SWchild = NULL ;
// //     F_NEchild = right ; F_SEchild = NULL ;
// //     B_NEchild = NULL  ; B_NWchild = NULL ; 
// //     B_SEchild = NULL  ; B_SWchild = NULL ;
// // 
// //     depth = 0 ; left->depth++ ; right->depth++ ;
// // }
// 
template<class Object, class Cell> QTreeNode4D<Object, Cell>::QTreeNode4D(Object object, Cell cell)
{ 
    this->type   = LLLL ;
    this->parent = NULL ;
    this->cell   = cell ;
    this->objects << object ;

	LLLLchild = NULL;
	LLLRchild = NULL;
	LLRLchild = NULL;
	LLRRchild = NULL;
	LRLLchild = NULL;
	LRLRchild = NULL;
	LRRLchild = NULL;
	LRRRchild = NULL;
	RLLLchild = NULL;
	RLLRchild = NULL;
	RLRLchild = NULL;
	RLRRchild = NULL;
	RRLLchild = NULL;
	RRLRchild = NULL;
	RRRLchild = NULL;
	RRRRchild = NULL;

    depth = 0;
    index = 0;
} 

/*! This is the constructor for the node, doing some stuff of 
  initialization. This constructor must only be called by the
  constructor of HexaTree. */
template<class Object, class Cell> QTreeNode4D<Object, Cell>::QTreeNode4D(void)
{ 
    type = LLLL;
    parent = NULL; 

	LLLLchild = NULL;
	LLLRchild = NULL;
	LLRLchild = NULL;
	LLRRchild = NULL;
	LRLLchild = NULL;
	LRLRchild = NULL;
	LRRLchild = NULL;
	LRRRchild = NULL;
	RLLLchild = NULL;
	RLLRchild = NULL;
	RLRLchild = NULL;
	RLRRchild = NULL;
	RRLLchild = NULL;
	RRLRchild = NULL;
	RRRLchild = NULL;
	RRRRchild = NULL;

    depth = 0;

    index = 0;
} 

/************************************************ 
 *           Some additional functions          * 
 ************************************************/ 

/* Set the appropriate child corresponding to the given integer */ 
template<class Object, class Cell>
void QTreeNode4D<Object, Cell>::SetChild(int i, QTreeNode4D<Object, Cell> * n)
{
	switch(i)
	{
		case 0: SetLLLLchild(n);
		break;  
		case 1: SetLLLRchild(n);
		break;  
		case 2: SetLLRLchild(n);
		break;  
		case 3: SetLLRRchild(n);
		break;  
		case 4: SetLRLLchild(n);
		break;  
		case 5: SetLRLRchild(n);
		break;  
		case 6: SetLRRLchild(n);
		break;  
		case 7: SetLRRRchild(n);
		break;  
		case 8: SetRLLLchild(n);
		break;  
		case 9: SetRLLRchild(n);
		break;  
		case 10:SetRLRLchild(n);
		break;  
		case 11:SetRLRRchild(n);
		break;  
		case 12:SetRRLLchild(n);
		break;  
		case 13:SetRRLRchild(n);
		break;  
		case 14:SetRRRLchild(n);
		break;  
		case 15:SetRRRRchild(n);
		break;              
		default: std::cout << "Bad choice of child number" << std::endl;
		break;
	}
}

/* Give the appropriate child corresponding to the given integer */ 
template<class Object, class Cell>
QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::GetChild(int i)
// QTreeNode4D * QTreeNode4D<Object, Cell>::GetChild(int i)
{
	switch(i)
	{
		case 0: return GetLLLLchild();
		break;
		case 1: return GetLLLRchild();
		break;
		case 2: return GetLLRLchild();
		break;
		case 3: return GetLLRRchild();
		break;
		case 4: return GetLRLLchild();
		break;
		case 5: return GetLRLRchild();
		break;
		case 6: return GetLRRLchild();
		break;
		case 7: return GetLRRRchild();
		break;
		case 8: return GetRLLLchild();
		break;
		case 9: return GetRLLRchild();
		break;
		case 10: return GetRLRLchild();
		break;
		case 11: return GetRLRRchild();
		break;
		case 12: return GetRRLLchild();
		break;
		case 13: return GetRRLRchild();
		break;
		case 14: return GetRRRLchild();
		break;
		case 15: return GetRRRRchild();
		break;              
		default: std::cout << "Bad choice of child number" << std::endl; return NULL;
		break;
	}
}

/*! Check to see if current node is a leaf or not, remember to check all. */ 
template<class Object, class Cell>
bool QTreeNode4D<Object, Cell>::isLeaf(void)
{
    if(LLLLchild == NULL &&
		LLLRchild == NULL &&
		LLRLchild == NULL &&
		LLRRchild == NULL &&
		LRLLchild == NULL &&
		LRLRchild == NULL &&
		LRRLchild == NULL &&
		LRRRchild == NULL &&
		RLLLchild == NULL &&
		RLLRchild == NULL &&
		RLRLchild == NULL &&
		RLRRchild == NULL &&
		RRLLchild == NULL &&
		RRLRchild == NULL &&
		RRRLchild == NULL &&
		RRRRchild == NULL)
	return true; 
	else return false; 
} 


/**************************************** 
 *     DUE PARTS						* 
 ****************************************/ 
 
//--------------- DUE SMIN--------------------- 

template <class Object, class Cell>
void QTreeNode4D<Object, Cell>::getAllSminNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
{ 
    //get all smin neighb of the current node 
    QTreeNode4D<Object, Cell>* tmpNode = getSminNeighborNode(); 
    if(tmpNode != NULL) 
	tmpNode->getSmaxBoundary(nbrs); 
}

/*! This is for the child to call. */
template<class Object, class Cell>
QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::getSminNeighborNode(void)
{
    //get one special node on the smin side 
    if(parent == NULL) return NULL; 

	// switch(type)
	// 	{ 
	// 		case R*: { return (parent->GetR*child()); } 
	// 		
	// 		case L*:
	// 		{
	// 			QTreeNode4D<Object, Cell>* tmp = parent->getSminNeighborNode(); 
	// 			if(tmp == NULL)
	// 			    return NULL; 
	// 			if(tmp->isLeaf())
	// 			    return tmp; 
	// 			else
	// 			    return (tmp->GetR*child());
	// 		} 
	// 		default: return NULL; 
	// 	} 
	if (type >= 8) { return (parent->GetChild(type)) ;}
	else
	{
		QTreeNode4D<Object, Cell>* tmp = parent->getSminNeighborNode(); 
		if(tmp == NULL)
		    return NULL; 
		if(tmp->isLeaf())
		    return tmp; 
		else
		    return (tmp->GetChild(type+8));
	}

} 

/*! Get all the node on the smax side. */
template<class Object, class Cell>
void QTreeNode4D<Object, Cell>::getSmaxBoundary(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
{ 
    if(isLeaf())
	nbrs.push_back(this); 
    else
    { 
	// if ( R*child != NULL )
	//     R*child->getSmaxBoundary(nbrs); 
	for (int i = 8; i < 16; ++i)
		if ( GetChild(i) != NULL ) GetChild(i)->getSmaxBoundary(nbrs);
    } 
} 

//--------------- DUE SMAX--------------------- 

template <class Object, class Cell>
void QTreeNode4D<Object, Cell>::getAllSmaxNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
{ 
    //get all smin neighb of the current node 
    QTreeNode4D<Object, Cell>* tmpNode = getSmaxNeighborNode(); 
    if(tmpNode != NULL) 
	tmpNode->getSminBoundary(nbrs); 
}

/*! This is for the child to call. */
template<class Object, class Cell>
QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::getSmaxNeighborNode(void)
{
    //get one special node on the smax side 
    if(parent == NULL) return NULL; 

	// switch(type)
	// 	{ 
	// 		case L*: { return (parent->GetL*child()); } 
	// 		
	// 		case R*:
	// 		{
	// 			QTreeNode4D<Object, Cell>* tmp = parent->getSmaxNeighborNode(); 
	// 			if(tmp == NULL)
	// 			    return NULL; 
	// 			if(tmp->isLeaf())
	// 			    return tmp; 
	// 			else
	// 			    return (tmp->GetL*child());
	// 		} 
	// 		default: return NULL; 
	// 	} 
	if (type <= 8) { return (parent->GetChild(type)) ;}
	else
	{
		QTreeNode4D<Object, Cell>* tmp = parent->getSmaxNeighborNode(); 
		if(tmp == NULL)
		    return NULL; 
		if(tmp->isLeaf())
		    return tmp; 
		else
		    return (tmp->GetChild(type-8));
	}

} 

/*! Get all the node on the smin side. */
template<class Object, class Cell>
void QTreeNode4D<Object, Cell>::getSminBoundary(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
{ 
    if(isLeaf())
	nbrs.push_back(this); 
    else
    { 
	// if ( L*child != NULL )
	//     L*child->getSminBoundary(nbrs); 
	for (int i = 0; i < 8; ++i)
		if ( GetChild(i) != NULL ) GetChild(i)->getSminBoundary(nbrs);
    } 
} 


//--------------- DUE TMIN--------------------- 

template <class Object, class Cell>
void QTreeNode4D<Object, Cell>::getAllTminNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
{ 
    //get all tmin neighb of the current node 
    QTreeNode4D<Object, Cell>* tmpNode = getTminNeighborNode(); 
    if(tmpNode != NULL) 
	tmpNode->getTmaxBoundary(nbrs); 
}

/*! This is for the child to call. */
template<class Object, class Cell>
QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::getTminNeighborNode(void)
{
    //get one special node on the tmin side 
    if(parent == NULL) return NULL; 

	// switch(type)
	// 	{ 
	// 		case *R**: { return (parent->Get*R**child()); } 
	// 		
	// 		case *L**:
	// 		{
	// 			QTreeNode4D<Object, Cell>* tmp = parent->getTminNeighborNode(); 
	// 			if(tmp == NULL)
	// 			    return NULL; 
	// 			if(tmp->isLeaf())
	// 			    return tmp; 
	// 			else
	// 			    return (tmp->Get*R**child());
	// 		} 
	// 		default: return NULL; 
	// 	} 
	for (int i = 0; i < 2; ++i)
		for (int j = 0; j < 2; ++j)
			for (int k = 0; k < 2; ++k)
			{
				if (type == 8*i+4+2*j+k) { return (parent->GetChild(type)) ;}
				if (type == 8*i+2*j+k)
				{
					QTreeNode4D<Object, Cell>* tmp = parent->getTminNeighborNode(); 
					if(tmp == NULL)
					    return NULL; 
					if(tmp->isLeaf())
					    return tmp; 
					else
					    return (tmp->GetChild(type+4));
				}
				
			}
} 

/*! Get all the node on the tmax side. */
template<class Object, class Cell>
void QTreeNode4D<Object, Cell>::getTmaxBoundary(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
{ 
    if(isLeaf())
	nbrs.push_back(this); 
    else
    { 
	// if ( *R**child != NULL )
	//     *R**child->getTmaxBoundary(nbrs); 
		for (int i = 0; i < 2; ++i)
			for (int j = 0; j < 2; ++j)
				for (int k = 0; k < 2; ++k)
					if (GetChild(8*i+4+2*j+k) != NULL) (GetChild(8*i+4+2*j+k)->getTmaxBoundary(nbrs));
    } 
} 

//--------------- DUE TMAX--------------------- 

template <class Object, class Cell>
void QTreeNode4D<Object, Cell>::getAllTmaxNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
{ 
    //get all tmax neighb of the current node 
    QTreeNode4D<Object, Cell>* tmpNode = getTmaxNeighborNode(); 
    if(tmpNode != NULL) 
	tmpNode->getTminBoundary(nbrs); 
}

/*! This is for the child to call. */
template<class Object, class Cell>
QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::getTmaxNeighborNode(void)
{
    //get one special node on the tmax side 
    if(parent == NULL) return NULL; 

	// switch(type)
	// 	{ 
	// 		case *L**: { return (parent->Get*L**child()); } 
	// 		
	// 		case *R**:
	// 		{
	// 			QTreeNode4D<Object, Cell>* tmp = parent->getTmaxNeighborNode(); 
	// 			if(tmp == NULL)
	// 			    return NULL; 
	// 			if(tmp->isLeaf())
	// 			    return tmp; 
	// 			else
	// 			    return (tmp->Get*L**child());
	// 		} 
	// 		default: return NULL; 
	// 	} 
	for (int i = 0; i < 2; ++i)
		for (int j = 0; j < 2; ++j)
			for (int k = 0; k < 2; ++k)
			{
				if (type == 8*i+2*j+k) { return (parent->GetChild(type)) ;}
				if (type == 8*i+4+2*j+k)
				{
					QTreeNode4D<Object, Cell>* tmp = parent->getTmaxNeighborNode(); 
					if(tmp == NULL)
					    return NULL; 
					if(tmp->isLeaf())
					    return tmp; 
					else
					    return (tmp->GetChild(type-4));
				}
				
			}
} 

/*! Get all the node on the tmin side. */
template<class Object, class Cell>
void QTreeNode4D<Object, Cell>::getTminBoundary(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
{ 
    if(isLeaf())
	nbrs.push_back(this); 
    else
    { 
	// if ( *L**child != NULL )
	//     *L**child->getTminBoundary(nbrs); 
		for (int i = 0; i < 2; ++i)
			for (int j = 0; j < 2; ++j)
				for (int k = 0; k < 2; ++k)
					if (GetChild(8*i+2*j+k) != NULL) (GetChild(8*i+2*j+k)->getTminBoundary(nbrs));
    } 
}

//--------------- DUE UMIN--------------------- 

template <class Object, class Cell>
void QTreeNode4D<Object, Cell>::getAllUminNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
{ 
    //get all umin neighb of the current node 
    QTreeNode4D<Object, Cell>* tmpNode = getUminNeighborNode(); 
    if(tmpNode != NULL) 
	tmpNode->getUmaxBoundary(nbrs); 
}

/*! This is for the child to call. */
template<class Object, class Cell>
QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::getUminNeighborNode(void)
{
    //get one special node on the umin side 
    if(parent == NULL) return NULL; 

	// switch(type)
	// 	{ 
	// 		case **R*: { return (parent->Get**R*child()); } 
	// 		
	// 		case **L*:
	// 		{
	// 			QTreeNode4D<Object, Cell>* tmp = parent->getUminNeighborNode(); 
	// 			if(tmp == NULL)
	// 			    return NULL; 
	// 			if(tmp->isLeaf())
	// 			    return tmp; 
	// 			else
	// 			    return (tmp->Get**R*child());
	// 		} 
	// 		default: return NULL; 
	// 	} 
	for (int i = 0; i < 2; ++i)
		for (int j = 0; j < 2; ++j)
			for (int k = 0; k < 2; ++k)
			{
				if (type == 8*i+4*j+2+k) { return (parent->GetChild(type)) ;}
				if (type == 8*i+4*j+k)
				{
					QTreeNode4D<Object, Cell>* tmp = parent->getUminNeighborNode(); 
					if(tmp == NULL)
					    return NULL; 
					if(tmp->isLeaf())
					    return tmp; 
					else
					    return (tmp->GetChild(type+2));
				}
				
			}
} 

/*! Get all the node on the umax side. */
template<class Object, class Cell>
void QTreeNode4D<Object, Cell>::getUmaxBoundary(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
{ 
    if(isLeaf())
	nbrs.push_back(this); 
    else
    { 
	// if ( **R*child != NULL )
	//     **R*child->getUmaxBoundary(nbrs); 
		for (int i = 0; i < 2; ++i)
			for (int j = 0; j < 2; ++j)
				for (int k = 0; k < 2; ++k)
					if (GetChild(8*i+4*j+2+k) != NULL) (GetChild(8*i+4*j+2+k)->getUmaxBoundary(nbrs));
    } 
} 

//--------------- DUE UMAX--------------------- 

template <class Object, class Cell>
void QTreeNode4D<Object, Cell>::getAllUmaxNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
{ 
    //get all umax neighb of the current node 
    QTreeNode4D<Object, Cell>* tmpNode = getUmaxNeighborNode(); 
    if(tmpNode != NULL) 
	tmpNode->getUminBoundary(nbrs); 
}

/*! This is for the child to call. */
template<class Object, class Cell>
QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::getUmaxNeighborNode(void)
{
    //get one special node on the umax side 
    if(parent == NULL) return NULL; 

	// switch(type)
	// 	{ 
	// 		case **L*: { return (parent->Get**L*child()); } 
	// 		
	// 		case **R*:
	// 		{
	// 			QTreeNode4D<Object, Cell>* tmp = parent->getUmaxNeighborNode(); 
	// 			if(tmp == NULL)
	// 			    return NULL; 
	// 			if(tmp->isLeaf())
	// 			    return tmp; 
	// 			else
	// 			    return (tmp->Get**L*child());
	// 		} 
	// 		default: return NULL; 
	// 	} 
	for (int i = 0; i < 2; ++i)
		for (int j = 0; j < 2; ++j)
			for (int k = 0; k < 2; ++k)
			{
				if (type == 8*i+4*j+k) { return (parent->GetChild(type)) ;}
				if (type == 8*i+4*j+2+k)
				{
					QTreeNode4D<Object, Cell>* tmp = parent->getUmaxNeighborNode(); 
					if(tmp == NULL)
					    return NULL; 
					if(tmp->isLeaf())
					    return tmp; 
					else
					    return (tmp->GetChild(type-2));
				}
				
			}
} 

/*! Get all the node on the umin side. */
template<class Object, class Cell>
void QTreeNode4D<Object, Cell>::getUminBoundary(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
{ 
    if(isLeaf())
	nbrs.push_back(this); 
    else
    { 
	// if ( **L*child != NULL )
	//     **L*child->getUminBoundary(nbrs); 
		for (int i = 0; i < 2; ++i)
			for (int j = 0; j < 2; ++j)
				for (int k = 0; k < 2; ++k)
					if (GetChild(8*i+4*j+k) != NULL) (GetChild(8*i+4*j+k)->getUminBoundary(nbrs));
    } 
} 


//--------------- DUE VMIN--------------------- 

template <class Object, class Cell>
void QTreeNode4D<Object, Cell>::getAllVminNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
{ 
    //get all vmin neighb of the current node 
    QTreeNode4D<Object, Cell>* tmpNode = getVminNeighborNode(); 
    if(tmpNode != NULL) 
	tmpNode->getVmaxBoundary(nbrs); 
}

/*! This is for the child to call. */
template<class Object, class Cell>
QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::getVminNeighborNode(void)
{
    //get one special node on the vmin side 
    if(parent == NULL) return NULL; 

	// switch(type)
	// 	{ 
	// 		case ***R: { return (parent->Get***Rchild()); } 
	// 		
	// 		case ***L:
	// 		{
	// 			QTreeNode4D<Object, Cell>* tmp = parent->getVminNeighborNode(); 
	// 			if(tmp == NULL)
	// 			    return NULL; 
	// 			if(tmp->isLeaf())
	// 			    return tmp; 
	// 			else
	// 			    return (tmp->Get***Rchild());
	// 		} 
	// 		default: return NULL; 
	// 	} 
	for (int i = 0; i < 2; ++i)
		for (int j = 0; j < 2; ++j)
			for (int k = 0; k < 2; ++k)
			{
				if (type == 8*i+4*j+2*k+1) { return (parent->GetChild(type)) ;}
				if (type == 8*i+4*j+2*k)
				{
					QTreeNode4D<Object, Cell>* tmp = parent->getVminNeighborNode(); 
					if(tmp == NULL)
					    return NULL; 
					if(tmp->isLeaf())
					    return tmp; 
					else
					    return (tmp->GetChild(type+1));
				}
				
			}
} 

/*! Get all the node on the vmax side. */
template<class Object, class Cell>
void QTreeNode4D<Object, Cell>::getVmaxBoundary(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
{ 
    if(isLeaf())
	nbrs.push_back(this); 
    else
    { 
	// if ( ***Rchild != NULL )
	//     ***Rchild->getVmaxBoundary(nbrs); 
		for (int i = 0; i < 2; ++i)
			for (int j = 0; j < 2; ++j)
				for (int k = 0; k < 2; ++k)
					if (GetChild(8*i+4*j+2*k+1) != NULL) (GetChild(8*i+4*j+2*k+1)->getVmaxBoundary(nbrs));
    } 
} 

//--------------- DUE VMAX--------------------- 

template <class Object, class Cell>
void QTreeNode4D<Object, Cell>::getAllVmaxNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
{ 
    //get all vmax neighb of the current node 
    QTreeNode4D<Object, Cell>* tmpNode = getVmaxNeighborNode(); 
    if(tmpNode != NULL) 
	tmpNode->getVminBoundary(nbrs); 
}

/*! This is for the child to call. */
template<class Object, class Cell>
QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::getVmaxNeighborNode(void)
{
    //get one special node on the vmax side 
    if(parent == NULL) return NULL; 

	// switch(type)
	// 	{ 
	// 		case ***L: { return (parent->Get***Lchild()); } 
	// 		
	// 		case ***R:
	// 		{
	// 			QTreeNode4D<Object, Cell>* tmp = parent->getVmaxNeighborNode(); 
	// 			if(tmp == NULL)
	// 			    return NULL; 
	// 			if(tmp->isLeaf())
	// 			    return tmp; 
	// 			else
	// 			    return (tmp->Get***Lchild());
	// 		} 
	// 		default: return NULL; 
	// 	} 
	for (int i = 0; i < 2; ++i)
		for (int j = 0; j < 2; ++j)
			for (int k = 0; k < 2; ++k)
			{
				if (type == 8*i+4*j+2*k) { return (parent->GetChild(type)) ;}
				if (type == 8*i+4*j+2*k+1)
				{
					QTreeNode4D<Object, Cell>* tmp = parent->getVmaxNeighborNode(); 
					if(tmp == NULL)
					    return NULL; 
					if(tmp->isLeaf())
					    return tmp; 
					else
					    return (tmp->GetChild(type-1));
				}
				
			}
} 

/*! Get all the node on the vmin side. */
template<class Object, class Cell>
void QTreeNode4D<Object, Cell>::getVminBoundary(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
{ 
    if(isLeaf())
	nbrs.push_back(this); 
    else
    { 
	// if ( ***Lchild != NULL )
	//     ***Lchild->getVminBoundary(nbrs); 
		for (int i = 0; i < 2; ++i)
			for (int j = 0; j < 2; ++j)
				for (int k = 0; k < 2; ++k)
					if (GetChild(8*i+4*j+2*k) != NULL) (GetChild(8*i+4*j+2*k)->getVminBoundary(nbrs));
    } 
} 
// 
// /************************************************ 
//  *             CORNER-LINE PARTS                * 
//  ************************************************/ 
// /*! The line means inner line, these routine will  
//   get the nodes , say, on the NW edge of the current node. */
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getNWLine(std::vector< QTreeNode4D<Object, Cell>*>& line)
// { 
//     if(isLeaf()) 
// 	line.push_back(this); 
//     else
//     { 
// 	if ( F_NWchild!=NULL )
// 	    F_NWchild->getNWLine(line);
// 	if (B_NWchild!=NULL )
// 	    B_NWchild->getNWLine(line); 
//     } 
// } 
// 
// /*! The line means inner line, these routine will  
//   get the nodes , say, on the NE edge of the current node. */
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getNELine(std::vector< QTreeNode4D<Object, Cell>*>& line)
// { 
//     if (isLeaf())
// 	line.push_back(this); 
//     else
//     {
// 	if ( F_NEchild!=NULL )
// 	    F_NEchild->getNELine(line);
// 	if (  B_NEchild!=NULL )
// 	    B_NEchild->getNELine(line); 
//     } 
// } 
// 
// /*! The line means inner line, these routine will  
//   get the nodes , say, on the SW edge of the current node. */
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getSWLine(std::vector< QTreeNode4D<Object, Cell>*>& line)
// { 
//     if(isLeaf())
// 	line.push_back(this); 
//     else
//     { 
// 	if (F_SWchild!=NULL )
// 	    F_SWchild->getSWLine(line); 
// 	if (B_SWchild!=NULL )
// 	    B_SWchild->getSWLine(line); 
//     } 
// } 
// 
// /*! The line means inner line, these routine will  
//   get the nodes , say, on the SE edge of the current node. */
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getSELine(std::vector< QTreeNode4D<Object, Cell>*>& line)
// { 
//     if(isLeaf())
// 	line.push_back(this); 
//     else
//     { 
// 	if (F_SEchild!=NULL )
// 	    F_SEchild->getSELine(line); 
// 	if (B_SEchild!=NULL )
// 	    B_SEchild->getSELine(line); 
//     } 
// } 
// 
// 
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getF_NLine(std::vector< QTreeNode4D<Object, Cell>*>& line)
// { 
//     if(isLeaf())
// 	line.push_back(this); 
//     else
//     {
// 	if (F_NEchild!=NULL )
// 	    F_NEchild->getF_NLine(line); 
// 	if (F_NWchild!=NULL )
// 	    F_NWchild->getF_NLine(line); 
//     } 
// } 
// 
// 
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getF_SLine(std::vector< QTreeNode4D<Object, Cell>*>& line)
// { 
//     if(isLeaf()) 
// 	line.push_back(this); 
//     else
//     { 
// 	if (F_SEchild!=NULL )
// 	    F_SEchild->getF_SLine(line); 
// 	if (F_SWchild!=NULL )
// 	    F_SWchild->getF_SLine(line); 
//     } 
// }
// 
// 
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getF_ELine(std::vector< QTreeNode4D<Object, Cell>*>& line)
// { 
//     if(isLeaf()) 
// 	line.push_back(this); 
//     else
//     {
// 	if (F_NEchild!=NULL )
// 	    F_NEchild->getF_ELine(line); 
// 	if (F_SEchild!=NULL )
// 	    F_SEchild->getF_ELine(line); 
//     } 
// } 
// 
// 
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getF_WLine(std::vector< QTreeNode4D<Object, Cell>*>& line)
// { 
//     if(isLeaf())
// 	line.push_back(this); 
//     else
//     { 
// 	if (F_NWchild!=NULL )
// 	    F_NWchild->getF_WLine(line); 
// 	if (F_SWchild!=NULL )
// 	    F_SWchild->getF_WLine(line); 
//     } 
// } 
// 
// 
// template <class Object, class Cell> 
// void QTreeNode4D<Object, Cell>::getB_NLine(std::vector< QTreeNode4D<Object, Cell>*>& line)
// { 
//     if(isLeaf()) 
// 	line.push_back(this); 
//     else
//     { 
// 	if (B_NEchild!=NULL )
// 	    B_NEchild->getB_NLine(line); 
// 	if (B_NWchild!=NULL )
// 	    B_NWchild->getB_NLine(line); 
//     } 
// } 
// 
// 
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getB_SLine(std::vector< QTreeNode4D<Object, Cell>*>& line)
// { 
//     if(isLeaf()) 
// 	line.push_back(this); 
//     else
//     { 
// 	if (B_SEchild!=NULL )
// 	    B_SEchild->getB_SLine(line); 
// 	if (B_SWchild!=NULL )
// 	    B_SWchild->getB_SLine(line); 
//     }
// } 
// 
// 
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getB_ELine(std::vector< QTreeNode4D<Object, Cell>*>& line)
// { 
//     if(isLeaf())
// 	line.push_back(this); 
//     else
//     { 
// 	if (B_NEchild!=NULL )
// 	    B_NEchild->getB_ELine(line); 
// 	if (B_SEchild!=NULL )
// 	    B_SEchild->getB_ELine(line); 
//     } 
// } 
// 
// 
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getB_WLine(std::vector< QTreeNode4D<Object, Cell>*>& line)
// { 
//     if(isLeaf()) 
// 	line.push_back(this); 
//     else
//     { 
// 	if (B_NWchild!=NULL )
// 	    B_NWchild->getB_WLine(line); 
// 	if (B_SWchild!=NULL )
// 	    B_SWchild->getB_WLine(line); 
//     } 
// } 
//  
// 
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getNWNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
// { 
//     //get the west and the north node (equivalent or greater) 
//     QTreeNode4D<Object, Cell>* WestNode = getWestNeighborNode();
//     if(WestNode == NULL)
// 	return; 
//     QTreeNode4D<Object, Cell>* NorthNode = getNorthNeighborNode();
//     if(NorthNode == NULL) 
// 	return; 
//   
//     //get the north of the west 
//     QTreeNode4D<Object, Cell>* NorthOfWestNode = WestNode->getNorthNeighborNode(); 
//     if(NorthOfWestNode == NULL)
// 	return; 
//     //get the west of the north 
//     QTreeNode4D<Object, Cell>* WestOfNorthNode = NorthNode->getWestNeighborNode();
//     if(WestOfNorthNode == NULL)
// 	return; 
//   
//     //get the south east edges of both nodes 
//     std::vector< QTreeNode4D<Object, Cell>*> SELine1; 
//     NorthOfWestNode->getSELine(SELine1);
//     if(SELine1.size() == 0)
// 	return; 
//     std::vector< QTreeNode4D<Object, Cell>*> SELine2; 
//     WestOfNorthNode->getSELine(SELine2);
//     if(SELine2.size() == 0)
// 	return; 
//   
//     for(int i = 0; i < SELine1.size(); i++) 
// 	for(int j =0; j < SELine2.size(); j++)
// 	    if(SELine1[i] == SELine2[j])
// 		nbrs.push_back(SELine1[i]); 
// } 
// 
// 
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getNENeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
// { 
//     //get the East and the north node (equivalent or greater) 
//     QTreeNode4D<Object, Cell>* EastNode = getEastNeighborNode();
//     if(EastNode == NULL)
// 	return; 
//     QTreeNode4D<Object, Cell>* NorthNode = getNorthNeighborNode();
//     if(NorthNode == NULL)
// 	return; 
//  
//     //get the north of the East 
//     QTreeNode4D<Object, Cell>* NorthOfEastNode = EastNode->getNorthNeighborNode();
//     if(NorthOfEastNode == NULL)
// 	return; 
//     //get the East of thre north 
//     QTreeNode4D<Object, Cell>* EastOfNorthNode = NorthNode->getEastNeighborNode();
//     if(EastOfNorthNode == NULL) 
// 	return; 
// 	 
//     //get the south east edges of both nodes 
//     std::vector< QTreeNode4D<Object, Cell>*> SWLine1; 
//     NorthOfEastNode->getSWLine(SWLine1);
//     if(SWLine1.size() == 0)
// 	return; 
//     std::vector< QTreeNode4D<Object, Cell>*> SWLine2; 
//     EastOfNorthNode->getSWLine(SWLine2);
//     if(SWLine2.size() == 0)
// 	return; 
// 	 
//     for(int i = 0; i < SWLine1.size(); i++)
// 	for(int j =0; j < SWLine2.size(); j++)
// 	    if(SWLine1[i] == SWLine2[j])
// 		nbrs.push_back(SWLine1[i]);
// } 
// 
// 
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getSWNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
// { 
//     //get the west and the South node (equivalent or greater) 
//     QTreeNode4D<Object, Cell>* WestNode = getWestNeighborNode();
//     if(WestNode == NULL)
// 	return; 
//     QTreeNode4D<Object, Cell>* SouthNode = getSouthNeighborNode();
//     if(SouthNode == NULL)
// 	return; 
//     //get the South of the west 
//     QTreeNode4D<Object, Cell>* SouthOfWestNode = WestNode->getSouthNeighborNode();
//     if(SouthOfWestNode == NULL) 
// 	return; 
//     //get the west of the South 
//     QTreeNode4D<Object, Cell>* WestOfSouthNode = SouthNode->getWestNeighborNode();
//     if(WestOfSouthNode == NULL)
// 	return; 
//   
//     //get the south east edges of both nodes 
//     std::vector< QTreeNode4D<Object, Cell>*> NELine1; 
//     SouthOfWestNode->getNELine(NELine1); 
//     if(NELine1.empty())
// 	return;
//     if(NELine1.size() == 0)
// 	return; 
//    
//     std::vector< QTreeNode4D<Object, Cell>*> NELine2;
//     WestOfSouthNode->getNELine(NELine2);
// 
//     if(NELine2.empty())
// 	return;
//     if(NELine2.size() == 0)
// 	return; 
//    
//     for(int i = 0; i < NELine1.size(); i++)
// 	for(int j =0; j < NELine2.size(); j++)
// 	    if(NELine1[i] == NELine2[j])
// 		nbrs.push_back(NELine1[i]); 
// } 
// 
// 
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getSENeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
// { 
//     //get the East and the South node (equivalent or greater) 
//     QTreeNode4D<Object, Cell>* EastNode = getEastNeighborNode();
//     if(EastNode == NULL)
// 	return; 
//     QTreeNode4D<Object, Cell>* SouthNode = getSouthNeighborNode();
//     if(SouthNode == NULL)
// 	return; 
//  
//     //get the South of the East 
//     QTreeNode4D<Object, Cell>* SouthOfEastNode = EastNode->getSouthNeighborNode(); 
//     if(SouthOfEastNode == NULL)
// 	return; 
//     //get the East of thre South 
//     QTreeNode4D<Object, Cell>* EastOfSouthNode = SouthNode->getEastNeighborNode();
//     if(EastOfSouthNode == NULL)
// 	return; 
// 	 
//     //get the south east edges of both nodes 
//     std::vector< QTreeNode4D<Object, Cell>*> NWLine1; 
//     SouthOfEastNode->getNWLine(NWLine1);
//     if(NWLine1.size() == 0)
// 	return; 
//     std::vector< QTreeNode4D<Object, Cell>*> NWLine2; 
//     EastOfSouthNode->getNWLine(NWLine2);
//     if(NWLine2.size() == 0)
// 	return; 
// 	 
//     for(int i = 0; i < NWLine1.size(); i++)
// 	for(int j =0; j < NWLine2.size(); j++)
// 	    if(NWLine1[i] == NWLine2[j])
// 		nbrs.push_back(NWLine1[i]); 
// }
// 
// 
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getF_NNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
// { 
//     //get the Front and the North node (equivalent or greater) 
//     QTreeNode4D<Object, Cell>* FrontNode = getFrontNeighborNode();
//     if(FrontNode == NULL)
// 	return; 
//     QTreeNode4D<Object, Cell>* NorthNode = getNorthNeighborNode();
//     if(NorthNode == NULL)
// 	return; 
//  
//     //get the Front of the North 
//     QTreeNode4D<Object, Cell>* FrontOfNorthNode = NorthNode->getFrontNeighborNode();
//     if(FrontOfNorthNode == NULL)
// 	return; 
//     //get the North of the Front 
//     QTreeNode4D<Object, Cell>* NorthOfFrontNode = FrontNode->getNorthNeighborNode();
//     if(NorthOfFrontNode == NULL)
// 	return; 
// 	 
//     //get the south east edges of both nodes 
//     std::vector< QTreeNode4D<Object, Cell>*> B_SLine1; 
//     FrontOfNorthNode->getB_SLine(B_SLine1);
//     if(B_SLine1.size() == 0)
// 	return; 
//     std::vector< QTreeNode4D<Object, Cell>*> B_SLine2; 
//     NorthOfFrontNode->getB_SLine(B_SLine2);
//     if(B_SLine2.size() == 0)
// 	return; 
// 	 
//     for(int i = 0; i < B_SLine1.size(); i++)
// 	for(int j =0; j < B_SLine2.size(); j++)
// 	    if(B_SLine1[i] == B_SLine2[j])
// 		nbrs.push_back(B_SLine1[i]); 
// } 
// 
// 
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getF_SNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
// { 
//     //get the Front and the South node (equivalent or greater) 
//     QTreeNode4D<Object, Cell>* FrontNode = getFrontNeighborNode();
//     if(FrontNode == NULL)
// 	return; 
//     QTreeNode4D<Object, Cell>* SouthNode = getSouthNeighborNode();
//     if(SouthNode == NULL)
// 	return; 
//  
//     //get the Front of the South 
//     QTreeNode4D<Object, Cell>* FrontOfSouthNode = SouthNode->getFrontNeighborNode();
//     if(FrontOfSouthNode == NULL)
// 	return; 
//     //get the South of the Front 
//     QTreeNode4D<Object, Cell>* SouthOfFrontNode = FrontNode->getSouthNeighborNode();
//     if(SouthOfFrontNode == NULL)
// 	return; 
// 	 
//     //get the south east edges of both nodes 
//     std::vector< QTreeNode4D<Object, Cell>*> B_NLine1; 
//     FrontOfSouthNode->getB_NLine(B_NLine1);
//     if(B_NLine1.size() == 0)
// 	return; 
//     std::vector< QTreeNode4D<Object, Cell>*> B_NLine2; 
//     SouthOfFrontNode->getB_NLine(B_NLine2);
//     if(B_NLine2.size() == 0)
// 	return; 
// 	 
//     for(int i = 0; i < B_NLine1.size(); i++)
// 	for(int j =0; j < B_NLine2.size(); j++)
// 	    if(B_NLine1[i] == B_NLine2[j])
// 		nbrs.push_back(B_NLine1[i]); 
// } 
// 
// 
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getF_ENeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
// { 
//     //get the Front and the East node (equivalent or greater) 
//     QTreeNode4D<Object, Cell>* FrontNode = getFrontNeighborNode(); 
//     if(FrontNode == NULL)
// 	return; 
//     QTreeNode4D<Object, Cell>* EastNode = getEastNeighborNode();
//     if(EastNode == NULL)
// 	return; 
//  
//     //get the Front of the East 
//     QTreeNode4D<Object, Cell>* FrontOfEastNode = EastNode->getFrontNeighborNode();
//     if(FrontOfEastNode == NULL)
// 	return; 
//     //get the East of the Front 
//     QTreeNode4D<Object, Cell>* EastOfFrontNode = FrontNode->getEastNeighborNode(); 
//     if(EastOfFrontNode == NULL) 
// 	return; 
// 	 
//     //get the East east edges of both nodes 
//     std::vector< QTreeNode4D<Object, Cell>*> B_WLine1; 
//     FrontOfEastNode->getB_WLine(B_WLine1);
//     if(B_WLine1.size() == 0) 
// 	return; 
//     std::vector< QTreeNode4D<Object, Cell>*> B_WLine2; 
//     EastOfFrontNode->getB_WLine(B_WLine2); 
//     if(B_WLine2.size() == 0)
// 	return; 
// 	 
//     for(int i = 0; i < B_WLine1.size(); i++)
// 	for(int j =0; j < B_WLine2.size(); j++)
// 	    if(B_WLine1[i] == B_WLine2[j])
// 		nbrs.push_back(B_WLine1[i]); 
// } 
// 
// 
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getF_WNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
// { 
//     //get the Front and the West node (equivalent or greater) 
//     QTreeNode4D<Object, Cell>* FrontNode = getFrontNeighborNode();
//     if(FrontNode == NULL) 
// 	return; 
//     QTreeNode4D<Object, Cell>* WestNode = getWestNeighborNode();
//     if(WestNode == NULL) 
// 	return; 
//  
//     //get the Front of the West 
//     QTreeNode4D<Object, Cell>* FrontOfWestNode = WestNode->getFrontNeighborNode();
//     if(FrontOfWestNode == NULL)
// 	return; 
//     //get the West of the Front 
//     QTreeNode4D<Object, Cell>* WestOfFrontNode = FrontNode->getWestNeighborNode();
//     if(WestOfFrontNode == NULL)
// 	return; 
// 	 
//     //get the West West edges of both nodes 
//     std::vector< QTreeNode4D<Object, Cell>*> B_ELine1; 
//     FrontOfWestNode->getB_ELine(B_ELine1);
//     if(B_ELine1.size() == 0)
// 	return; 
//     std::vector< QTreeNode4D<Object, Cell>*> B_ELine2; 
//     WestOfFrontNode->getB_ELine(B_ELine2);
//     if(B_ELine2.size() == 0)
// 	return; 
// 	 
//     for(int i = 0; i < B_ELine1.size(); i++)
// 	for(int j =0; j < B_ELine2.size(); j++)
// 	    if(B_ELine1[i] == B_ELine2[j])
// 		nbrs.push_back(B_ELine1[i]); 
// } 
// 
// 
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getB_NNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
// { 
//     //get the Back and the North node (equivalent or greater) 
//     QTreeNode4D<Object, Cell>* BackNode = getBackNeighborNode();
//     if(BackNode == NULL)
// 	return; 
//     QTreeNode4D<Object, Cell>* NorthNode = getNorthNeighborNode();
//     if(NorthNode == NULL)
// 	return; 
//  
//     //get the Back of the North 
//     QTreeNode4D<Object, Cell>* BackOfNorthNode = NorthNode->getBackNeighborNode();
//     if(BackOfNorthNode == NULL)
// 	return; 
//     //get the North of the Back 
//     QTreeNode4D<Object, Cell>* NorthOfBackNode = BackNode->getNorthNeighborNode(); 
//     if(NorthOfBackNode == NULL) 
// 	return; 
// 	 
//     //get the south east edges of both nodes 
//     std::vector< QTreeNode4D<Object, Cell>*> F_SLine1; 
//     BackOfNorthNode->getF_SLine(F_SLine1);
//     if(F_SLine1.size() == 0)
// 	return; 
//     std::vector< QTreeNode4D<Object, Cell>*> F_SLine2; 
//     NorthOfBackNode->getF_SLine(F_SLine2);
//     if(F_SLine2.size() == 0) 
// 	return; 
// 	 
//     for(int i = 0; i < F_SLine1.size(); i++)
// 	for(int j =0; j < F_SLine2.size(); j++) 
// 	    if(F_SLine1[i] == F_SLine2[j])
// 		nbrs.push_back(F_SLine1[i]); 
// } 
// 
// 
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getB_SNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
// { 
//     //get the Back and the South node (equivalent or greater) 
//     QTreeNode4D<Object, Cell>* BackNode = getBackNeighborNode();
//     if(BackNode == NULL)
// 	return; 
//     QTreeNode4D<Object, Cell>* SouthNode = getSouthNeighborNode();
//     if(SouthNode == NULL)
// 	return; 
//  
//     //get the Back of the South 
//     QTreeNode4D<Object, Cell>* BackOfSouthNode = SouthNode->getBackNeighborNode(); 
//     if(BackOfSouthNode == NULL)
// 	return; 
//     //get the South of the Back 
//     QTreeNode4D<Object, Cell>* SouthOfBackNode = BackNode->getSouthNeighborNode();
//     if(SouthOfBackNode == NULL)
// 	return; 
// 	 
//     //get the south east edges of both nodes 
//     std::vector< QTreeNode4D<Object, Cell>*> F_NLine1; 
//     BackOfSouthNode->getF_NLine(F_NLine1);
//     if(F_NLine1.size() == 0)
// 	return; 
//     std::vector< QTreeNode4D<Object, Cell>*> F_NLine2; 
//     SouthOfBackNode->getF_NLine(F_NLine2);
//     if(F_NLine2.size() == 0)
// 	return; 
// 	 
//     for(int i = 0; i < F_NLine1.size(); i++) 
// 	for(int j =0; j < F_NLine2.size(); j++) 
// 	    if(F_NLine1[i] == F_NLine2[j])
// 		nbrs.push_back(F_NLine1[i]); 
// } 
// 
// 
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getB_ENeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
// { 
//     //get the Back and the East node (equivalent or greater) 
//     QTreeNode4D<Object, Cell>* BackNode = getBackNeighborNode();
//     if(BackNode == NULL)
// 	return; 
//     QTreeNode4D<Object, Cell>* EastNode = getEastNeighborNode();
//     if(EastNode == NULL)
// 	return; 
//  
//     //get the Back of the East 
//     QTreeNode4D<Object, Cell>* BackOfEastNode = EastNode->getBackNeighborNode();
//     if(BackOfEastNode == NULL)
// 	return; 
//     //get the East of the Back 
//     QTreeNode4D<Object, Cell>* EastOfBackNode = BackNode->getEastNeighborNode(); 
//     if(EastOfBackNode == NULL)
// 	return; 
// 	 
//     //get the East east edges of both nodes 
//     std::vector< QTreeNode4D<Object, Cell>*> F_WLine1; 
//     BackOfEastNode->getF_WLine(F_WLine1);
//     if(F_WLine1.size() == 0) 
// 	return; 
//     std::vector< QTreeNode4D<Object, Cell>*> F_WLine2; 
//     EastOfBackNode->getF_WLine(F_WLine2); 
//     if(F_WLine2.size() == 0) 
// 	return; 
// 	 
//     for(int i = 0; i < F_WLine1.size(); i++)
// 	for(int j =0; j < F_WLine2.size(); j++)
// 	    if(F_WLine1[i] == F_WLine2[j])
// 		nbrs.push_back(F_WLine1[i]); 
// } 
// 
// 
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getB_WNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
// { 
//     //get the Back and the West node (equivalent or greater) 
//     QTreeNode4D<Object, Cell>* BackNode = getBackNeighborNode();
//     if(BackNode == NULL)
// 	return; 
//     QTreeNode4D<Object, Cell>* WestNode = getWestNeighborNode();
//     if(WestNode == NULL)
// 	return; 
//  
//     //get the Back of the West 
//     QTreeNode4D<Object, Cell>* BackOfWestNode = WestNode->getBackNeighborNode();
//     if(BackOfWestNode == NULL)
// 	return; 
//     //get the West of the Back 
//     QTreeNode4D<Object, Cell>* WestOfBackNode = BackNode->getWestNeighborNode();
//     if(WestOfBackNode == NULL) 
// 	return; 
// 	 
//     //get the West east edges of both nodes 
//     std::vector< QTreeNode4D<Object, Cell>*> F_ELine1; 
//     BackOfWestNode->getF_ELine(F_ELine1);
//     if(F_ELine1.size() == 0) 
// 	return; 
//     std::vector< QTreeNode4D<Object, Cell>*> F_ELine2; 
//     WestOfBackNode->getF_ELine(F_ELine2); 
//     if(F_ELine2.size() == 0) 
// 	return; 
// 	 
//     for(int i = 0; i < F_ELine1.size(); i++)
// 	for(int j =0; j < F_ELine2.size(); j++)
// 	    if(F_ELine1[i] == F_ELine2[j])
// 		nbrs.push_back(F_ELine1[i]); 
// } 
//  
// /*!********************************************** 
//  *             TRUE CORNER PARTS                * 
//  ************************************************/ 
// /*! This part is to get the 8 boxes each of them  
//   share only one vertex with the current node. */
//  
// /*! Get the most Front NW child of the current node
//   - corner means the corner inside current node. */
// template <class Object, class Cell>
// QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::getF_NWCorner()
// {//return the node on the Front NW corner 
//     if(isLeaf())
// 	return this; 
//     else
// 	return F_NWchild->getF_NWCorner(); 
// } 
// 
// /*! Get the most NE child of the current node. */
// template <class Object, class Cell>
// QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::getF_NECorner()
// { 
//     if(isLeaf())
// 	return this; 
//     else
// 	return F_NEchild->getF_NECorner(); 
// } 
// 
// /*! Get the most SW child of the current node. */ 
// template <class Object, class Cell>
// QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::getF_SWCorner()
// { 
//     if(isLeaf())
// 	return this; 
//     else
// 	return F_SWchild->getF_SWCorner(); 
// }
//  
// /*! Get the most SE child of the current node. */
// template <class Object, class Cell>
// QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::getF_SECorner()
// { 
//     if(isLeaf())
// 	return this; 
//     else
// 	return F_SEchild->getF_SECorner(); 
// } 
//  
// /*! get the most Front NW child of the current node
//   - corner means the corner inside current node. */
// template <class Object, class Cell>
// QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::getB_NWCorner()
// {//return the node on the Front NW corner 
//     if(isLeaf()) 
// 	return this; 
//     else 
// 	return B_NWchild->getB_NWCorner(); 
// }
//  
// /*! Get the most NE child of the current node. */
// template <class Object, class Cell>
// QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::getB_NECorner()
// { 
//     if(isLeaf()) 
// 	return this; 
//     else
// 	return B_NEchild->getB_NECorner(); 
// }
// 
// /*! Get the most SW child of the current node. */
// template <class Object, class Cell>
// QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::getB_SWCorner()
// { 
//     if(isLeaf())
// 	return this; 
//     else
// 	return B_SWchild->getB_SWCorner(); 
// } 
// 
// /*! Get the most SE child of the current node.*/
// template <class Object, class Cell>
// QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::getB_SECorner()
// { 
//     if(isLeaf())
// 	return this; 
//     else
// 	return B_SEchild->getB_SECorner(); 
// } 
//  
// // There are two possibilities :
// 
// /*!**** FRONT **********/ 
// //------ F_NE Corner------------ 
// 
// template <class Object, class Cell>
// QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::F_NENeighbor()
// { 
//     QTreeNode4D<Object, Cell>* NorthNode = getNorthNeighborNode();
//     if(NorthNode == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* EastNode = getEastNeighborNode(); 
//     if(EastNode == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* FrontNode = getFrontNeighborNode();
//     if(FrontNode == NULL) 
// 	return NULL; 
//     //get the north of front, east of north, and front of east 
//     QTreeNode4D<Object, Cell>* NorthOfFront = FrontNode->getNorthNeighborNode();
//     if(NorthOfFront == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* EastOfNorth = NorthNode->getEastNeighborNode(); 
//     if(EastOfNorth == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* FrontOfEast = EastNode->getFrontNeighborNode(); 
//     if(FrontOfEast == NULL) 
// 	return NULL; 
//     //get the corner box 
//     QTreeNode4D<Object, Cell>* EastOfNorthOfFront = NorthOfFront->getEastNeighborNode();
//     if(EastOfNorthOfFront == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* FrontOfEastOfNorth = EastOfNorth->getFrontNeighborNode();
//     if(FrontOfEastOfNorth == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* NorthOfFrontOfEast = FrontOfEast->getNorthNeighborNode();
//     if(NorthOfFrontOfEast == NULL) 
// 	return NULL; 
//     //get the corner of the corner 
//     QTreeNode4D<Object, Cell>* Cor1 = EastOfNorthOfFront->getB_SWCorner();
//     if(Cor1 == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* Cor2 = FrontOfEastOfNorth->getB_SWCorner();
//     if(Cor2 == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* Cor3 = NorthOfFrontOfEast->getB_SWCorner();
//     if(Cor3 == NULL)
// 	return NULL; 
// 
//     if((Cor1 == Cor2) && (Cor2 == Cor3))
// 	return Cor1; 
//     else
// 	return NULL; 
// } 
//  
// //------ F_NW Corner------------ 
// 
// template <class Object, class Cell>
// QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::F_NWNeighbor()
// { 
//     QTreeNode4D<Object, Cell>* NorthNode = getNorthNeighborNode();
//     if(NorthNode == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* WestNode = getWestNeighborNode();
//     if(WestNode == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* FrontNode = getFrontNeighborNode();
//     if(FrontNode == NULL)
// 	return NULL; 
//     //get the north of front, West of north, and front of West 
//     QTreeNode4D<Object, Cell>* NorthOfFront = FrontNode->getNorthNeighborNode();
//     if(NorthOfFront == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* WestOfNorth = NorthNode->getWestNeighborNode();
//     if(WestOfNorth == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* FrontOfWest = WestNode->getFrontNeighborNode();
//     if(FrontOfWest == NULL) 
// 	return NULL; 
//     //get the corner box 
//     QTreeNode4D<Object, Cell>* WestOfNorthOfFront = NorthOfFront->getWestNeighborNode();
//     if(WestOfNorthOfFront == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* FrontOfWestOfNorth = WestOfNorth->getFrontNeighborNode();
//     if(FrontOfWestOfNorth == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* NorthOfFrontOfWest = FrontOfWest->getNorthNeighborNode();
//     if(NorthOfFrontOfWest == NULL)
// 	return NULL; 
//     //get the corner of the corner 
//     QTreeNode4D<Object, Cell>* Cor1 = WestOfNorthOfFront->getB_SECorner();
//     if(Cor1 == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* Cor2 = FrontOfWestOfNorth->getB_SECorner();
//     if(Cor2 == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* Cor3 = NorthOfFrontOfWest->getB_SECorner();
//     if(Cor3 == NULL)
// 	return NULL; 
// 
//     if((Cor1 == Cor2) && (Cor2 == Cor3))
// 	return Cor1; 
//     else 
// 	return NULL; 
// } 
//  
// //------ F_SW Corner------------ 
// 
// template <class Object, class Cell>
// QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::F_SWNeighbor()
// { 
//     QTreeNode4D<Object, Cell>* SouthNode = getSouthNeighborNode();
//     if(SouthNode == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* WestNode = getWestNeighborNode();
//     if(WestNode == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* FrontNode = getFrontNeighborNode();
//     if(FrontNode == NULL) 
// 	return NULL; 
//     //get the South of front, West of South, and front of West 
//     QTreeNode4D<Object, Cell>* SouthOfFront = FrontNode->getSouthNeighborNode(); 
//     if(SouthOfFront == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* WestOfSouth = SouthNode->getWestNeighborNode(); 
//     if(WestOfSouth == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* FrontOfWest = WestNode->getFrontNeighborNode();
//     if(FrontOfWest == NULL) 
// 	return NULL; 
//     //get the corner box 
//     QTreeNode4D<Object, Cell>* WestOfSouthOfFront = SouthOfFront->getWestNeighborNode();
//     if(WestOfSouthOfFront == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* FrontOfWestOfSouth = WestOfSouth->getFrontNeighborNode(); 
//     if(FrontOfWestOfSouth == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* SouthOfFrontOfWest = FrontOfWest->getSouthNeighborNode();
//     if(SouthOfFrontOfWest == NULL) 
// 	return NULL; 
//     //get the corner of the corner 
//     QTreeNode4D<Object, Cell>* Cor1 = WestOfSouthOfFront->getB_NECorner();
//     if(Cor1 == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* Cor2 = FrontOfWestOfSouth->getB_NECorner();
//     if(Cor2 == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* Cor3 = SouthOfFrontOfWest->getB_NECorner();
//     if(Cor3 == NULL) 
// 	return NULL; 
//   
//     if((Cor1 == Cor2) && (Cor2 == Cor3))
// 	return Cor1; 
//     else
// 	return NULL; 
// } 
//  
// //------ F_SE Corner------------ 
// 
// template <class Object, class Cell>
// QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::F_SENeighbor()
// { 
//     QTreeNode4D<Object, Cell>* SouthNode = getSouthNeighborNode();
//     if(SouthNode == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* EastNode = getEastNeighborNode(); 
//     if(EastNode == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* FrontNode = getFrontNeighborNode(); 
//     if(FrontNode == NULL)
// 	return NULL; 
//     //get the South of front, East of South, and front of East 
//     QTreeNode4D<Object, Cell>* SouthOfFront = FrontNode->getSouthNeighborNode();
//     if(SouthOfFront == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* EastOfSouth = SouthNode->getEastNeighborNode(); 
//     if(EastOfSouth == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* FrontOfEast = EastNode->getFrontNeighborNode(); 
//     if(FrontOfEast == NULL)
// 	return NULL; 
//     //get the corner box 
//     QTreeNode4D<Object, Cell>* EastOfSouthOfFront = SouthOfFront->getEastNeighborNode();
//     if(EastOfSouthOfFront == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* FrontOfEastOfSouth = EastOfSouth->getFrontNeighborNode();
//     if(FrontOfEastOfSouth == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* SouthOfFrontOfEast = FrontOfEast->getSouthNeighborNode(); 
//     if(SouthOfFrontOfEast == NULL) 
// 	return NULL; 
//     //get the corner of the corner 
//     QTreeNode4D<Object, Cell>* Cor1 = EastOfSouthOfFront->getB_NWCorner();
//     if(Cor1 == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* Cor2 = FrontOfEastOfSouth->getB_NWCorner();
//     if(Cor2 == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* Cor3 = SouthOfFrontOfEast->getB_NWCorner();
//     if(Cor3 == NULL)
// 	return NULL; 
//    
//     if((Cor1 == Cor2) && (Cor2 == Cor3))
// 	return Cor1; 
//     else 
// 	return NULL; 
// } 
//  
// /*!**** BACK **********/ 
// //------ B_NE Corner------------ 
// 
// template <class Object, class Cell>
// QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::B_NENeighbor()
// { 
//     QTreeNode4D<Object, Cell>* NorthNode = getNorthNeighborNode(); 
//     if(NorthNode == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* EastNode = getEastNeighborNode();
//     if(EastNode == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* BackNode = getBackNeighborNode();
//     if(BackNode == NULL) 
// 	return NULL; 
//     //get the north of Back, east of north, and Back of east 
//     QTreeNode4D<Object, Cell>* NorthOfBack = BackNode->getNorthNeighborNode();
//     if(NorthOfBack == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* EastOfNorth = NorthNode->getEastNeighborNode();
//     if(EastOfNorth == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* BackOfEast = EastNode->getBackNeighborNode(); 
//     if(BackOfEast == NULL) 
// 	return NULL; 
//     //get the corner box 
//     QTreeNode4D<Object, Cell>* EastOfNorthOfBack = NorthOfBack->getEastNeighborNode();
//     if(EastOfNorthOfBack == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* BackOfEastOfNorth = EastOfNorth->getBackNeighborNode();
//     if(BackOfEastOfNorth == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* NorthOfBackOfEast = BackOfEast->getNorthNeighborNode();
//     if(NorthOfBackOfEast == NULL) 
// 	return NULL; 
//     //get the corner of the corner 
//     QTreeNode4D<Object, Cell>* Cor1 = EastOfNorthOfBack->getF_SWCorner();
//     if(Cor1 == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* Cor2 = BackOfEastOfNorth->getF_SWCorner();
//     if(Cor2 == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* Cor3 = NorthOfBackOfEast->getF_SWCorner();
//     if(Cor3 == NULL) 
// 	return NULL; 
//   
//     if((Cor1 == Cor2) && (Cor2 == Cor3))
// 	return Cor1; 
//     else
// 	return NULL; 
// } 
//  
// //------ B_NW Corner------------ 
// 
// template <class Object, class Cell>
// QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::B_NWNeighbor()
// { 
//     QTreeNode4D<Object, Cell>* NorthNode = getNorthNeighborNode(); 
//     if(NorthNode == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* WestNode = getWestNeighborNode();
//     if(WestNode == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* BackNode = getBackNeighborNode();
//     if(BackNode == NULL)
// 	return NULL; 
//     //get the north of Back, West of north, and Back of West 
//     QTreeNode4D<Object, Cell>* NorthOfBack = BackNode->getNorthNeighborNode();
//     if(NorthOfBack == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* WestOfNorth = NorthNode->getWestNeighborNode();
//     if(WestOfNorth == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* BackOfWest = WestNode->getBackNeighborNode(); 
//     if(BackOfWest == NULL) 
// 	return NULL; 
//     //get the corner box 
//     QTreeNode4D<Object, Cell>* WestOfNorthOfBack = NorthOfBack->getWestNeighborNode();
//     if(WestOfNorthOfBack == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* BackOfWestOfNorth = WestOfNorth->getBackNeighborNode();
//     if(BackOfWestOfNorth == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* NorthOfBackOfWest = BackOfWest->getNorthNeighborNode(); 
//     if(NorthOfBackOfWest == NULL) 
// 	return NULL; 
//     //get the corner of the corner 
//     QTreeNode4D<Object, Cell>* Cor1 = WestOfNorthOfBack->getF_SECorner();
//     if(Cor1 == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* Cor2 = BackOfWestOfNorth->getF_SECorner();
//     if(Cor2 == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* Cor3 = NorthOfBackOfWest->getF_SECorner();
//     if(Cor3 == NULL)
// 	return NULL; 
//   
//     if((Cor1 == Cor2) && (Cor2 == Cor3))
// 	return Cor1; 
//     else 
// 	return NULL; 
// } 
// 
// //------ B_SW Corner------------ 
// 
// template <class Object, class Cell>
// QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::B_SWNeighbor()
// { 
//     QTreeNode4D<Object, Cell>* SouthNode = getSouthNeighborNode();
//     if(SouthNode == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* WestNode = getWestNeighborNode();
//     if(WestNode == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* BackNode = getBackNeighborNode(); 
//     if(BackNode == NULL)
// 	return NULL; 
//     //get the South of Back, West of South, and Back of West 
//     QTreeNode4D<Object, Cell>* SouthOfBack = BackNode->getSouthNeighborNode();
//     if(SouthOfBack == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* WestOfSouth = SouthNode->getWestNeighborNode();
//     if(WestOfSouth == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* BackOfWest = WestNode->getBackNeighborNode();
//     if(BackOfWest == NULL)
// 	return NULL; 
//     //get the corner box 
//     QTreeNode4D<Object, Cell>* WestOfSouthOfBack = SouthOfBack->getWestNeighborNode();
//     if(WestOfSouthOfBack == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* BackOfWestOfSouth = WestOfSouth->getBackNeighborNode();
//     if(BackOfWestOfSouth == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* SouthOfBackOfWest = BackOfWest->getSouthNeighborNode(); 
//     if(SouthOfBackOfWest == NULL)
// 	return NULL; 
//     //get the corner of the corner 
//     QTreeNode4D<Object, Cell>* Cor1 = WestOfSouthOfBack->getF_NECorner();
//     if(Cor1 == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* Cor2 = BackOfWestOfSouth->getF_NECorner();
//     if(Cor2 == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* Cor3 = SouthOfBackOfWest->getF_NECorner();
//     if(Cor3 == NULL) 
// 	return NULL; 
//   
//     if((Cor1 == Cor2) && (Cor2 == Cor3))
// 	return Cor1; 
//     else 
// 	return NULL; 
// } 
//  
// //------ B_SE Corner------------ 
// 
// template <class Object, class Cell>
// QTreeNode4D<Object, Cell>* QTreeNode4D<Object, Cell>::B_SENeighbor()
// { 
//     QTreeNode4D<Object, Cell>* SouthNode = getSouthNeighborNode();
//     if(SouthNode == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* EastNode = getEastNeighborNode();
//     if(EastNode == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* BackNode = getBackNeighborNode(); 
//     if(BackNode == NULL)
// 	return NULL; 
//     //get the South of Back, East of South, and Back of East 
//     QTreeNode4D<Object, Cell>* SouthOfBack = BackNode->getSouthNeighborNode();
//     if(SouthOfBack == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* EastOfSouth = SouthNode->getEastNeighborNode();
//     if(EastOfSouth == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* BackOfEast = EastNode->getBackNeighborNode();
//     if(BackOfEast == NULL)
// 	return NULL; 
//     //get the corner box 
//     QTreeNode4D<Object, Cell>* EastOfSouthOfBack = SouthOfBack->getEastNeighborNode();
//     if(EastOfSouthOfBack == NULL) 
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* BackOfEastOfSouth = EastOfSouth->getBackNeighborNode();
//     if(BackOfEastOfSouth == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* SouthOfBackOfEast = BackOfEast->getSouthNeighborNode();
//     if(SouthOfBackOfEast == NULL) 
// 	return NULL; 
//     //get the corner of the corner 
//     QTreeNode4D<Object, Cell>* Cor1 = EastOfSouthOfBack->getF_NWCorner();
//     if(Cor1 == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* Cor2 = BackOfEastOfSouth->getF_NWCorner();
//     if(Cor2 == NULL)
// 	return NULL; 
//     QTreeNode4D<Object, Cell>* Cor3 = SouthOfBackOfEast->getF_NWCorner();
//     if(Cor3 == NULL)
// 	return NULL; 
//   
//     if((Cor1 == Cor2) && (Cor2 == Cor3))
// 	return Cor1; 
//     else
// 	return NULL; 
// } 
//  
// /************************************************** 
//  *        THE TOP FUNCTION in Neighbor finding    * 
//  **************************************************/ 
// 
// /*! 
//   PURPOSES: get all the neighbor of this node; 
//   RECEIVES:  
//   RETURNS:  the list of neighbor store in the vector nbrs (vector of pointers)	 
//   REMARKS: 
//   USING THIS FUNCTION to get all the neighbor of  
//   the current node, including S, W, N , E neighbor  
//   as well as NW, SE, SW,... (exterior corners) 
// */ 
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getAllNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
// { 
//     // get the neighbors on the four due directions 
//     getAllSouthNeighbors(nbrs); 
//     getAllNorthNeighbors(nbrs); 
//     getAllWestNeighbors(nbrs); 
//     getAllEastNeighbors(nbrs); 
//     getAllFrontNeighbors(nbrs); 
//     getAllBackNeighbors(nbrs); 
// 
//     //get the nbr on the edges 
//     getNWNeighbors(nbrs); 
//     getNENeighbors(nbrs); 
//     getSWNeighbors(nbrs); 
//     getSENeighbors(nbrs); 
//     getF_NNeighbors(nbrs); 
//     getF_SNeighbors(nbrs); 
//     getF_ENeighbors(nbrs); 
//     getF_WNeighbors(nbrs); 
//     getB_NNeighbors(nbrs); 
//     getB_SNeighbors(nbrs); 
//     getB_ENeighbors(nbrs); 
//     getB_WNeighbors(nbrs); 
// 	 
//     //get the neighbor on the corner 
//     QTreeNode4D<Object, Cell>* F_NE = F_NENeighbor();//------ NE Corner------------ 
//     if(F_NE != NULL) 
// 	nbrs.push_back(F_NE); 
//     QTreeNode4D<Object, Cell>* F_NW = F_NWNeighbor(); 
//     if(F_NW != NULL)
// 	nbrs.push_back(F_NW); 
//     QTreeNode4D<Object, Cell>* F_SW = F_SWNeighbor(); 
//     if(F_SW != NULL)
// 	nbrs.push_back(F_SW); 
//     QTreeNode4D<Object, Cell>* F_SE = F_SENeighbor(); 
//     if(F_SE != NULL)
// 	nbrs.push_back(F_SE); 
//     QTreeNode4D<Object, Cell>* B_NE = B_NENeighbor(); 
//     if(B_NE != NULL)
// 	nbrs.push_back(B_NE); 
//     QTreeNode4D<Object, Cell>* B_NW = B_NWNeighbor(); 
//     if(B_NW != NULL)
// 	nbrs.push_back(B_NW); 
//     QTreeNode4D<Object, Cell>* B_SW = B_SWNeighbor(); 
//     if(B_SW != NULL) 
// 	nbrs.push_back(B_SW); 
//     QTreeNode4D<Object, Cell>* B_SE = B_SENeighbor(); 
//     if(B_SE != NULL)
// 	nbrs.push_back(B_SE); 
// } 
// 
// /*! Get the neighbors on the four due directions. */
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getAllFaceNeighbors(std::vector< QTreeNode4D<Object, Cell>*>& nbrs)
// { 
//     getAllSouthNeighbors( nbrs );
//     getAllNorthNeighbors( nbrs );
//     getAllWestNeighbors ( nbrs );
//     getAllEastNeighbors ( nbrs );
//     getAllFrontNeighbors( nbrs );
//     getAllBackNeighbors ( nbrs );
// }
// 
// /*! Get the neighbors on the edges. */
// template <class Object, class Cell>
// void QTreeNode4D<Object, Cell>::getAllEdgeNeighbors(std::vector< QTreeNode4D<Object, Cell> * > &nbrs) 
// {
//     getNWNeighbors ( nbrs );
//     getNENeighbors ( nbrs );
//     getSWNeighbors ( nbrs );
//     getSENeighbors ( nbrs );
//     getF_NNeighbors( nbrs );
//     getF_SNeighbors( nbrs );
//     getF_ENeighbors( nbrs );
//     getF_WNeighbors( nbrs );
//     getB_NNeighbors( nbrs );
//     getB_SNeighbors( nbrs );
//     getB_ENeighbors( nbrs );
//     getB_WNeighbors( nbrs );
// }
// 
// /*! Return true if the current node is smaller than its neighbors,
//   forgery if not (it is has to say has a major level in the octree). */
// template <class Object, class Cell>
// bool QTreeNode4D<Object, Cell>::isSmallerThanNeighbors( std::vector< QTreeNode4D<Object, Cell> * > &nbrs )
// {  
//     // If a node has no neighbors.
//     if ( nbrs.empty() )
// 	return true;
//   
//     // Si un voisin est plus petit.
//     for ( int i = 0 ; i < nbrs.size() ; i++ )
// 	if ( nbrs[i]->depth > depth) 
// 	    return false;
// 
//     return true;
// }
