/*****************************************************************************
 * A x e l
 *****************************************************************************
 * ArrangementDock
 * 2006-11-17
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# ifndef ARRANGEMENTDOCK_H
# define ARRANGEMENTDOCK_H

# include "ui_AlgorithmControlWidget.h"
# include "ui_ArrangementQueriesControlWidget.h"

# include <QDockWidget>
# include <QData/QGraph.h>
# include <QArrangement/Curves/QImplicitCurvesArrangement.h>

using namespace std ;
using namespace QAxel ;
using namespace QData ;
using namespace QArrangement ;

class MainWindow ;
class QHBoxLayout ;
class QSplitter ;
class QToolBox ;
class QWidget ;
class QGraphWidget ;
class QLabel ;

class AlgorithmControlWidget : public QWidget
{
    Q_OBJECT
    
public:
    AlgorithmControlWidget(QWidget * parent = 0) : QWidget(parent) {
	ui.setupUi(this);
	connect(ui.playButton, SIGNAL(clicked()), this, SLOT(emitPlaySignal())) ;
	connect(ui.pauseButton, SIGNAL(clicked()), this, SLOT(emitPauseSignal())) ;
	connect(ui.stopButton, SIGNAL(clicked()), this, SLOT(emitStopSignal())) ;
	connect(ui.cellVerbosityCheckBox, SIGNAL(stateChanged(int)), this, SLOT(emitCellVerbosityModeChanged(int))) ;
    }
    
    double multiplicityFilteringRadius(void) { return ui.multipleRootsFilteringRadiusBox->value() ; }
    double isoleSolverPrecision(void) { return ui.isoleSolverPrecisionBox->value() ; }
    double sbdSolverPrecision(void) { return ui.sbdSolverPrecisionBox->value() ; }
    double cellPrecision(void) { return ui.minCellBox->value() ; }
    double cellPrecisionMax(void) { return ui.maxCellBox->value() ; }
    bool cellVerbosity(void) { return ui.cellVerbosityCheckBox->checkState() == Qt::Checked ; }
    bool fusionVerbosity(void) { return ui.fusionVerbosityCheckBox->checkState() == Qt::Checked ; }
    
public slots:
    void emitPlaySignal(void)  { emit play() ; }
    void emitPauseSignal(void) { emit pause() ; }
    void emitStopSignal(void)  { emit stop() ; }
    
    void emitCellVerbosityModeChanged(int state) {
	switch(state) {
	case Qt::Unchecked:
	    emit hideSubdivisionCells() ;
	    break ;
	case Qt::Checked:
       	    emit showSubdivisionCells() ;
	    break ;
	default:
	    break ;
	}
    }
    
public: signals:
    void play(void) ;
    void pause(void) ;
    void stop(void) ;
    void showSubdivisionCells(void) ;
    void hideSubdivisionCells(void) ;
    
private:
    Ui::AlgorithmControlWidget ui ;
} ;

class ArrangementQueriesControlWidget : public QWidget
{
    Q_OBJECT

public:
    ArrangementQueriesControlWidget(QWidget * parent = 0) : QWidget(parent) {
	ui.setupUi(this);
	connect(ui.clickLocationCheckBox, SIGNAL(stateChanged(int)), this, SLOT(emitClickLocationMode(int))) ;
	connect(ui.clearButton, SIGNAL(clicked()), this, SLOT(emitClearLocation())) ;
    }

public: signals:
    void enterClickLocationMode(void) ;
    void leaveClickLocationMode(void) ;
    void clearLocation(void) ;

public slots:
    void emitClickLocationMode(int state) {
	switch(state) {
	case Qt::Unchecked:
	    emit leaveClickLocationMode() ;
	    break ;
	case Qt::Checked:
       	    emit enterClickLocationMode() ;
	    break ;
	default:
	    break ;
	}
    }

    void emitClearLocation(void) {
	emit clearLocation() ;
    }

private:
    Ui::ArrangementQueriesControlWidget ui ;
} ;

class ArrangementDock : public QDockWidget 
{
    Q_OBJECT       
    
public:
    ArrangementDock(const QString & title, QWidget * parent) ;
							     
public slots:
    void enterClickLocationMode(void) ;
    void leaveClickLocationMode(void) ;
    void clickLocation(const QPoint & p) ;
    void clearLocation(void) ;
    void showSubdivisionCells(void) ;
    void hideSubdivisionCells(void) ;

private:
    QImplicitCurvesArrangement * algo ;
				      
private slots:
    void play(void) ;
    void pause(void) ;
    void stop(void) ;
    
private:
    void setupUi(void) ;
    
private :
    MainWindow * mainwindow ;
    AlgorithmControlWidget * acw ;
    ArrangementQueriesControlWidget * aqw ;
    QHBoxLayout *  hboxLayout ;
    QSplitter * splitter ;
    QToolBox *toolBox ;
    QWidget *page1, *page2, *page3 ;
    QGraphWidget *graphWidget ;
    QGraphWidget *quad3Widget ;
    QLabel * deamonPixmap ;

    QList<QPoint3f *> locatedPoints ;
} ;

# endif // ARRANGEMENTDOCK_H
