/****************************************************************************
**
** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
**
** This file is part of the QtGui module of the Qt Toolkit.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** sales department at sales@trolltech.com.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/
#include <qgl.h>
#include <qglpixelbuffer.h>
#include <qpainter.h>
#include <qimage.h>
#include <qevent.h>

class GLSelectorView : public QWidget
{
    QGLPixelBuffer *pbuf;
    QGLWidget *dummy;

public:
    GLSelectorView(QWidget *parent, Qt::WindowFlags f = 0)
        : QWidget(parent, f)
    {
        dummy = new QGLWidget((QWidget *) 0);
        QGLFormat format = QGLFormat::defaultFormat();
        format.setSampleBuffers(true);
        format.setAlphaBufferSize(8);
        pbuf = new QGLPixelBuffer(width(), height(), format, dummy);
        pbuf->makeCurrent();
    }

    ~GLSelectorView()
    {
        delete pbuf;
        delete dummy;
    }

    GLuint bindTexture(const QImage &image, GLenum target = GL_TEXTURE_2D) {
        Q_ASSERT(pbuf);
        return dummy->bindTexture(image, target);
    }

    void deleteTexture(GLuint texture_id) {
        Q_ASSERT(pbuf);
        dummy->deleteTexture(texture_id);
    }

    bool event(QEvent *e) {
        if (e->type() == QEvent::Resize) {
            QResizeEvent *re = (QResizeEvent *) e;
            QGLFormat format = pbuf->format();
#ifndef Q_WS_MAC
            delete pbuf;
#endif
            pbuf = new QGLPixelBuffer(re->size(), format, dummy);
            return true;
        } else if (e->type() == QEvent::Paint) {
            Q_ASSERT(pbuf);
            pbuf->makeCurrent();
            // handle background filling
            if (autoFillBackground()) {
                // NB! we use the premultiplied format internally in
                // QOpenGLPaintEngine
                QColor bg = palette().color(backgroundRole());
                glClearColor(bg.redF()*bg.alphaF(),
                             bg.greenF()*bg.alphaF(),
                             bg.blueF()*bg.alphaF(),
                             bg.alphaF());
            } else {
                glClearColor(0, 0, 0, 0);
            }
            glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

            // redirect this widget's painting into the pbuffer
            QPainter::setRedirected(this, pbuf);
            paintEvent((QPaintEvent *) e);
            QPainter::restoreRedirected(this);

            // draw the pbuffer contents to the backingstore
            QImage image = pbuf->toImage();
            QPainter p(this);
            p.drawImage(0, 0, image);
            p.end();
            return true;
        }
        return QWidget::event(e);
    }
};
