/*****************************************************************************
 * A x e l
 *****************************************************************************
 * ParametricCurvesArrangementDock
 * 2007-06-08
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 ****************************************************************************/

# include <iostream>

# include <QtGui/QButtonGroup>
# include <QtGui/QSplitter>
# include <QtGui/QToolBox>
# include <QtGui/QHBoxLayout>
# include <QtGui/QWidget>
# include <QtGui/QGroupBox>
# include <QtGui/QLabel>

# include <QGui/MainWindow.h>
# include <QGui/Viewer.h>
# include <QGui/QGraphWidget.h>
# include <QGui/QAlgorithmWidget.h>
# include <QGui/ParametricCurvesArrangementDock.h>

# include <QData/QGraph.h>

# include <QArrangement/QArrangement.h>

# include <QGLViewer/qglviewer.h>

ParametricCurvesArrangementDock::ParametricCurvesArrangementDock(const QString & title, QWidget * parent) : QDockWidget(title, parent) 
{
    mainwindow = dynamic_cast<MainWindow *>(parent) ;
    setObjectName("ParametricCurvesArrangementDock") ;
    setupUi() ;
}

void ParametricCurvesArrangementDock::play()
{
    mainwindow->algorithmWidget()->addElement("Arrangement", ":images/16x16/arrangementDeamon.png", "Arrangement deamon running") ;
	
    double multiFiltering   = acw->multiplicityFilteringRadius() ;
    double isolePrecision   = acw->isoleSolverPrecision() ;
    double sbdPrecision     = acw->sbdSolverPrecision() ;
    double cellPrecision    = acw->cellPrecision() ;
    double cellPrecisionMax = acw->cellPrecisionMax() ;
	
    algo = new QParametricCurvesArrangement(mainwindow) ;
    algo->setGraphicalCellsVerbosity(acw->cellVerbosity()) ;
    algo->setGraphicalFusionVerbosity(acw->fusionVerbosity()) ;
    graphWidget->setGraph(algo->graph()) ;

    connect(algo->graph(),        SIGNAL(changed()),                     graphWidget, SLOT(update())) ;
    connect(algo,                 SIGNAL(display(QGraph *)),             quad3Widget, SLOT(setGraph(QGraph *))) ;
    connect(mainwindow->viewer(), SIGNAL(objectSelected(QAxelObject *)), algo,        SLOT(setTree(QAxelObject *))) ;
    connect(mainwindow->viewer(), SIGNAL(noObjectSelected()),            quad3Widget, SLOT(clear())) ;
# ifdef Q_WS_MAC
    connect(mainwindow->viewer(), SIGNAL(objectAdded(QAxelObject *)),    qobject_cast<QArrangement::QArrangement *>(algo), SLOT(addObject(QAxelObject *))) ;
# else
    connect(mainwindow->viewer(), SIGNAL(objectAdded(QAxelObject *)),    algo,        SLOT(addObject(QAxelObject *))) ;
# endif

    // if(!algo->isRunning()) 	algo->start() ;
    // algo->play() ;
    algo->run() ;
}

void ParametricCurvesArrangementDock::pause()
{
    mainwindow->algorithmWidget()->setPixmap("Arrangement", ":images/16x16/pausedDeamon.png") ;
    mainwindow->algorithmWidget()->setToolTip("Arrangement", "Arrangement deamon running") ;
    // algo->pause() ;
}

void ParametricCurvesArrangementDock::stop()
{
    mainwindow->algorithmWidget()->removeElement("Arrangement") ;
    // algo->stop() ;
    // algo->wait() ;
}

void ParametricCurvesArrangementDock::setupUi(void)
{
    splitter = new QSplitter(this) ;
    splitter->setOrientation(Qt::Horizontal) ;

    scrollArea = new QScrollArea(this) ;
    scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded) ;
    scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded) ;
    scrollArea->setWidgetResizable(true) ;
    acw = new ParametricCurvesArrangementWidget(scrollArea->viewport()) ;
    scrollArea->setWidget(acw) ;

    graphWidget = new QGraphWidget(this) ; graphWidget->setViewer(mainwindow->viewer()) ;
    quad3Widget = new QGraphWidget(this) ; quad3Widget->setViewer(mainwindow->viewer()) ;
	
    splitter->addWidget(scrollArea) ;
    splitter->addWidget(graphWidget) ;
    splitter->addWidget(quad3Widget) ;
    
    connect(acw, SIGNAL(play()), this, SLOT(play())) ;
    connect(acw, SIGNAL(pause()), this, SLOT(pause())) ;
    connect(acw, SIGNAL(stop()), this, SLOT(stop())) ;
    connect(acw, SIGNAL(showSubdivisionCells()), this, SLOT(showSubdivisionCells())) ;
    connect(acw, SIGNAL(hideSubdivisionCells()), this, SLOT(hideSubdivisionCells())) ;
    connect(acw, SIGNAL(enterClickLocationMode()), this, SLOT(enterClickLocationMode())) ;
    connect(acw, SIGNAL(leaveClickLocationMode()), this, SLOT(leaveClickLocationMode())) ;
    connect(acw, SIGNAL(clearLocation()), this, SLOT(clearLocation())) ;

    setWidget(splitter) ;
}

void ParametricCurvesArrangementDock::enterClickLocationMode(void)
{
    connect(mainwindow->viewer(), SIGNAL(pointClicked(const QPoint &)), this, SLOT(clickLocation(const QPoint &))) ;
    mainwindow->viewer()->setOrthographicDisplay() ;
}

void ParametricCurvesArrangementDock::leaveClickLocationMode(void)
{
    disconnect(mainwindow->viewer(), SIGNAL(pointClicked(const QPoint &)), this, SLOT(clickLocation(const QPoint &))) ;
    mainwindow->viewer()->setPerspectiveDisplay() ;
}

void ParametricCurvesArrangementDock::clickLocation(const QPoint & p)
{
    using qglviewer::Camera ;
    using qglviewer::Vec ;

    Viewer * viewer = mainwindow->viewer() ;
    Camera * camera = mainwindow->viewer()->camera() ;
    
    Vec orig, dir ;
    camera->convertClickToLine(p, orig, dir) ;
    
    Vec poi = orig + camera->position().norm() * dir ;

    QPoint3f * point = new QPoint3f(poi.x, poi.y, 0.0) ;
    
    viewer->addObject(point) ; locatedPoints << point ;
    
    algo->locate(point) ;
}

void ParametricCurvesArrangementDock::clearLocation(void)
{
    foreach(QPoint3f * p, locatedPoints)
	mainwindow->viewer()->removeObject(p) ;    
    locatedPoints.clear() ;
    algo->graph()->resetColors() ;
}

void ParametricCurvesArrangementDock::showSubdivisionCells(void)
{
    QList<QAxelObject *> cells = mainwindow->viewer()->listObjectsFromType("Cell") ;
    foreach(QAxelObject * o, cells)
	o->setHidden(false) ;
    mainwindow->viewer()->update() ;
}

void ParametricCurvesArrangementDock::hideSubdivisionCells(void)
{
    QList<QAxelObject *> cells = mainwindow->viewer()->listObjectsFromType("Cell") ;
    foreach(QAxelObject * o, cells)
	o->setHidden(true) ;
    mainwindow->viewer()->update() ;
}
