/****************************************************************************
**
** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
**
** This file is part of the QtGui module of the Qt Toolkit.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** sales department at sales@trolltech.com.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/
/*
#include <iostream>
#include <qdebug.h>
#include <math.h>
#include <QtNetwork>
#include <QtGui>

#include <QGui/MainWindow.h>
#include <QGui/SampleSelector.h>

using namespace std ;

GLSampleSelector::GLSampleSelector(QWidget *parent)
    : GLSelectorView(parent)
{
    carousel_rot = 0.0f;
    y_rot = -3.0f;
    scale = 1.0f;

    mouse_down = false;

    select_anim = false;
    anim_scale = 1.0f;
    anim_alpha = 1.0f;

    zoom_in = false;
    zoom_out = false;
    zoomed_in = false;

    logo = bindTexture(QImage(":images/128x128/fileopen.png"));
    current_image = logo;

    http = new QHttp(this);
    connect(http, SIGNAL(requestFinished(int, bool)), SLOT(processResults(int, bool)));

    startTimer(50);
}

GLSampleSelector::~GLSampleSelector()
{
    for (int i=0; i<textures.size(); ++i)
        deleteTexture(textures.at(i));
    delete http;
}

void GLSampleSelector::timerEvent(QTimerEvent *)
{
    if (!mouse_down) {
        carousel_rot += 0.5;
        update();
    }
    if (zoom_in) {
        scale += scale*0.1f;
        if (scale >= 2.4f) {
            scale = 2.4f;
            zoom_in = false;
            zoomed_in = true;
        }
    }
    if (zoom_out) {
        scale -= scale*0.1f;
        if (scale <= 1.0f) {
            scale = 1.0f;
            zoom_out = false;
            zoomed_in = false;
        }
    }
}

void GLSampleSelector::mousePressEvent(QMouseEvent *e)
{
    mouse_down = true;
    current_at_press = current_blade;
    anchor = e->pos();
}

void GLSampleSelector::mouseReleaseEvent(QMouseEvent *e)
{
    mouse_down = false;
    if (e->button() == Qt::LeftButton) {
        if (textures.size() > 0 && current_blade < textures.size()) {
            if (current_blade != -1) {
                current_image = textures.at(current_blade);
//# ifdef Q_WS_MAC
//		emit fileSelected(images.at(current_image-2)) ;
		emit fileSelected(map[current_image]) ;
//# else
//		emit fileSelected(images.at(current_image-1)) ;
//# endif
            } else
                current_blade = logo;
            if (current_blade != -1 && current_at_press == current_blade) {
                select_anim = true;
                if (zoomed_in)
                    zoom_out = true;
            }
        }
    } else if (e->button() == Qt::RightButton) {
        if (!zoom_out && !zoom_in) {
            if (zoomed_in)
                zoom_out = true;
            else
                zoom_in = true;
        }
    }
}

void GLSampleSelector::mouseMoveEvent(QMouseEvent *e)
{
    QPoint diff = e->pos() - anchor;
    if (e->buttons() & Qt::LeftButton)
        y_rot += diff.x()/5.0f;
    anchor = e->pos();
    update();
}

int GLSampleSelector::bladeAtPos(const QPoint &p)
{
    const int max_buf = 512;
    GLuint buffer[max_buf];
    GLint viewport[4];
    glGetIntegerv(GL_VIEWPORT, viewport);
    glSelectBuffer(max_buf, buffer);
    glRenderMode(GL_SELECT);

    glInitNames();
    glPushName(-1);

    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    gluPickMatrix(GLdouble(p.x()), GLdouble(viewport[3] - p.y()), 1.0, 1.0, viewport);

    drawBlades(true);
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();

    int records = glRenderMode(GL_RENDER);
    int blade = -1;
    GLuint min_z = 0xffffffff;
    if (records > 1) {
        for (int i=0; i<records; ++i) {
            // check for the min z coordinate on multiple hit records
            if (min_z > (buffer[i*4+1])) {
                min_z = (buffer[i*4+1]);
                blade = buffer[i*4+3];
            }
        }
    } else if (records > 0) {
        blade = buffer[3];
    }
    return blade;
}

void GLSampleSelector::wheelEvent(QWheelEvent *e)
{
    e->delta() > 0 ? scale += scale*0.2f : scale -= scale*0.2f;
    if (scale > 2.4f)
        scale = 2.4f;
    else if (scale < 1.0f)
        scale = 1.0f;
    update();
}

void GLSampleSelector::paintEvent(QPaintEvent *)
{
    // clear is done before paintEvent() is called
    glViewport(0, 0, width(), height());
    glColor4f(1.0f, 1.0, 1.0f, 1.0f);
    glLineWidth(2.0f);

    glEnable(GL_BLEND);
    glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glPushMatrix();
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glPushMatrix();

    glDisable(GL_DEPTH_TEST);
    glDisable(GL_TEXTURE_2D);
    glBegin(GL_QUADS);
    {
        glColor4f(0.0f, 0.0f, 0.0f, 0.0f);
        glVertex2f(1.0f, 0.5f);
        glVertex2f(-1.0f, 0.5f);

        glColor4f(0.68f, 0.68f, 1.0f, 1.0f);
        glVertex2f(-1.0f, -1.0f);
        glVertex2f(1.0f, -1.0f);
    }
    glEnd();

    glEnable(GL_DEPTH_TEST);
    glEnable(GL_TEXTURE_2D);

    current_blade = bladeAtPos(mapFromGlobal(QCursor::pos()));

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    drawBlades();

    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
    glPopMatrix();

    glDisable(GL_DEPTH_TEST);
    GLfloat sw, sh;

    if (width() > height()) {
        sw = 1.0f;
        sh = width()/GLfloat(height());
    } else {
        sw = height()/GLfloat(width());
        sh = 1.0f;
    }

    glTranslatef(-0.3f*scale - scale/8.0f, -0.1f*scale - scale/8.0f, 0.0f);
    glScalef(0.5f*sw, 0.5f*sh, 0.5f);

    GLfloat alpha = 1.0f - (scale-1.0f)/1.4f;
    glDisable(GL_TEXTURE_2D);
    glColor4f(0.5f*alpha, 0.5f*alpha, 0.5f*alpha, 0.5f*alpha);
    glRectf(-1.1f, -1.1f, 1.1f, 1.1f);

    // Drawing current image
    glEnable(GL_TEXTURE_2D);
    glColor4f(1.0f*alpha, 1.0f*alpha, 1.0f*alpha, 1.0f*alpha);
    glBindTexture(GL_TEXTURE_2D, current_image);
    glBegin(GL_QUADS);
    {
        glTexCoord2f(0.0f, 0.0f); glVertex2f(-1.0f,-1.0f);
        glTexCoord2f(0.0f, 1.0f); glVertex2f(-1.0f,1.0f);
        glTexCoord2f(1.0f, 1.0f); glVertex2f(1.0f,1.0f);
        glTexCoord2f(1.0f, 0.0f); glVertex2f(1.0f,-1.0f);
    }
    glEnd();
}

void GLSampleSelector::drawBlades(bool select_mode)
{
    glTranslatef((((2.4f-scale) - 0.2857f*(2.4f-scale))*(0.43f))/scale,
                 ((2.4f-scale)*0.43f)/scale, 0.0f);
    glScalef(scale*0.43f, scale*0.43f, scale*0.43f);

    gluPerspective(90.0f, 1.0f, 1.0f, 26.0f);
    glTranslatef(0.0f, 0.0f, -4.0f);
    glScalef(0.43f, 0.43f, 0.43f);
    glRotatef(36.4f, 1.0f, 0.0f, 0.0f);
    glRotatef(y_rot, 0.0f, 1.0f, 0.0f);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    glRotatef(carousel_rot, 0, 1, 0);
    int num_blades = images.size() != 0 ? images.size() : 10;
    GLfloat blade_rotation = 360.0f / num_blades;
    for (int x = 0; x < num_blades; ++x) {
        if (x > textures.size() - 1)
            glBindTexture(GL_TEXTURE_2D, logo);
        else
            glBindTexture(GL_TEXTURE_2D, textures.at(x));

        glRotatef(blade_rotation, 0.0f, 1.0f, 0.0f);
        glTranslatef(0.0f, 0.0f, -4.0f);
        if (select_anim && current_blade == x) {
            glMatrixMode(GL_MODELVIEW);
            glPushMatrix();
            glScalef(anim_scale, anim_scale, anim_scale);
            glColor4f(anim_alpha, anim_alpha, anim_alpha, anim_alpha);
            anim_scale += 0.3f;
            anim_alpha -= 0.1f;
        } else {
            if (current_blade == x)
                glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
            else
                glColor4f(0.9f, 0.9f, 0.9f, 1.0f);
        }
        glLoadName(x);
        glBegin(GL_QUADS);
        {
            glTexCoord2f(1.0f, 0.0f); glVertex3f(0.0f, -2.0f,2.0f);
            glTexCoord2f(1.0f, 1.0f); glVertex3f(0.0f, 2.0f,2.0f);
            glTexCoord2f(0.0f, 1.0f); glVertex3f(0.0f, 2.0f,-2.0f);
            glTexCoord2f(0.0f, 0.0f); glVertex3f(0.0f, -2.0f,-2.0f);
        }
        glEnd();
        if (current_blade == x) {
            glDisable(GL_TEXTURE_2D);
            glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
            glBegin(GL_LINE_LOOP);
            {
                glVertex3f(0.0f, -2.1f,2.1f);
                glVertex3f(0.0f, 2.1f,2.1f);
                glVertex3f(0.0f, 2.1f,-2.1f);
                glVertex3f(0.0f, -2.1f,-2.1f);
            }
            glEnd();
            glEnable(GL_TEXTURE_2D);
        }

        if (!select_mode) {
            glTranslatef(0.0f, -4.2f, 0.0f);
            glBegin(GL_QUADS);
            {
                glColor4f(0.0f, 0.0f, 0.0f, 0.0f);
                glTexCoord2f(0.0f, 1.0f); glVertex3f(0.0f, -2.0f,-2.0f);
                glTexCoord2f(1.0f, 1.0f); glVertex3f(0.0f, -2.0f,2.0f);
                glColor4f(0.2f, 0.2f, 0.2f, 0.2f);
                glTexCoord2f(1.0f, 0.0f); glVertex3f(0.0f, 2.0f,2.0f);
                glTexCoord2f(0.0f, 0.0f); glVertex3f(0.0f, 2.0f,-2.0f);
            }
            glEnd();
            if (current_blade == x) {
                glDisable(GL_TEXTURE_2D);
                glBegin(GL_LINE_LOOP);
                {
                    glColor4f(0.0f, 0.0f, 0.0f, 0.0f);
                    glVertex3f(0.0f, -2.1f,2.1f);
                    glVertex3f(0.0f, 2.1f,2.1f);
                    glColor4f(0.2f, 0.2f, 0.2f, 0.2f);
                    glVertex3f(0.0f, 2.1f,-2.1f);
                    glVertex3f(0.0f, -2.1f,-2.1f);
                }
                glEnd();
                glEnable(GL_TEXTURE_2D);
            }
            glTranslatef(0.0f, 4.2f, 0.0f);
        }
        if (select_anim && current_blade == x) {
            glPopMatrix();
            if (anim_alpha <= 0.0f) {
                select_anim = false;
                anim_scale = 1.0f;
                anim_alpha = 1.0f;
            }
        }
        glTranslatef(0.0f, 0.0f, 4.0f);
    }
}

void GLSampleSelector::prepareTextures(const QString &search)
{
    for (int i=0; i<textures.size(); ++i)
        deleteTexture(textures.at(i));
    current_image = logo;
    images.clear();
    textures.clear();
    http->setHost("excalibur.inria.fr");
    if(!search.isEmpty())
	search_id = http->get(QString("/samples/search.php?q=%1").arg(search));
    else
	search_id = http->get(QString("/samples/search.php"));
    emit statusChanged("Starting image search..");
}

void GLSampleSelector::processResults(int id, bool error)
{
    if (error) {
        emit statusChanged("An error occured while searching for images.");
        return;
    }

    if (id == search_id) {
        if (http->bytesAvailable() > 0) {
            QByteArray ba = http->readAll();
            const char *data = ba.constData();
            const int len = 0;
            int i = 0;
            while (i < ba.size()-5) {
                if (data[i] == '.') {
                    if (data[i+1] == 'j' && data[i+2] == 'p'
                        && data[i+3] == 'g' && data[i+4] == '\"') {
                        int j = i;
                        while (j > 0 && data[j] != '\"') --j;
                        QByteArray addr(ba.mid(j+1, i-j+3));
			QString fileName = QString(addr.mid(len, addr.size()-len).constData()) ;
                        images << fileName ;
                    }
                }
                ++i;
            }
        }

        if (images.size() > 0) {
            http->setHost("excalibur.inria.fr");
	    currentImageFile = images.at(0) ;
            image_id = http->get(QString("/samples/%1").arg(images.at(0)));
            emit statusChanged(QString("Found %1 images. Getting image 1 of %1..").arg(images.size()));
        }
    }

    if (id == image_id) {
        if (http->bytesAvailable() > 0) {
            QByteArray ba = http->readAll();
            QImage im(QImage::fromData(ba));
            if (!im.isNull()) {
		GLuint t_id = bindTexture(im) ;
                textures.append(t_id);
		map[t_id] = currentImageFile ;
            } else {
                images.removeAt(textures.size());
			}
            if (images.size() > textures.size()) {
                emit statusChanged(QString("Found %1 images. Getting image %2 of %1..").arg(images.size()).arg(textures.size()));
		currentImageFile = images.at(textures.size()) ;
                image_id = http->get(QString("/samples/%1").arg(images.at(textures.size()))) ;
            } else {
                emit statusChanged("Ready.");
            }
        }
    }

    if (id == file_id) {
		tmp->close() ;
		emit fileLoaded(tmp->fileName()) ;
    }
}

void GLSampleSelector::load(void) 
{
// # ifdef Q_WS_MAC
//     QString imageFile = images.at(current_image-2) ; 
// # else
//     QString imageFile = images.at(current_image-1) ; 
// # endif
    QString imageFile = map[current_image] ;

	QString fileName = imageFile.replace(imageFile.size()-3, 3, "axl").remove(0,2) ; // removing leading "./"
	tmp = new QFile("/tmp/tmp.axl") ;
	if(!tmp->open(QIODevice::WriteOnly)) {
		QMessageBox::information(0, "Axel","Error of reading","&Ok",0) ;
		return ;
	}

    http->setHost("excalibur.inria.fr") ;
    emit statusChanged("Loading file " + fileName) ;
    file_id = http->get(QString("/samples/%1").arg(fileName), tmp) ;
}

SampleSelector::SampleSelector(QWidget * parent) : QWidget(parent)
{
    QPalette pal = palette();
    pal.setColor(QPalette::Window, QColor(234, 233, 232));
    setPalette(pal);
    QVBoxLayout *vlay = new QVBoxLayout(this);
    view = new GLSampleSelector(this);
    view->prepareTextures("") ;
    vlay->addWidget(view);
    connect(view, SIGNAL(fileSelected(const QString&)), this, SLOT(updateFileLabel(const QString&))) ;

    QWidget *searchPanel = new QWidget(this);
    vlay = new QVBoxLayout(searchPanel);
    QLabel *label = new QLabel("Do an Axel file search for:", searchPanel);
    searchText = new QLineEdit(searchPanel);
    pal = searchText->palette();
    pal.setColor(QPalette::Base, QColor(255, 255, 255, 127));
    searchText->setPalette(pal);

    QHBoxLayout *hlay = new QHBoxLayout;
    hlay->addWidget(label);
    hlay->addWidget(searchText);
    vlay->addLayout(hlay);
    statusText = new QLabel("Ready.", searchPanel);
    connect(view, SIGNAL(statusChanged(const QString &)),
            statusText, SLOT(setText(const QString &)));
    vlay->addWidget(statusText);
    searchPanel->move(20, 30);
    searchPanel->resize(vlay->sizeHint());

    instructionBox = new QGroupBox(this);
//         instructionBox->setAttribute(Qt::WA_TransparentForMouseEvents);
    pal = instructionBox->palette();
    pal.setColor(QPalette::Window, QColor(255, 255, 255, 127));
    instructionBox->setPalette(pal);
    instructionBox->setAutoFillBackground(true);
//        instructionBox->move(380, 180);
    instructionBox->resize(235, 275);
    vlay = new QVBoxLayout(instructionBox);
    label = new QLabel("<center><b>Instructions</b></center><br>"
                       "- Click the right mouse button to zoom the file selector in/out.<br>"
                       "- Hold the left button down while moving the mouse left/right to rotate the file selector.<br>", instructionBox);
//         label->setAttribute(Qt::WA_TransparentForMouseEvents);
    label->setWordWrap(true);
    fileLabel = new QLabel("<center><b>Selection</b></center><br>"
			   "No file selected") ;
    fileLabel->setWordWrap(true);
    vlay->addWidget(label);
    vlay->addWidget(fileLabel);

    hlay = new QHBoxLayout(0);
    QPushButton *load_btn = new QPushButton("Load", instructionBox);
    hlay->addItem(new QSpacerItem(0, 0, QSizePolicy::Expanding));
    hlay->addWidget(load_btn);
    hlay->addItem(new QSpacerItem(0, 0,QSizePolicy::Expanding));
    vlay->addLayout(hlay);

    connect(load_btn, SIGNAL(clicked()), view, SLOT(load()));
    connect(searchText, SIGNAL(returnPressed()), SLOT(startSearch()));
    searchText->setText("");
    view->prepareTextures(searchText->text());
//    resize(640, 480);
}

void SampleSelector::updateFileLabel(const QString & s)
{
	fileLabel->setText("<center><b>Selection</b></center><br>" + s) ;
}

*/
