/*****************************************************************************
 * A x e l
 *****************************************************************************
 * SceneInspectorDock
 * 2006-03-31
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 *
 * The scene inspector implements Qt's MVC pattern (stands for Model/View/
 * Controller)
 ****************************************************************************/
  
# ifndef SCENEINSPECTORDOCK_H
# define SCENEINSPECTORDOCK_H

# include <QList>
# include <QTreeView>
# include <QItemDelegate>
# include <QAbstractItemModel>
# include <QModelIndex>
# include <QVariant>
# include <QDockWidget>
# include <QPixmap>

# include <QAxel/QAxelObject.h>

using namespace QAxel ;

class Viewer ;
class QTreeView ;

/*****************************************************************************
 * AXelObjectItem : An entry in the MODEL of the scene inspector
 ****************************************************************************/

class AxelObjectItem
{
public:
	AxelObjectItem(QAxelObject * o, const QList<QVariant> &data, AxelObjectItem *parent = 0);
	~AxelObjectItem();

	void appendChild(AxelObjectItem *child);

	AxelObjectItem *child(int row);
	int childCount() const;
	int columnCount() const;
	QVariant data(int column) const;
	int row() const;
	AxelObjectItem *parent();
	QAxelObject * object(void) { return itemObject ; }
	
	void clear(void) { qDeleteAll(childItems); }

private:
	QList<AxelObjectItem*> childItems;
	QList<QVariant> itemData;
	AxelObjectItem *parentItem;
	QAxelObject * itemObject ;
} ;

/*****************************************************************************
 * AXelObjectModel : The model used in the view to link datas (AxelObjects)
 ****************************************************************************/

class AxelObjectModel : public QAbstractItemModel
{
	Q_OBJECT

public:
	AxelObjectModel(QList<QAxelObject *> objects, QObject *parent = 0);
	~AxelObjectModel();

	QVariant data(const QModelIndex &index, int role) const;
	Qt::ItemFlags flags(const QModelIndex &index) const;
	QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
	QModelIndex index(int row, int column,const QModelIndex &parent = QModelIndex()) const;
	QModelIndex parent(const QModelIndex &index) const;
	int rowCount(const QModelIndex &parent = QModelIndex()) const;
	int columnCount(const QModelIndex &parent = QModelIndex()) const;
	AxelObjectItem * root(void) ;
	QAxelObject * object(int i) { return objects[i] ; }

	void setupModelData(QList<QAxelObject *> objects, AxelObjectItem * parent);

public: signals:
	void dataChanged(const QModelIndex & topLeft, const QModelIndex & bottomRight) ;
	
private :
	AxelObjectItem *rootItem ;
	QList<QAxelObject *> objects ;
} ;

/*****************************************************************************
 * AxelObjectDelegate
 ****************************************************************************/

class AxelObjectItemDelegate : public QItemDelegate
{
	Q_OBJECT
	
public:
	AxelObjectItemDelegate(Viewer * v, AxelObjectModel * m, QObject *parent);

    void paint(QPainter *painter, const QStyleOptionViewItem &option,
               const QModelIndex &index) const;

    QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const;

    bool editorEvent(QEvent *event, QAbstractItemModel *model,
				const QStyleOptionViewItem &option,
				const QModelIndex &index);

    QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option,
				const QModelIndex &index) const;
	
signals:
	void update(void) ;
	
private:
	QPixmap * viewactive ;
	QPixmap * viewinactive ;
	AxelObjectModel * mod ;
	Viewer * v ;
} ;

/*****************************************************************************
 * AxelObjectView
 ****************************************************************************/

class AxelObjectView : public QTreeView
{
	Q_OBJECT
			
public :	
	AxelObjectView(AxelObjectModel * model, Viewer * v, QWidget * parent = 0) ;
} ;

/*****************************************************************************
 * SceneInspectorDock
 ****************************************************************************/

class SceneInspectorDock : public QDockWidget {
	Q_OBJECT       

public :
	SceneInspectorDock(Viewer * v, const QString & title, QWidget * parent) ;
	
public slots :
	void update(void) ;
	void selectObject(const QModelIndex & index) ;
	
private :
	Viewer * viewer ;
	AxelObjectModel * model ;
	AxelObjectView * tree ;
} ;

# endif
