/*****************************************************************************
 * A x e l
 *****************************************************************************
 * ToolPropertiesWidgets
 * 2006-09-28
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# include <QAction>
# include <QProcess>
# include <QComboBox>
# include <QMessageBox>
# include <QProgressDialog>

# include <QAxel/QConstraintIncidence.h>
# include <QAxel/QConstraintDistance.h>
# include <QAxel/QConstraintAngle.h>
# include <QAxel/QMesh.h>
# include <QAxel/QBoundingBox.h>
# include <QAxel/QPoint3f.h>
# include <QAxel/QCurve.h>
# include <QAxel/QSurface.h>
# include <QAxel/PluginManager.h>

# include <QGui/Commands.h>
# include <QGui/MainWindow.h>
# include <QGui/Viewer.h>
# include <QGui/ImageViewer.h>
# include <QGui/ToolPropertiesWidgets.h>
# include <QGui/ScriptingDock.h>
# include <QGui/ImplicitCurvesArrangementDock.h>
# include <QGui/ParametricCurvesArrangementDock.h>

# include <QSynaps/QSynapsInterface.h>

# include <QStream/OStream/OStreamSurf.h>
# include <QStream/OStream/OStreamYafray.h>

using namespace QStream ;

NoToolWidget::NoToolWidget(MainWindow * mw, QWidget * parent) : QWidget(parent)
{
    this->mw = mw ;
    ui.setupUi(this) ;
}

PointToolWidget::PointToolWidget(MainWindow * mw, QWidget * parent) : QWidget(parent) 
{
    this->mw = mw ;
    ui.setupUi(this);
    connect(ui.create, SIGNAL(clicked()), this, SLOT(create())) ;
}                

void PointToolWidget::create(void) 
{
    CreatePointCommand * command = new CreatePointCommand(mw->viewer(), ui.xspin->value(), ui.yspin->value(), ui.zspin->value()) ;
    mw->stack()->push(command) ;
}                                               

PlaneToolWidget::PlaneToolWidget(MainWindow * mw, QWidget * parent) : QWidget(parent) 
{
    this->mw = mw ;
    ui.setupUi(this) ;
    connect(ui.create, SIGNAL(clicked()), this, SLOT(create())) ;
}

void PlaneToolWidget::create(void) 
{
    float a = 0.0, b = 0.0, c = 0.0, d = 0.0 ;
    QString polynomial = ui.equation->text().simplified() ;
    QRegExp ra("(\\d*)(\\.\\d+)*(x)(\\^\\d+)*") ;
    QRegExp rb("(\\d*)(\\.\\d+)*(y)(\\^\\d+)*") ;
    QRegExp rc("(\\d*)(\\.\\d+)*(z)(\\^\\d+)*") ;
    QRegExp rd("(\\d+)$") ;
    if(ra.indexIn(polynomial) > -1) { QString as = ra.cap(1) ; if(as == "") a = 1 ; else a = as.toFloat() ; }
    if(rb.indexIn(polynomial) > -1) { QString bs = rb.cap(1) ; if(bs == "") b = 1 ; else b = bs.toFloat() ; }
    if(rc.indexIn(polynomial) > -1) { QString cs = rc.cap(1) ; if(cs == "") c = 1 ; else c = cs.toFloat() ; }
    if(rd.indexIn(polynomial) > -1) { QString ds = rd.cap(1) ; if(ds == "") d = 0 ; else d = ds.toFloat() ; }
    CreatePlaneCommand * command = new CreatePlaneCommand(mw->viewer(), a, b, c, d) ;
    mw->stack()->push(command) ;
}

CurveToolWidget::CurveToolWidget(MainWindow * mw, QWidget * parent) : QWidget(parent) 
{
    this->mw = mw ;
    ui.setupUi(this);
    bb = new QBoundingBox(-2.01, 2.01, -2.01, 2.01, -2.01, 2.01) ;
    method = QImplicitCurve::Subdivision ;
    
    connect(ui.create, SIGNAL(clicked()), this, SLOT(create()));
}

CurveToolWidget::~CurveToolWidget(void)
{

}

void CurveToolWidget::create(void) 
{
    bb->setXMin(ui.xmin->value()-0.01) ;
    bb->setXMax(ui.xmax->value()+0.01) ;
    bb->setYMin(ui.ymin->value()-0.01) ;
    bb->setYMax(ui.ymax->value()+0.01) ;
    bb->setZMin(ui.zmin->value()-0.01) ;
    bb->setZMax(ui.zmax->value()+0.01) ;
    method = (ui.comboBox->currentText()=="Sweep") ? QImplicitCurve::Sweep : QImplicitCurve::Subdivision ;
    if(ui.tabWidget->currentIndex() == 0) {
	if(ui.checkBox2D->checkState() == Qt::Checked) {
	    bb->setZMin(0.0) ; bb->setZMax(0.0) ;
	    CreateImplicitCurve2DCommand * command = new CreateImplicitCurve2DCommand(mw->viewer(), ui.equation2D->text(), bb, method) ;
	    mw->stack()->push(command) ;
	} else {
	    CreateImplicitCurve3DCommand * command = new CreateImplicitCurve3DCommand(mw->viewer(), ui.equation2D->text(), ui.equation3D->text(), bb, method) ;
	    mw->stack()->push(command) ;
	}
    } 
    else if(ui.tabWidget->currentIndex() == 1) {
	QString xt = ui.xt->text() ;
	QString yt = ui.yt->text() ;
	QString zt = ui.zt->text() ;
	QString wt = ui.wt->text() ;
	float tmin = ui.tmin->value() ;
	float tmax = ui.tmax->value() ;
	CreateRationalCurveCommand * command = new CreateRationalCurveCommand(mw->viewer(), xt, yt, zt, wt, tmin, tmax) ;
	mw->stack()->push(command) ;
    }
    else if(ui.tabWidget->currentIndex() == 2) {
	int iopen = 1 ;
	int ikind = 1 ;
	int order = ui.order->value() ;
	int dimension = ui.dimension->value() ;
	CreateBSplineCurveCommand * command = new CreateBSplineCurveCommand(mw->viewer(),iopen ,ikind ,order ,dimension) ;
	mw->stack()->push(command) ;
    }
    else
	qDebug("unexpected current index of tab") ;
}

SurfaceToolWidget::SurfaceToolWidget(MainWindow * mw, QWidget * parent) : QWidget(parent) 
{
    mainwindow = mw ;
    bb = new QBoundingBox(-2.01, 2.01, -2.01, 2.01, -2.01, 2.01) ;
    ui.setupUi(this);
    connect(ui.create, SIGNAL(clicked()), this, SLOT(create())) ;
}      
          

void SurfaceToolWidget::create(void) 
{    
    if(ui.tabWidget->currentIndex() == 0) {
	bb->setXMin(ui.xmin->value()-0.01) ;
	bb->setXMax(ui.xmax->value()+0.01) ;
	bb->setYMin(ui.ymin->value()-0.01) ;
	bb->setYMax(ui.ymax->value()+0.01) ;
	bb->setZMin(ui.zmin->value()-0.01) ;
	bb->setZMax(ui.zmax->value()+0.01) ;
	QString eq = ui.equation->text() ;
	if(!eq.contains("x")) eq.append("+0.0000001*x") ;
	if(!eq.contains("y")) eq.append("+0.0000001*y") ;
	if(!eq.contains("z")) eq.append("+0.0000001*z") ;
	QImplicitSurface * s = new QImplicitSurface(eq, bb) ;
	mainwindow->viewer()->addObject(s) ;	    
    } else if(ui.tabWidget->currentIndex() == 1) {
	QString xst = ui.xst->text() ;
	QString yst = ui.yst->text() ;
	QString zst = ui.zst->text() ;
	QString wst = ui.wst->text() ;
	float smin = ui.smin->value() ;
	float smax = ui.smax->value() ;
	float tmin = ui.tmin->value() ;
	float tmax = ui.tmax->value() ;
	CreateRationalSurfaceCommand * command = new CreateRationalSurfaceCommand(mainwindow->viewer(), xst, yst, zst, wst, smin, smax, tmin, tmax) ;
	mainwindow->stack()->push(command) ;  
    } else if(ui.tabWidget->currentIndex() == 2) {

	int order1 = ui.order1->value() ;
	int order2 = ui.order2->value() ;
	int number1 = ui.number1->value() ;
	int number2 = ui.number2->value() ;
	int dimension = ui.dimension->value() ;
	CreateBSplineSurfaceCommand * command = new CreateBSplineSurfaceCommand(mainwindow->viewer(), order1, order2, number1, number2, dimension) ;
	mainwindow->stack()->push(command) ;

    } else 
	cout << __func__ << " : Unexpected tab value in for surface tool widget : " << endl ;
}

ConvertToMeshToolWidget::ConvertToMeshToolWidget(MainWindow * mw, QWidget * parent) : QWidget(parent) 
{
    mainwindow = mw ;
    ui.setupUi(this);
    connect(ui.convert, SIGNAL(clicked()), this, SLOT(convert())) ;
}                

void ConvertToMeshToolWidget::convert(void) 
{
    QList<QAxelObject *> selection = mainwindow->viewer()->listSelectedObjects() ;
    for(int i=0 ; i < selection.size() ; i++) {
        if(QImplicitCurve2D * c = dynamic_cast<QImplicitCurve2D *>(selection[i])) {
	    Q_UNUSED(c) ;
// 	    QMesh * m = new QMesh(c->mesh()) ;
// 	    mainwindow->viewer()->addObject(m) ;
// 	    QMessageBox::information(this, "Convert to mesh", selection[i]->name() + " has been converted to mesh", "&Ok", 0) ;
	} else if(QImplicitCurve3D * c = dynamic_cast<QImplicitCurve3D *>(selection[i])) {
	    Q_UNUSED(c) ;
// 	    QMesh * m = new QMesh(c->mesh()) ;
// 	    mainwindow->viewer()->addObject(m) ;
// 	    QMessageBox::information(this, "Convert to mesh", selection[i]->name() + " has been converted to mesh", "&Ok", 0) ;
	} else if(QImplicitSurface * s = dynamic_cast<QImplicitSurface *>(selection[i])) {
	    Q_UNUSED(s) ;
// 	    QMesh * m = new QMesh(s->mesh()) ;
// 	    mainwindow->viewer()->addObject(m) ;
// 	    QMessageBox::information(this, "Convert to mesh", selection[i]->name() + " has been converted to mesh", "&Ok", 0) ;
	} else {
	    QMessageBox::information(this, "Convert to mesh", selection[i]->name() + " is not yet convertible to mesh", "&Ok", 0) ;
	}
    }
}

InterpolationBSplineCurveToolWidget::InterpolationBSplineCurveToolWidget(MainWindow * mainwindow, QWidget * parent) : QWidget(parent) 
{
    mw = mainwindow ;
    ui.setupUi(this) ;
    connect(ui.compute, SIGNAL(clicked()), this, SLOT(compute())) ;
}

void InterpolationBSplineCurveToolWidget::compute(void)
{
    QMessageBox::information(this, "Interpolates selected points in order into a B-Spline Curve", "Not yet connected", "&Ok", 0) ; 
}


ApproximationBSplineCurveToolWidget::ApproximationBSplineCurveToolWidget(MainWindow * mainwindow, QWidget * parent) : QWidget(parent) 
{
    mw = mainwindow ;
    ui.setupUi(this) ;
    connect(ui.compute, SIGNAL(clicked()), this, SLOT(compute())) ;
}

void ApproximationBSplineCurveToolWidget::compute(void)
{
    mw->viewer()->approximatePointsAsBSplineCurve() ;
}

ApproximationBSplineSurfaceToolWidget::ApproximationBSplineSurfaceToolWidget(MainWindow * mainwindow, QWidget * parent) : QWidget(parent) 
{
    mw = mainwindow ;
    ui.setupUi(this) ;
    connect(ui.compute, SIGNAL(clicked()), this, SLOT(compute())) ;
}

void ApproximationBSplineSurfaceToolWidget::compute(void)
{
    int iopen1 = ui.iopen1->value() ;
    int iopen2 = ui.iopen2->value() ;
    int order1 = ui.order1->value() ;
    int order2 = ui.order2->value() ;
    int dimension = ui.dimension->value() ;
    mw->viewer()->approximatePointsAsBSplineSurface(iopen1, iopen2, order1, order2, dimension) ;
}

JoinBSplineCurveToolWidget::JoinBSplineCurveToolWidget(MainWindow * mainwindow, QWidget * parent) : QWidget(parent) 
{
    mw = mainwindow ;
    ui.setupUi(this) ;
    connect(ui.compute, SIGNAL(clicked()), this, SLOT(compute())) ;
}

void JoinBSplineCurveToolWidget::compute(void)
{
    mw->viewer()->joinBSplineCurves() ;
    //QMessageBox::information(this, "Computes an Approximate B-Spline Curve with the selected vertices as Control Points", "Not yet connected", "&Ok", 0) ;
}


DistanceConstraintWidget::DistanceConstraintWidget(MainWindow * mw, QWidget * parent) : QWidget(parent) 
{
    mainwindow = mw ;
    ui.setupUi(this);
    QList<QString> l = mainwindow->viewer()->getListObjectsName();
    ui.comboBox1->addItem("Select a point");
    ui.comboBox2->addItem("Select a point");
    for (int i = 0; i < l.size(); i++)
    {
	if (qobject_cast<QPoint3f *>(mainwindow->viewer()->getObjectFromName(l.at(i))))
	{
	    ui.comboBox1->addItem(l.at(i));
	    ui.comboBox2->addItem(l.at(i));
	}
    }
    if(l.size())
    {
	select();
    }
	
    connect(ui.create, SIGNAL(clicked()), this, SLOT(create())) ;
    connect(ui.comboBox1, SIGNAL(currentIndexChanged(int)), this, SLOT(changed())) ;
    connect(ui.comboBox2, SIGNAL(currentIndexChanged(int)), this, SLOT(changed())) ;
    connect(mainwindow->viewer(), SIGNAL(objectSelected(QAxelObject *)), this, SLOT(select())) ;
    connect(mainwindow->viewer(), SIGNAL(noObjectSelected()), this, SLOT(select())) ;
}

void DistanceConstraintWidget::changed()
{
    QAxelObject *A=mainwindow->viewer()->getObjectFromName(ui.comboBox1->currentText());
    QAxelObject *B=mainwindow->viewer()->getObjectFromName(ui.comboBox2->currentText());
	
    QList<QAxelObject *> l;
    if (A)
	l.append(A);
    if (B)
	l.append(B);
    mainwindow->viewer()->setSelectedObjects(l);
}

void DistanceConstraintWidget::select(void)
{
    QList<QAxelObject *> selected = mainwindow->viewer()->listSelectedObjectsFromType("Point3f");
    switch(selected.size())
    {
    case 2 :
	ui.comboBox2->setCurrentIndex(ui.comboBox2->findText(selected.at(1)->name()));
    case 1 :
	ui.comboBox1->setCurrentIndex(ui.comboBox1->findText(selected.at(0)->name()));
	break;
    default :
	ui.comboBox1->setCurrentIndex(0);
	ui.comboBox2->setCurrentIndex(0);
	break;
    }
    if(selected.size()==2)
    {
	create();
    }
}

void DistanceConstraintWidget::create(void) 
{
    QAxelObject *A=mainwindow->viewer()->getObjectFromName(ui.comboBox1->currentText());
    QAxelObject *B=mainwindow->viewer()->getObjectFromName(ui.comboBox2->currentText());
    if (A && B && A!=B)
    {
	bool notIn=true;
	QList<QAxelObject *> l=mainwindow->viewer()->listObjectsFromType("ConstraintDistance");
	for(int i=0; i<l.size() && notIn; i++)
	{
	    QConstraintDistance *d=qobject_cast<QConstraintDistance *>(l.at(i));
	    if((d->getObject(0)==A && d->getObject(1)==B) || (d->getObject(0)==B && d->getObject(1)==A))
	    {
		notIn=false;
	    }
	}
	if(notIn)
	{
	    QConstraintDistance * dc = new QConstraintDistance(A, B, ui.doubleSpinBox->value());
	    mainwindow->viewer()->addObject(dc);
	}
    }
}

IncidenceConstraintWidget::IncidenceConstraintWidget(MainWindow * mw, QWidget * parent) : QWidget(parent) 
{
    mainwindow = mw ;
    ui.setupUi(this);
	
    QList<QString> l = mainwindow->viewer()->getListObjectsName();
    ui.comboBox1->addItem("Select a point");
    ui.comboBox2->addItem("Select a plane");
    for (int i = 0; i < l.size(); i++)
    {
	if (qobject_cast<QPoint3f *>(mainwindow->viewer()->getObjectFromName(l.at(i))))
	    ui.comboBox1->addItem(l.at(i));
	if (qobject_cast<QPlane *>(mainwindow->viewer()->getObjectFromName(l.at(i))))
	    ui.comboBox2->addItem(l.at(i));
    }
    if(l.size())
    {
	select();
    }
	
    connect(ui.create, SIGNAL(clicked()), this, SLOT(create())) ;
    connect(mainwindow->viewer(), SIGNAL(objectSelected(QAxelObject *)), this, SLOT(select())) ;
    connect(mainwindow->viewer(), SIGNAL(noObjectSelected()), this, SLOT(select())) ;
    connect(ui.comboBox1, SIGNAL(currentIndexChanged(int)), this, SLOT(changed())) ;
    connect(ui.comboBox2, SIGNAL(currentIndexChanged(int)), this, SLOT(changed())) ;
}

void IncidenceConstraintWidget::changed()
{
    QAxelObject *A=mainwindow->viewer()->getObjectFromName(ui.comboBox1->currentText());
    QAxelObject *B=mainwindow->viewer()->getObjectFromName(ui.comboBox2->currentText());
	
    QList<QAxelObject *> l;
    if (A)
	l.append(A);
    if (B)
	l.append(B);
    mainwindow->viewer()->setSelectedObjects(l);
}

void IncidenceConstraintWidget::select(void)
{
    QList<QAxelObject *> selected1 = mainwindow->viewer()->listSelectedObjectsFromType("Point3f");
    QList<QAxelObject *> selected2 = mainwindow->viewer()->listSelectedObjectsFromType("Plane");
    if(selected1.size())
	ui.comboBox1->setCurrentIndex(ui.comboBox1->findText(selected1.at(0)->name()));
    else
	ui.comboBox1->setCurrentIndex(0);
    if(selected2.size())
	ui.comboBox2->setCurrentIndex(ui.comboBox2->findText(selected2.at(0)->name()));
    else
	ui.comboBox2->setCurrentIndex(0);
    if(selected1.size()&&selected2.size())
    {
	create();
    }
}

void IncidenceConstraintWidget::create(void) 
{
    QAxelObject *A=mainwindow->viewer()->getObjectFromName(ui.comboBox1->currentText());
    QAxelObject *B=mainwindow->viewer()->getObjectFromName(ui.comboBox2->currentText());
    if (A && B && A!=B)
    {
	bool notIn=true;
	QList<QAxelObject *> l=mainwindow->viewer()->listObjectsFromType("ConstraintIncidence");
	for(int i=0; i<l.size() && notIn; i++)
	{
	    QConstraintIncidence *d=qobject_cast<QConstraintIncidence *>(l.at(i));
	    if((d->getObject(0)==A && d->getObject(1)==B))
	    {
		notIn=false;
	    }
	}
	if(notIn)
	{
	    QConstraintIncidence * ic = new QConstraintIncidence(A,B);
	    mainwindow->viewer()->addObject(ic);
	}
    }
}

AngleConstraintWidget::AngleConstraintWidget(MainWindow * mw, QWidget * parent) : QWidget(parent) 
{
    mainwindow = mw ;
    ui.setupUi(this);
	
    QList<QString> l = mainwindow->viewer()->getListObjectsName();
    ui.comboBox1->addItem("Select a point");
    ui.comboBox2->addItem("Select a plane");
    for (int i = 0; i < l.size(); i++)
    {
	if (qobject_cast<QPoint3f *>(mainwindow->viewer()->getObjectFromName(l.at(i))))
	    ui.comboBox1->addItem(l.at(i));
	if (qobject_cast<QPlane *>(mainwindow->viewer()->getObjectFromName(l.at(i))))
	    ui.comboBox2->addItem(l.at(i));
    }
    if(l.size())
    {
	select();
    }
	
    connect(ui.create, SIGNAL(clicked()), this, SLOT(create())) ;
    connect(mainwindow->viewer(), SIGNAL(objectSelected(QAxelObject *)), this, SLOT(select())) ;
    connect(mainwindow->viewer(), SIGNAL(noObjectSelected()), this, SLOT(select())) ;
    connect(ui.comboBox1, SIGNAL(currentIndexChanged(int)), this, SLOT(changed())) ;
    connect(ui.comboBox2, SIGNAL(currentIndexChanged(int)), this, SLOT(changed())) ;
}

void AngleConstraintWidget::changed()
{
    QAxelObject *A=mainwindow->viewer()->getObjectFromName(ui.comboBox1->currentText());
    QAxelObject *B=mainwindow->viewer()->getObjectFromName(ui.comboBox2->currentText());
	
    QList<QAxelObject *> l;
    if (A)
	l.append(A);
    if (B)
	l.append(B);
    mainwindow->viewer()->setSelectedObjects(l);
}

void AngleConstraintWidget::select(void)
{
    QList<QAxelObject *> selected1 = mainwindow->viewer()->listSelectedObjectsFromType("Point3f");
    QList<QAxelObject *> selected2 = mainwindow->viewer()->listSelectedObjectsFromType("Plane");
    if(selected1.size())
	ui.comboBox1->setCurrentIndex(ui.comboBox1->findText(selected1.at(0)->name()));
    else
	ui.comboBox1->setCurrentIndex(0);
    if(selected2.size())
	ui.comboBox2->setCurrentIndex(ui.comboBox2->findText(selected2.at(0)->name()));
    else
	ui.comboBox2->setCurrentIndex(0);
    if(selected1.size()&&selected2.size())
    {
	create();
    }
}

void AngleConstraintWidget::create(void) 
{
    QAxelObject *A=mainwindow->viewer()->getObjectFromName(ui.comboBox1->currentText());
    QAxelObject *B=mainwindow->viewer()->getObjectFromName(ui.comboBox2->currentText());
    if (A && B && A!=B)
    {
	bool notIn=true;
	QList<QAxelObject *> l=mainwindow->viewer()->listObjectsFromType("ConstraintAngle");
	for(int i=0; i<l.size() && notIn; i++)
	{
	    QConstraintAngle *d=qobject_cast<QConstraintAngle *>(l.at(i));
	    if((d->getObject(0)==A && d->getObject(1)==B))
	    {
		notIn=false;
	    }
	}
	if(notIn)
	{
	    QConstraintAngle * ic = new QConstraintAngle(A,B, ui.doubleSpinBox->value());
	    mainwindow->viewer()->addObject(ic);
	}
    }
}

CurveArrangementWidget::CurveArrangementWidget(MainWindow * mw, QWidget * parent) : QWidget(parent) 
{
    mainwindow = mw ;
    ui.setupUi(this);
    connect(ui.launch, SIGNAL(clicked()), this, SLOT(launch())) ;
}

void CurveArrangementWidget::launch(void) 
{
    ParametricCurvesArrangementDock * pcdock ;
    ImplicitCurvesArrangementDock   * icdock ;

    switch(ui.comboBox->currentIndex()) {
    case 0:
	// PiecewiseLinearcurves
    case 1:
	pcdock = new ParametricCurvesArrangementDock("Parametric Curves Arrangement", mainwindow) ;
	pcdock->setAllowedAreas(Qt::BottomDockWidgetArea) ;
	mainwindow->addDockWidget(Qt::BottomDockWidgetArea, pcdock) ;
	if(mainwindow->scriptingDock()->toggleViewAction()->isChecked()) 
	    mainwindow->tabifyDockWidget(mainwindow->scriptingDock(), pcdock) ;
	break ;
    case 2:
	icdock = new ImplicitCurvesArrangementDock("Implicit Curves Arrangement", mainwindow) ;
	icdock->setAllowedAreas(Qt::BottomDockWidgetArea) ;
	mainwindow->addDockWidget(Qt::BottomDockWidgetArea, icdock) ;
	if(mainwindow->scriptingDock()->toggleViewAction()->isChecked()) 
	    mainwindow->tabifyDockWidget(mainwindow->scriptingDock(), icdock) ;
	break ;
    default:
	break ;
    }
}

FirstDerivativeWidget::FirstDerivativeWidget(MainWindow * mw, QWidget * parent) : QWidget(parent) 
{
    mainwindow = mw ;
    ui.setupUi(this);
    connect(ui.compute, SIGNAL(clicked()), this, SLOT(compute())) ;
}

void FirstDerivativeWidget::compute(void)
{
    QSynapsInterface * iSynaps = dynamic_cast<QSynapsInterface *>(PluginManager::instance()->plugin("Synaps")) ;
    QString v = ui.comboBox->currentText() ;
    QList<QAxelObject *> curves = mainwindow->viewer()->listSelectedObjects() ;	
    foreach(QAxelObject * o, curves) {
	QImplicitCurve2D * c = dynamic_cast<QImplicitCurve2D *>(o) ;
	if(c) {
	    QImplicitCurve2D * dc = iSynaps->computeFirstDerivativeCurve(c,v) ;
	    mainwindow->viewer()->addObject(dc) ;
	} else {
	    QMessageBox::information(this, "Compute First Derivative Curve", o->name() + " is not an implicit 2D curve !", "&Ok", 0) ;
	}
    }
}

SecondDerivativeWidget::SecondDerivativeWidget(MainWindow * mw, QWidget * parent) : QWidget(parent) 
{
    mainwindow = mw ;
    ui.setupUi(this);
    connect(ui.compute, SIGNAL(clicked()), this, SLOT(compute())) ;
}

void SecondDerivativeWidget::compute(void)
{
    QSynapsInterface * iSynaps = dynamic_cast<QSynapsInterface *>(PluginManager::instance()->plugin("Synaps")) ;
    QList<QAxelObject *> curves = mainwindow->viewer()->listSelectedObjects() ;	
    QString v = ui.comboBox->currentText() ;
    foreach(QAxelObject * o, curves) {
	QImplicitCurve2D * c = dynamic_cast<QImplicitCurve2D *>(o) ;
	if(c) {
	    QImplicitCurve2D * dc = iSynaps->computeSecondDerivativeCurve(c,v) ;
	    mainwindow->viewer()->addObject(dc) ;
	} else {
	    QMessageBox::information(this, "Compute Second Derivative Curve", o->name() + " is not an implicit 2D curve !", "&Ok", 0) ;
	}
    }
}

JacobiCurveWidget::JacobiCurveWidget(MainWindow * mw, QWidget * parent) : QWidget(parent) 
{
    mainwindow = mw ;
    ui.setupUi(this);
    connect(ui.compute, SIGNAL(clicked()), this, SLOT(compute())) ;
}

void JacobiCurveWidget::compute(void)
{
    QSynapsInterface * iSynaps = dynamic_cast<QSynapsInterface *>(PluginManager::instance()->plugin("Synaps")) ;
    QList<QAxelObject *> curves = mainwindow->viewer()->listSelectedObjects() ;	
    if(curves.size() != 2) 
	QMessageBox::information(this, 
				 "Compute Jacobi Curve", 
				 "You must implicit 2D curve !", "&Ok", 0) ;
	
    QImplicitCurve2D * c1 = dynamic_cast<QImplicitCurve2D *>(curves.first()) ;
    QImplicitCurve2D * c2 = dynamic_cast<QImplicitCurve2D *>(curves.last()) ;
    if(c1 && c2) {
	QImplicitCurve2D * dc = iSynaps->computeJacobiCurve(c1, c2) ;
	dc->setRGB(0,255,0) ;
	mainwindow->viewer()->addObject(dc) ;
    } else {
	QMessageBox::information(this, 
				 "Compute Jacobi Curve", 
				 "You must implicit 2D curve !", "&Ok", 0) ;
    }
}

HessianCurveWidget::HessianCurveWidget(MainWindow * mw, QWidget * parent) : QWidget(parent) 
{
    mainwindow = mw ;
    ui.setupUi(this);
    connect(ui.compute, SIGNAL(clicked()), this, SLOT(compute())) ;
}

void HessianCurveWidget::compute(void)
{
    QSynapsInterface * iSynaps = dynamic_cast<QSynapsInterface *>(PluginManager::instance()->plugin("Synaps")) ;
    QList<QAxelObject *> curves = mainwindow->viewer()->listSelectedObjects() ;	
    foreach(QAxelObject * o, curves) {
	QImplicitCurve2D * c = dynamic_cast<QImplicitCurve2D *>(o) ;
	if(c) {
	    QImplicitCurve2D * dc = iSynaps->computeHessianCurve(c) ;
	    dc->setRGB(255,0,200) ;
	    mainwindow->viewer()->addObject(dc) ;
	} else {
	    QMessageBox::information(this, "Compute Hessian Curve", o->name() + " is not an implicit 2D curve !", "&Ok", 0) ;
	}
    }
}

GraphedSurfaceWidget::GraphedSurfaceWidget(MainWindow * mw, QWidget * parent) : QWidget(parent)
{
    mainwindow = mw ;
    ui.setupUi(this);
    connect(ui.create, SIGNAL(clicked()), this, SLOT(create())) ;
}

void GraphedSurfaceWidget::create(void)
{
    QList<QAxelObject *> curves = mainwindow->viewer()->listSelectedObjects() ;	
    foreach(QAxelObject * o, curves) {
	QImplicitCurve2D * c = dynamic_cast<QImplicitCurve2D *>(o) ;
	if(c) {
	    QString equation = c->equation() ;
	    equation.append("-z") ;
	    QBoundingBox * bb = new QBoundingBox(-2.01, 2.01, -2.01, 2.01, -2.01, 2.01) ;
	    bb->setXMin(ui.xmin->value()-0.01) ;
	    bb->setXMax(ui.xmax->value()+0.01) ;
	    bb->setYMin(ui.ymin->value()-0.01) ;
	    bb->setYMax(ui.ymax->value()+0.01) ;
	    bb->setZMin(ui.zmin->value()-0.01) ;
	    bb->setZMax(ui.zmax->value()+0.01) ;
	    QImplicitSurface * s = new QImplicitSurface(equation, bb) ;
	    mainwindow->viewer()->addObject(s) ;
	} else {
	    QMessageBox::information(this, "Compute Graph Curve", o->name() + " is not an implicit 2D curve !", "&Ok", 0) ;
	}
    }
}

IcIntersectionWidget::IcIntersectionWidget(MainWindow * mw, QWidget * parent) : QWidget(parent)
{
    mainwindow = mw ;
    ui.setupUi(this) ;
    connect(ui.compute, SIGNAL(clicked()), this, SLOT(compute())) ;
}

void IcIntersectionWidget::compute(void)
{
    QMessageBox::information(this, "Compute Implicit Curves intersection", "Not yet connected", "&Ok", 0) ;
}

PcIntersectionWidget::PcIntersectionWidget(MainWindow * mw, QWidget * parent) : QWidget(parent)
{
    mainwindow = mw ;
    ui.setupUi(this) ;
    connect(ui.compute, SIGNAL(clicked()), this, SLOT(compute())) ;
}

void PcIntersectionWidget::compute(void)
{
    QMessageBox::information(this, "Compute Parametric Curves intersection", "Not yet connected", "&Ok", 0) ;
}

IsIntersectionWidget::IsIntersectionWidget(MainWindow * mw, QWidget * parent) : QWidget(parent)
{
    mainwindow = mw ;
    ui.setupUi(this) ;
    connect(ui.compute, SIGNAL(clicked()), this, SLOT(compute())) ;
}

void IsIntersectionWidget::compute(void)
{
    mainwindow->viewer()->intersectImplicitSurfaces() ;
}

PsIntersectionWidget::PsIntersectionWidget(MainWindow * mw, QWidget * parent) : QWidget(parent)
{
    mainwindow = mw ;
    ui.setupUi(this) ;
    connect(ui.compute, SIGNAL(clicked()), this, SLOT(compute())) ;
}

void PsIntersectionWidget::compute(void)
{
    mainwindow->viewer()->intersectParametricSurfaces() ;
}

YafrayRenderWidget::YafrayRenderWidget(MainWindow * mw, QWidget * parent) : QWidget(parent)
{
    mainwindow = mw ;
    ui.setupUi(this) ;
    connect(ui.render, SIGNAL(clicked()), this, SLOT(render())) ;
}

void YafrayRenderWidget::render(void)
{
    QString xmlFile = "/tmp/axel_render.xml" ;
    QString tgaFile = "/tmp/axel_render.tga" ;
    QString pngFile = "/tmp/axel_render.png" ;
    OStreamYafray parser(xmlFile, mainwindow->viewer()) ;
    parser.run() ;

# ifdef Q_WS_MAC	
    QString yafrayPath = "/usr/local/bin/yafray" ;
    QString convertPath = "/usr/local/bin/convert" ;
# else
    QString yafrayPath = "/usr/bin/yafray" ;
    QString convertPath = "/usr/bin/convert" ;
# endif
    QStringList yafrayArguments ; yafrayArguments << xmlFile ;
    QStringList convertArguments ; convertArguments << tgaFile << pngFile ;

    QProcess * renderProcess = new QProcess(NULL);
    renderProcess->start(yafrayPath, yafrayArguments);
    renderProcess->waitForFinished();

    QProcess * convertProcess = new QProcess(NULL);
    convertProcess->start(convertPath, convertArguments) ;
    convertProcess->waitForFinished();

    ImageViewer * imageViewer = new ImageViewer(this, Qt::WindowStaysOnTopHint) ;
    imageViewer->open(pngFile) ;
    imageViewer->show();

    delete renderProcess ;
    delete convertProcess ;	
}

SurfRenderWidget::SurfRenderWidget(MainWindow * mw, QWidget * parent) : QWidget(parent)
{
    mainwindow = mw ;
    ui.setupUi(this) ;
    ui.bgColorLabel->setPalette(QPalette(Qt::white));
    ui.bgColorLabel->setAutoFillBackground(true);
    connect(ui.render, SIGNAL(clicked()), this, SLOT(render())) ;
}

void SurfRenderWidget::render(void)
{
    QString picFile = "/tmp/axel_render.pic" ;
    QString ppmFile = "/tmp/axel_render.ppm" ;
    QString pngFile = "/tmp/axel_render.png" ;
	
    QProgressDialog * progress = new QProgressDialog("Surf Render", "Cancel", 0, 5) ;
    progress->setWindowModality(Qt::WindowModal) ;
	
    progress->setLabel(new QLabel("Parsing Scene")) ;
    progress->setValue(1) ;
    OStreamSurf parser(picFile, mainwindow->viewer()) ;
    parser.run() ;


    progress->setLabel(new QLabel("Locating Surf")) ;
    progress->setValue(2) ;
# ifdef Q_WS_MAC	
    QString surfPath = "/usr/local/bin/surf" ;
    QString convertPath = "/usr/local/bin/convert" ;
# else
    QString surfPath = "/usr/local/bin/surf" ;
    QString convertPath = "/usr/bin/convert" ;
# endif
    QStringList surfArguments ; surfArguments << "-n" << picFile ;
    QStringList convertArguments ; convertArguments << ppmFile << pngFile ;

    progress->setLabel(new QLabel("Rendering")) ;
    progress->setValue(3) ;
    QProcess * renderProcess = new QProcess(NULL);
    renderProcess->start(surfPath, surfArguments);
    renderProcess->waitForFinished();

    progress->setLabel(new QLabel("Converting")) ;
    progress->setValue(4) ;
    QProcess * convertProcess = new QProcess(NULL);
    convertProcess->start(convertPath, convertArguments) ;
    convertProcess->waitForFinished();

    progress->setLabel(new QLabel("Display result")) ;
    progress->setValue(5) ;
    ImageViewer * imageViewer = new ImageViewer(this, Qt::WindowStaysOnTopHint);
    imageViewer->open(pngFile);
    imageViewer->fitToWindow();
    imageViewer->show();

    delete progress ;
    delete renderProcess ;
    delete convertProcess ;
}

BezierClipperToolWidget::BezierClipperToolWidget(MainWindow * mw, QWidget * parent) : QWidget(parent)
{
    mainwindow = mw ;
    ui.setupUi(this) ;
    
    QList<QAxelObject *> l ; l << mw->viewer()->listSelectedObjectsFromType("ImplicitCurve2D") ;
    QList<QImplicitCurve2D *> curves ; foreach(QAxelObject * o, l) curves << dynamic_cast<QImplicitCurve2D *>(o) ;
    
    if(l.size() != 2) {
	QMessageBox::information(0, "Axel","Implicit curves bezier quadratic clipping : Select ONLY two implicit curves","&Ok",0) ;
	return ;
    } else {
	algo = new QBezierClipper(curves[0], curves[1], mw) ;
	algo->setEpsilon(ui.doubleSpinBox->value()) ;
    }

    connect(ui.doubleSpinBox, SIGNAL(valueChanged(double)), algo, SLOT(setEpsilon(double))) ;
    connect(ui.showLinearGraphedSurfaceCheckBox, SIGNAL(toggled(bool)), algo, SLOT(setLinearGraphedSurfacesAreShown(bool))) ;
    connect(ui.showLinearOffsetSurfaceCheckBox, SIGNAL(toggled(bool)), algo, SLOT(setLinearOffsetSurfacesAreShown(bool))) ;
    connect(ui.showQuadraticGraphedSurfaceCheckBox, SIGNAL(toggled(bool)), algo, SLOT(setQuadraticGraphedSurfacesAreShown(bool))) ;
    connect(ui.showQuadraticOffsetSurfaceCheckBox, SIGNAL(toggled(bool)), algo, SLOT(setQuadraticOffsetSurfacesAreShown(bool))) ;
    
    connect(ui.checkBox1, SIGNAL(toggled(bool)), algo, SLOT(setCriticalPointsAreShown(bool))) ;
    connect(ui.checkBox2, SIGNAL(toggled(bool)), algo, SLOT(setOnlyRelevantCriticalPointsAreShown(bool))) ;

    connect(ui.runLinearApproximationButton, SIGNAL(clicked()), algo, SLOT(linearApproximation())) ;
    connect(ui.runQuadraticApproximationButton, SIGNAL(clicked()), algo, SLOT(quadraticApproximation())) ;
    connect(ui.runClippingButton, SIGNAL(clicked()), algo, SLOT(clip())) ;
    connect(ui.runButton, SIGNAL(clicked()), algo, SLOT(run())) ;
}
