/*****************************************************************************
 * A x e l
 *****************************************************************************
 * ViewerDrawingEvents.h
 * 2006-11-08
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# include <QFile>
# include <QCursor>
# include <QAction>
# include <QMenu>
# include <QPainter>

# include <QGui/Viewer.h>

# include <QAxel/QVertex.h>
# include <QAxel/QEdge.h>
# include <QAxel/QFace.h>
# include <QAxel/QMesh.h>

# include <QAxel/PluginManager.h>
# include <QAxel/QBoundingBox.h>
# include <QAxel/QPoint3f.h>
# include <QAxel/QPlane.h>
# include <QAxel/QCurve.h>
# include <QAxel/QSurface.h>

# include <QP5Glove/QP5GloveInterface.h>

void Viewer::draw()
{	
//     QPainter painter;
//     painter.begin(this);
//     painter.setRenderHint(QPainter::Antialiasing);

    if(xyGridIsDrawn_) drawXYGrid() ;
    if(xzGridIsDrawn_) drawXZGrid() ;
    if(yzGridIsDrawn_) drawYZGrid() ;

    for (int i=0; i<objects_.size(); i++) 
	if(!objects_.at(i)->hidden())
	    objects_.at(i)->draw() ;

    for (int i=0; i<temporary_.size(); i++) 
	if(!temporary_.at(i)->hidden())
	    temporary_.at(i)->draw() ;

    foreach(QAxelObject * o, ObjectManager::instance()->objects())
	if(!o->hidden())
	    o->draw() ;

    // Draws manipulatedFrame (the set's rotation center)
    if (manipulatedFrame()->isManipulated() && !selection_.empty()) {
	glPushMatrix();
	glMultMatrixd(manipulatedFrame()->matrix());
	drawAxis(0.5);
	glPopMatrix();
    }

    // Draws rectangular selection area. Could be done in postDraw() instead.
    if (selectionMode_ != NONE) drawSelectionRectangle();

    QP5GloveInterface * iP5Glove = dynamic_cast<QP5GloveInterface *>(PluginManager::instance()->plugin("P5Glove")) ;
    if(iP5Glove) iP5Glove->drawIRs() ;

// 	painter.setRenderHint(QPainter::Antialiasing) ;
// 	painter.drawImage((width() - image.width())/2, 0, image) ;
// 	painter.end() ;
}

void Viewer::drawWithNames() 
{            
    for (int i=0; i<objects_.size(); i++) objects_.at(i)->drawWithNames() ;
    foreach(QAxelObject * o, ObjectManager::instance()->objects())
	if(!o->hidden())
	    o->drawWithNames() ;

}

void Viewer::animate()
{

}

void Viewer::drawXYGrid(float size, int nbSubdivisions) 
{
    glDisable(GL_LIGHTING);
    glLineWidth(1.0);
    glColor3f(0.7,0.7,0.7) ;

    glBegin(GL_LINES);
    for (int i=0; i<=nbSubdivisions; ++i) {
	const float pos = size*(2.0*i/nbSubdivisions-1.0);
	glVertex3f(pos, -size, -1.0);
	glVertex3f(pos, +size, -1.0);
	glVertex3f(-size, pos, -1.0);
	glVertex3f( size, pos, -1.0);
    }
    glEnd();
    glEnable(GL_LIGHTING);
}

void Viewer::drawXZGrid(float size, int nbSubdivisions) 
{
    glDisable(GL_LIGHTING);
    glLineWidth(1.0);
    glColor3f(0.7,0.7,0.7) ;

    glBegin(GL_LINES);
    for (int i=0; i<=nbSubdivisions; ++i) {
	const float pos = size*(2.0*i/nbSubdivisions-1.0);
	glVertex3f(pos, -1.0, -size);
	glVertex3f(pos, -1.0, +size);
	glVertex3f(-size, -1.0, pos);
	glVertex3f( size, -1.0, pos);
    }
    glEnd();
    glEnable(GL_LIGHTING);
}

void Viewer::drawYZGrid(float size, int nbSubdivisions) 
{
    glDisable(GL_LIGHTING);
    glLineWidth(1.0);
    glColor3f(0.7,0.7,0.7) ;
    glBegin(GL_LINES);
    for (int i=0; i<=nbSubdivisions; ++i) {
	const float pos = size*(2.0*i/nbSubdivisions-1.0);
	glVertex3f(-1.0, pos, -size);
	glVertex3f(-1.0, pos, +size);
	glVertex3f(-1.0, -size, pos);
	glVertex3f(-1.0, size, pos);
    }
    glEnd();
    glEnable(GL_LIGHTING);
}
