/*****************************************************************************
 * A x e l
 *****************************************************************************
 * ViewerKeyboardEvents.h
 * 2006-11-08
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# include <QFile>
# include <QCursor>
# include <QAction>
# include <QMenu>
# include <QMessageBox>
# include <QGui/Viewer.h>

# include <QAxel/QVertex.h>
# include <QAxel/QEdge.h>
# include <QAxel/QFace.h>
# include <QAxel/QMesh.h>
# include <QAxel/PluginManager.h>

# include <QAxel/QBoundingBox.h>
# include <QAxel/QPoint3f.h>
# include <QAxel/QPlane.h>
# include <QAxel/QRationalCurve.h>
# include <QAxel/QBSplineCurve.h>
# include <QAxel/QNurbsCurve.h>
# include <QAxel/QImplicitCurve2D.h>
# include <QAxel/QImplicitCurve3D.h>
# include <QAxel/QImplicitCurve4D.h>
# include <QAxel/QRationalSurface.h>
# include <QAxel/QBSplineSurface.h>
# include <QAxel/QNurbsSurface.h>
# include <QAxel/QImplicitSurface.h>

# include <QSisl/QSislInterface.h>

void Viewer::resizeGL(int width,int  height)
{
    formatInstructions(width, height) ;
    QGLViewer::resizeGL(width, height) ;      
}

void Viewer::keyPressEvent(QKeyEvent *e) 
{
    if(underMouse()) {
		
	const Qt::KeyboardModifiers modifiers = e->modifiers();

	// A simple switch on e->key() is not sufficient if we want to take state key into account.
	// With a switch, it would have been impossible to separate 'F' from 'CTRL+F'.
	// That's why we use imbricated if...else and a "handled" boolean.
	bool handled = false;
	
	
	/****************************************************************
	 * Key X
	 ****************************************************************/
	
	if ((e->key()==Qt::Key_X)) {

	    QList<QAxelObject * > l = listEditedObjectsFromType("BSplineCurve") ;
	    
	    if(l.size() == 1) {
		     
		QBSplineCurve * c = qobject_cast<QBSplineCurve *>(l.first()) ;
		
		QList<QAxelObject * > cps = listSelectedObjectsFromType("Point3f") ;
		
		for(int i = 0 ; i < cps.size() ; i++) {
		    QPoint3f * cp = qobject_cast<QPoint3f *>(cps[i]) ;
		    removeObjectFromSelection(cp) ;
		    c->points.removeAll(cp) ;
		    c->computeKnots() ;
		}
		
		c->compute() ;
	    }
	    
	    removeSelectedObjects() ;
	    update() ;
	    
	    handled = true ;
	} 

	/****************************************************************
	 * Key E
	 ****************************************************************/

	else if ((e->key()==Qt::Key_E)) { 
	    
	    QList<QAxelObject *> l = listEditedObjectsFromType("BSplineCurve") ;
	    if(l.size() == 1) {
		
		QBSplineCurve * c = dynamic_cast<QBSplineCurve *>(l.first()) ;
		
		QList<QAxelObject *> cps = listSelectedObjectsFromType("Point3f") ;
		
		if(cps.size() > 1) 
		    return ;
	       
		QPoint3f * p = dynamic_cast<QPoint3f *>(cps.first()) ;

		if(p != c->points.first() && p != c->points.last()) 
		    return ;
		
		QPoint3f * np = new QPoint3f(p->x(),p->y()) ;

		removeObjectFromSelection(p) ;
		addObjectToSelection(np) ;
		
		if(p == c->points.first()) 
		    c->push_front(np) ;
		else
		    c->push_point(np) ;
		
		c->computeKnots() ;
		c->compute() ;
		c->setMode(QAxelObject::Extrude) ;
		
		connect(this, SIGNAL(mouseMoved(QPoint3f *)), np, SLOT(setPosition(QPoint3f *))) ;
		setMouseTracking(true) ;
		update() ;

	    }

	    
	    l = listEditedObjectsFromType("BSplineSurface") ;
	    
	    if(l.size() == 1) {

//		cout << __func__ << " entered extrusion mode " << endl ;
		
		QBSplineSurface * s = dynamic_cast<QBSplineSurface *>(l.first()) ;
		
		QList<QAxelObject *> cps = listSelectedObjectsFromType("Point3f") ;
		
//		cout << "list of " << cps.size() << " control points" << endl ;

		if(cps.empty()) return ;

		QBSplineSurface::ExtrusionEdge side = s->extrusionPossible() ;
		
		int& number1 = s->number1 ;
		int& number2 = s->number2 ;

		QList<QPoint3f *>& points = s->points ;

		clearSelection() ;
		
		switch(side) {
		    
		case QBSplineSurface::UP :

		    for(int i = number1 - 1; i >= 0; i--) {
			QPoint3f * np = new QPoint3f(points[number1-1]->x(),
						     points[number1-1]->y(),
						     points[number1-1]->z()) ;
			
			s->push_point(np, 0) ;
			addObjectToSelection(np) ;
		    }
		    
		    number2++ ;
//		    cout << __func__ << " : Up side extruded : " << endl ;
		    break ;

		case QBSplineSurface::DOWN :

		    for(int i = 0; i < number1; i++) {
			QPoint3f * np = new QPoint3f(points[number1*(number2-1)+i]->x(), 
						     points[number1*(number2-1)+i]->y(), 
						     points[number1*(number2-1)+i]->z()) ;

			s->push_point(np, points.size()) ;
			addObjectToSelection(np) ;
		    }
		    
		    number2++ ;
//		    cout << __func__ << " : Down side extruded : " << endl ;
		    break ;

		case QBSplineSurface::LEFT :

		    for(int i = 0; i < number2; i++) {
			QPoint3f * np = new QPoint3f(points[number1*i+i]->x(),
						     points[number1*i+i]->y(),
						     points[number1*i+i]->z()) ;

			s->push_point(np, number1*i+i) ;
			addObjectToSelection(np) ;
		    }
		    
		    number1++ ;
//		    cout << __func__ << " : left side extruded : " << endl ;
		    break ;

		case QBSplineSurface::RIGHT :

//		    cout << __func__ << " : entered right case : " << endl ;
		    for(int i = 0; i < number2; i++) {
//			cout << __func__ << " : index value : " <<  number1*(i+1)-1+i << endl ;
			QPoint3f * np = new QPoint3f(points[number1*(i+1)-1+i]->x(),
						     points[number1*(i+1)-1+i]->y(),
						     points[number1*(i+1)-1+i]->z()) ;

//			cout << __func__ << " : new point created : " << np << endl ;
			s->push_point(np, number1*(i+1)+i) ;
			addObjectToSelection(np) ;
		    }
		    
		    number1++ ;
//		    cout << __func__ << " : Right side extruded : " << endl ;
		    break ;

		case QBSplineSurface::NONE :
//		    cout << __func__ << " : No side extruded : " << endl ;
		    return ;

		default :
//		    cout << __func__ << " : unexpected extrusion side : " << endl ;
		    return ;
		}

//  		foreach(QPoint3f * p, s->points)
//  		    cout << __func__ << " : points list : " << p << endl ;

//		cout << __func__ << " : list size : " << s->points.size() << endl ;

		s->computeKnots1() ;
		s->computeKnots2() ;

// 		foreach(QPoint3f * p, s->points)
// 		    p->setBlackListed(true) ;

		s->compute() ;
		s->setMode(QAxelObject::Extrude) ;

		setMouseTracking(true) ;

		mf_move = true ;
		//connect(this, SIGNAL(prout(qglviewer::Vec v)), manipulatedFrame(), SLOT(translate(v))) ;

		update() ;

	    }
		
		
	    l = listExtrusionObjects() ;

//	    cout << "Extrude Key Pressed : extrusion objects size : " << l.size() << endl ;
		  
	    handled = true ;
	}

	/********************************************************
	 * Key G
	 ****************************************************************/
	
	else if (e->key()==Qt::Key_G) { 

	    QList<QAxelObject *> l = listSelectedObjects() ;

	    if(l.size() != 1) {
		QMessageBox::information(0, "Axel","Grab implemented only for a single point right now","&Ok",0) ;		
		return ;
	    }
	    
	    foreach(QAxelObject * o , l) 
		o->setMode(QAxelObject::Grab) ;
	
	    setMouseTracking(true) ;

	}

	/****************************************************************
	 * Key J
	 ****************************************************************/
		
	else if ((e->key()==Qt::Key_J))
	    joinBSplineCurves() ;

	/****************************************************************
	 * Key W
	 ****************************************************************/
		
	else if ((e->key()==Qt::Key_W)) { 

	    QList<QAxelObject *> l = listEditedObjectsFromType("BSplineCurve") ;

	    if(l.size() > 1)
		return ;

	    QBSplineCurve * c = qobject_cast<QBSplineCurve *>(l.first()) ;
	    
	    QList<QAxelObject *> pts = listSelectedObjectsFromType("Point3f") ;
	    
	    if(pts.size() != 2)
		return ;

	    QPoint3f * p1 = qobject_cast<QPoint3f *>(pts.first()) ;
	    QPoint3f * p2 = qobject_cast<QPoint3f *>(pts.last()) ;

	    if(c->points.indexOf(p2)-c->points.indexOf(p1) == 1) {
		
		QPoint3f * np = new QPoint3f((p1->x()+p2->x())/2, (p1->y()+p2->y())/2) ;
		c->insert(c->points.indexOf(p2), np) ;
		addObjectToSelection(np) ;

	    } else if(c->points.indexOf(p2)-c->points.indexOf(p1) == -1) {
		
		QPoint3f * np = new QPoint3f((p1->x()+p2->x())/2, (p1->y()+p2->y())/2) ;
		c->insert(c->points.indexOf(p1), np) ;
		addObjectToSelection(np) ;

	    } else
		return ;

	    removeObjectFromSelection(p1) ;
	    removeObjectFromSelection(p2) ;
	    	    
	    c->computeKnots() ;
	    c->compute() ;
	    update() ;
	    
	}
	
	/****************************************************************
	 * Key TAB
	 ****************************************************************/
		
	else if ((e->key()==Qt::Key_Tab)) { 
	    
	    QList<QAxelObject *> l = listSelectedObjectsFromType("BSplineCurve") ;

	    bool allToEdit = true ;
	    for(int i=0 ; i < l.size() ; i++)
		if(((QBSplineCurve *)l[i])->mode() == QAxelObject::Edit) 
		    allToEdit = false ;

	    for(int i=0 ; i < l.size() ; i++) {
		QBSplineCurve * bsc =  (QBSplineCurve *)l[i] ;
		if(allToEdit) { 
		    bsc->setMode(QAxelObject::Edit) ;
		    foreach(QPoint3f * p, bsc->points)
			p->deselect() ;
		    updateSelection() ;
		} else { 
		    bsc->setMode(QAxelObject::None) ;
		    foreach(QPoint3f * p, bsc->points)
			p->deselect() ;
		    updateSelection() ;
		}
	    }

	    l.empty() ; l = listSelectedObjectsFromType("BSplineSurface") ;

	    allToEdit = true ;
	    for(int i=0 ; i < l.size() ; i++)
		if(((QBSplineSurface *)l[i])->mode() == QAxelObject::Edit) 
		    allToEdit = false ;

	    for(int i=0 ; i < l.size() ; i++) {
		QBSplineSurface * bsc =  (QBSplineSurface *)l[i] ;
		if(allToEdit) { 
		    bsc->setMode(QAxelObject::Edit) ;
		    foreach(QPoint3f * p, bsc->points)
			    p->deselect() ;
		    updateSelection() ;
		} else { 
		    bsc->setMode(QAxelObject::None) ;
		    foreach(QPoint3f * p, bsc->points)
			p->deselect() ;
		    
		    updateSelection() ;
		}
	    }
			
	    l.clear() ; l = listSelectedObjectsFromType("Mesh") ;
	    // If a mesh is already selected among the selection, its edition mode is
	    // set to None. Otherwise, all selected meshes are set to edited.
	    allToEdit = true ;
	    for(int i=0 ; i < l.size() ; i++) {
		if(((QMesh *)l[i])->mode() == QMesh::Edit) allToEdit = false ;
	    }                                           
	    for(int i=0 ; i < l.size() ; i++) {
		if(allToEdit) 	{ 
		    ((QMesh *)l[i])->setMode(QMesh::Edit) ;
		} else { 
		    ((QMesh *)l[i])->setMode(QMesh::None) ;
		    QMesh * m = (QMesh *)l[i] ;
		    for(int i=0 ; i<m->vertices_.size() ; i++) m->vertices_[i]->deselect() ;
		    for(int i=0 ; i<m->edges_.size() ; i++) m->edges_[i]->deselect() ;
		    for(int i=0 ; i<m->faces_.size() ; i++) m->faces_[i]->deselect() ;
		    updateSelection() ;
		}
	    }

	    updateGL() ;
	    handled = true ;
	} 
		
	/****************************************************************
	 * Key I : Intersection Menu
	 ****************************************************************/
		
	else if ((e->key()==Qt::Key_I)) {			
	    QMenu menu("Intersection", this) ;
	    menu.setWindowTitle("Intersection Tools") ;
	    menu.setWindowOpacity(0.8) ;
	    menu.setWindowFlags(Qt::Tool) ;
	    menu.addAction(autoIntersectParametricSurfacesAction) ;
	    menu.addAction(intersectBSplineCurvesAction) ;
	    menu.addAction(intersectImplicitCurvesAction) ;
	    menu.addAction(intersectImplicitSurfacesAction) ;
	    menu.addAction(intersectRationalSurfacesAction) ;
	    menu.addAction(intersectParametricSurfacesAction) ;
	    menu.addAction(quadClipImplicitCurvesAction) ;
	    menu.exec(mapFrom(this, QCursor::pos())) ;
	    updateGL() ;
	    handled = true ;
	}

	/****************************************************************
	 * Key A : Approximation Menu
	 ****************************************************************/
	
	else if ((e->key()==Qt::Key_A)) {			
	    QMenu menu("Approximation", this) ;
	    menu.setWindowTitle("Approximation Tools") ;
	    menu.setWindowOpacity(0.8) ;
	    menu.setWindowFlags(Qt::Tool) ;
	    menu.addAction(approximatePointsAsBSplineAction) ;
	    menu.exec(mapFrom(this, QCursor::pos())) ;
	    updateGL() ;
	    handled = true ;
	}
		
	/****************************************************************
	 * Key P : Interpolation Menu
	 ****************************************************************/
		
	else if ((e->key()==Qt::Key_P)) {			
	    QMenu menu("Interpolation", this) ;
	    menu.setWindowTitle("Interpolation Tools") ;
	    menu.setWindowOpacity(0.8) ;
	    menu.setWindowFlags(Qt::Tool) ;
	    menu.addAction(interpolatePointsAsBSplineAction) ;
	    menu.exec(mapFrom(this, QCursor::pos())) ;
	    updateGL() ;
	    handled = true ;
	}
	
	/****************************************************************
	 * Key O : Offset Menu
	 ****************************************************************/
	
	else if ((e->key()==Qt::Key_O)) {
	    QMenu menu("Offset", this) ;
	    menu.setWindowTitle("Offset Tools") ;
	    menu.setWindowOpacity(0.8) ;
	    menu.setWindowFlags(Qt::Tool) ;
	    menu.addAction(offsetBSplineCurvesAction) ;
	    menu.exec(mapFrom(this, QCursor::pos())) ;
	    updateGL() ;
	    handled = true ;
	}
		
	/****************************************************************
	 * Key R : Refine Menu
	 ****************************************************************/

	else if ((e->key()==Qt::Key_R)) {

	    QMenu menu("Refinement", this) ;
	    menu.setWindowTitle("Refinement Tools") ;
	    menu.setWindowOpacity(0.8) ;
	    menu.setWindowFlags(Qt::Tool) ;
	    menu.addAction(refineImplicitCurveAction) ;
	    menu.exec(mapFrom(this, QCursor::pos())) ;
	    updateGL() ;
	    handled = true ;
	}
		
	/****************************************************************
	 * Key B : Blending Menu
	 ****************************************************************/

	else if ((e->key()==Qt::Key_B)) {

	    QMenu menu("Blending", this) ;
	    menu.setWindowTitle("Blending Tools") ;
	    menu.setWindowOpacity(0.8) ;
	    menu.setWindowFlags(Qt::Tool) ;
	    menu.addAction(blendBSplineCurvesAction) ;
	    menu.exec(mapFrom(this, QCursor::pos())) ;
	    updateGL() ;
	    handled = true ;
	}
  
	if (!handled) QGLViewer::keyPressEvent(e) ;
    }
}

bool Viewer::event(QEvent *event)
{
    if(underMouse() && event->type() == QEvent::KeyPress) {
	QKeyEvent *ke = static_cast<QKeyEvent *>(event);
	if (ke->key() == Qt::Key_Tab) {
	    keyPressEvent(ke) ;
	    return true;
	}
    }

    return QWidget::event(event);
}

