/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QRationalSurfaceIntersection
 * 2007-02-24
 * Stephane Chau
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * QRationalSurfaceIntersection * algo = new QRationalSurfaceIntersection(s1, s2) ;
 * QCurve * c = algo->run() ;
 *****************************************************************************/

# include <QString>

# include <QIntersection/Surfaces/QRationalSurfaceIntersection.h>
# include <QIntersection/Surfaces/testing.h>

# include <QAxel/PluginManager.h>

# include <QSynaps/QSynapsInterface.h>

# include <QData/QCell4D.h>


using namespace std ;

namespace QIntersection {

QRationalSurfaceIntersection::QRationalSurfaceIntersection(QRationalSurface * s1, QRationalSurface * s2, MainWindow * mw) : QSurfaceIntersection(mw) 
{
	// cout << "Debut de QRationalSurfaceIntersection" << endl ;
	this->s1 = s1 ;
	this->s2 = s2 ;
	this->curve = getIntersectionCurve4D() ;
	this->sbd_epsilon = 0.1;//1e-2;
	this->sbd_epsilon_max = 1;
    
	
}

QRationalSurfaceIntersection::~QRationalSurfaceIntersection(void) 
{
	
}

QImplicitCurve4D * QRationalSurfaceIntersection::getIntersectionCurve4D(void)
{
	QSynapsInterface * iSynaps = dynamic_cast<QSynapsInterface *>(PluginManager::instance()->plugin("Synaps")) ;
	QImplicitCurve4D * res = iSynaps->computeIntersectionEquation4D(s1,s2) ;

	// cout << "getIntersectionCurve4D dans la classe QRationalSurfaceIntersection" << endl ;
	// cout << res->umin() << endl;
	// cout << res->umax() << endl;
	// cout << res->vmin() << endl;
	// cout << res->vmax() << endl;
	// cout << res->f().toLatin1().data() << endl;
	// cout << res->g().toLatin1().data() << endl;
	// cout << res->h().toLatin1().data() << endl;
	
	return res;
}

bool QRationalSurfaceIntersection::isRegular(QImplicitCurve4D * c, Node_t * no, int i)
{
	if (i!=0 && i!=1 && i!=2 && i!=3) qCritical("Bad choice of integer for the variable in function isRegular");
	QSynapsInterface * iSynaps = dynamic_cast<QSynapsInterface *>(PluginManager::instance()->plugin("Synaps")) ;
	QBoundingBox4D * bb = no->cell;
	if (!(iSynaps->isLocallyInjective4D(c, bb, i))) return false;
	else
	{
		if (iSynaps->isEdgeMonotonic4D(c->f(),c->var(),i,bb))
			if (iSynaps->isBorderMonotonic4D(c->f(),c->g(),c->var(),i,bb) || iSynaps->isBorderMonotonic4D(c->f(),c->h(),c->var(),i,bb)) return true;
			else return false;
		else if (iSynaps->isEdgeMonotonic4D(c->g(),c->var(),i,bb))
			if (iSynaps->isBorderMonotonic4D(c->g(),c->f(),c->var(),i,bb) || iSynaps->isBorderMonotonic4D(c->g(),c->h(),c->var(),i,bb)) return true;
			else return false;
		else if (iSynaps->isEdgeMonotonic4D(c->h(),c->var(),i,bb))
			if (iSynaps->isBorderMonotonic4D(c->h(),c->f(),c->var(),i,bb) || iSynaps->isBorderMonotonic4D(c->h(),c->g(),c->var(),i,bb)) return true;
			else return false;
		else return false;
	}
}

// QList<Cell_t *> QRationalSurfaceIntersection::subdivide(QImplicitCurve4D * c, Node_t * n)
void QRationalSurfaceIntersection::subdivide(QImplicitCurve4D * c, Node_t * n)
{
	QSynapsInterface * iSynaps = dynamic_cast<QSynapsInterface *>(PluginManager::instance()->plugin("Synaps")) ;
    Cell_t * cell = n->GetCell();
    QList<Cell_t *> lcells;
	QList<QPoint4f *> lpts;
	int faceBorder[2];
	
	// if (n->parent == NULL) // then we have to compute all the border points.
	// {
	// 	faceBorder[0] = 0; faceBorder[1] = 0; 
	// 	lpts = iSynaps->intersectionPoints4D(c, cell, faceBorder);
	// 	cell->setSMinIntersections(lpts);
	// 	
	// 	faceBorder[0] = 0; faceBorder[1] = 1; 
	// 	lpts = iSynaps->intersectionPoints4D(c, cell, faceBorder);
	// 	cell->setSMaxIntersections(lpts);
	// 	
	// 	faceBorder[0] = 1; faceBorder[1] = 0; 
	// 	lpts = iSynaps->intersectionPoints4D(c, cell, faceBorder);
	// 	cell->setTMinIntersections(lpts);
	// 	
	// 	faceBorder[0] = 1; faceBorder[1] = 1; 
	// 	lpts = iSynaps->intersectionPoints4D(c, cell, faceBorder);
	// 	cell->setTMaxIntersections(lpts);
	// 	
	// 	faceBorder[0] = 2; faceBorder[1] = 0; 
	// 	lpts = iSynaps->intersectionPoints4D(c, cell, faceBorder);
	// 	cell->setUMinIntersections(lpts);
	// 	
	// 	faceBorder[0] = 2; faceBorder[1] = 1; 
	// 	lpts = iSynaps->intersectionPoints4D(c, cell, faceBorder);
	// 	cell->setUMaxIntersections(lpts);
	// 	
	// 	faceBorder[0] = 3; faceBorder[1] = 0; 
	// 	lpts = iSynaps->intersectionPoints4D(c, cell, faceBorder);
	// 	cell->setVMinIntersections(lpts);
	// 	
	// 	faceBorder[0] = 3; faceBorder[1] = 1; 
	// 	lpts = iSynaps->intersectionPoints4D(c, cell, faceBorder);
	// 	cell->setVMaxIntersections(lpts);
	// }

//Subdivision step the node has a parent so we have to use its border points plus subdivided points.
	float smid = (cell->smax() + cell->smin())/2; //cout << __func__ << " : smin = " << cell->smin() << "; smid = " << smid << "; smax = " << cell->smax() << endl;
	float tmid = (cell->tmax() + cell->tmin())/2; //cout << __func__ << " : tmin = " << cell->tmin() << "; tmid = " << tmid << "; tmax = " << cell->tmax() << endl;
	float umid = (cell->umax() + cell->umin())/2; //cout << __func__ << " : umin = " << cell->umin() << "; umid = " << umid << "; umax = " << cell->umax() << endl;
	float vmid = (cell->vmax() + cell->vmin())/2; //cout << __func__ << " : vmin = " << cell->vmin() << "; vmid = " << vmid << "; vmax = " << cell->vmax() << endl;
	QPoint4f * center = new QPoint4f(smid, tmid, umid, vmid);
	QList<Cell_t *> childCells = cell->subdivide4D(center);

	for (int i = 0; i < childCells.size(); ++i)
		n->SetChild(i, new Node_t(i, n, childCells[i]));
		
	lpts = cell->sMinIntersections();
	int code = 0; // it corresponds to a child (e.g code == 4 corresponds to the child LRLL, code == 10 corresponds to the child RLRL)
	for (int i = 0; i < lpts.size(); ++i) 
	{
		if (lpts[i]->t() > tmid) code+=4 ;
		if (lpts[i]->u() > umid) code+=2 ;
		if (lpts[i]->v() > vmid) code+=1 ;
		n->GetChild(code)->GetCell()->pushSMinIntersection(lpts[i]);  code = 0;
	 }
	
	lpts = cell->sMaxIntersections();
	code = 8;
	for (int i = 0; i < lpts.size(); ++i) 
	{
		if (lpts[i]->t() > tmid) code+=4 ;
		if (lpts[i]->u() > umid) code+=2 ;
		if (lpts[i]->v() > vmid) code+=1 ;
		n->GetChild(code)->GetCell()->pushSMaxIntersection(lpts[i]); code = 8;
	 }
	
	lpts = cell->tMinIntersections();
	code = 0;
	for (int i = 0; i < lpts.size(); ++i) 
	{
		if (lpts[i]->s() > smid) code+=8 ;
		if (lpts[i]->u() > umid) code+=2 ;
		if (lpts[i]->v() > vmid) code+=1 ;
		n->GetChild(code)->GetCell()->pushTMinIntersection(lpts[i]);  code = 0;
	 }

	lpts = cell->tMaxIntersections();
	code = 4;
	for (int i = 0; i < lpts.size(); ++i) 
	{
		if (lpts[i]->s() > smid) code+=8 ;
		if (lpts[i]->u() > umid) code+=2 ;
		if (lpts[i]->v() > vmid) code+=1 ;
		n->GetChild(code)->GetCell()->pushTMaxIntersection(lpts[i]);  code = 4;
	 }

	lpts = cell->uMinIntersections();
	code = 0;
	for (int i = 0; i < lpts.size(); ++i) 
	{
		if (lpts[i]->t() > tmid) code+=4 ;
		if (lpts[i]->s() > smid) code+=8 ;
		if (lpts[i]->v() > vmid) code+=1 ;
		n->GetChild(code)->GetCell()->pushUMinIntersection(lpts[i]);  code = 0;
	 }

	lpts = cell->uMaxIntersections();
	code = 2;
	for (int i = 0; i < lpts.size(); ++i) 
	{
		if (lpts[i]->t() > tmid) code+=4 ;
		if (lpts[i]->s() > smid) code+=8 ;
		if (lpts[i]->v() > vmid) code+=1 ;
		n->GetChild(code)->GetCell()->pushUMaxIntersection(lpts[i]);  code = 2;
	 }

	lpts = cell->vMinIntersections();
	code = 0;
	for (int i = 0; i < lpts.size(); ++i) 
	{
		if (lpts[i]->t() > tmid) code+=4 ;
		if (lpts[i]->u() > umid) code+=2 ;
		if (lpts[i]->s() > smid) code+=8 ;
		n->GetChild(code)->GetCell()->pushVMinIntersection(lpts[i]);  code = 0;
	 }

	lpts = cell->vMaxIntersections();
	code = 1;
	for (int i = 0; i < lpts.size(); ++i) 
	{
		if (lpts[i]->t() > tmid) code+=4 ;
		if (lpts[i]->u() > umid) code+=2 ;
		if (lpts[i]->s() > smid) code+=8 ;
		n->GetChild(code)->GetCell()->pushVMaxIntersection(lpts[i]);  code = 1;
	 }
//  and the mid points...

	QBoundingBox4D * bb = new QBoundingBox4D(cell->smin(),smid,cell->tmin(),cell->tmax(),cell->umin(),cell->umax(),cell->vmin(),cell->vmax()); // s = smid
	faceBorder[0] = 0; faceBorder[1] = 1; 
	lpts = iSynaps->intersectionPoints4D(c, bb, faceBorder);
	code = 0;
	for (int i = 0; i < lpts.size(); ++i) 
	{
		if (lpts[i]->t() > tmid) code+=4 ;
		if (lpts[i]->u() > umid) code+=2 ;
		if (lpts[i]->v() > vmid) code+=1 ;
		n->GetChild(code)->GetCell()->pushSMaxIntersection(lpts[i]);
		n->GetChild(8+code)->GetCell()->pushSMinIntersection(lpts[i]);  code = 0;
	 }
	delete bb;
	
	bb = new QBoundingBox4D(cell->smin(),cell->smax(),cell->tmin(),tmid,cell->umin(),cell->umax(),cell->vmin(),cell->vmax()); // t = tmid
	faceBorder[0] = 1; faceBorder[1] = 1; 
	lpts = iSynaps->intersectionPoints4D(c, bb, faceBorder);
	code = 0;
	for (int i = 0; i < lpts.size(); ++i) 
	{
		if (lpts[i]->s() > smid) code+=8 ;
		if (lpts[i]->u() > umid) code+=2 ;
		if (lpts[i]->v() > vmid) code+=1 ;
		n->GetChild(code)->GetCell()->pushTMaxIntersection(lpts[i]);
		n->GetChild(4+code)->GetCell()->pushTMinIntersection(lpts[i]);  code = 0;
	 }
	delete bb;

	bb = new QBoundingBox4D(cell->smin(),cell->smax(),cell->tmin(),cell->tmax(),cell->umin(),umid,cell->vmin(),cell->vmax()); // u = umid
	faceBorder[0] = 2; faceBorder[1] = 1; 
	lpts = iSynaps->intersectionPoints4D(c, bb, faceBorder);
	code = 0;
	for (int i = 0; i < lpts.size(); ++i) 
	{
		if (lpts[i]->s() > smid) code+=8 ;
		if (lpts[i]->t() > tmid) code+=4 ;
		if (lpts[i]->v() > vmid) code+=1 ;
		n->GetChild(code)->GetCell()->pushUMaxIntersection(lpts[i]);
		n->GetChild(2+code)->GetCell()->pushUMinIntersection(lpts[i]);  code = 0;
	 }
	delete bb;

	bb = new QBoundingBox4D(cell->smin(),cell->smax(),cell->tmin(),cell->tmax(),cell->umin(),cell->umax(),cell->vmin(),vmid); // v = vmid
	faceBorder[0] = 3; faceBorder[1] = 1; 
	lpts = iSynaps->intersectionPoints4D(c, bb, faceBorder);
	code = 0;
	for (int i = 0; i < lpts.size(); ++i) 
	{
		if (lpts[i]->s() > tmid) code+=8 ;
		if (lpts[i]->t() > umid) code+=4 ;
		if (lpts[i]->u() > smid) code+=2 ;
		n->GetChild(code)->GetCell()->pushVMaxIntersection(lpts[i]);
		n->GetChild(1+code)->GetCell()->pushVMinIntersection(lpts[i]);  code = 0;
	 }
	delete bb;
}


/*############## Sort a liste of points with respect to the variable i. ############## */

// At first the order relations :

// bool order(Point_t * p1, Point_t * p2, int i)
// {
// 	if (i!=0 && i!=1 && i!=2 && i!=3) qCritical("Bad choice of integer for the variable in function order");
// 	switch(i)
// 	{
// 		case 0: return(p1->s() < p2->s());
// 		break;
// 		case 1: return(p1->t() < p2->t());
// 		break;
// 		case 2: return(p1->u() < p2->u());
// 		break;
// 		case 3: return(p1->v() < p2->v());
// 		break;
// 		default: return false;
// 		break;
// 	}
// }
bool order0(Point_t * p1, Point_t * p2) { return(p1->s() < p2->s()); }
bool order1(Point_t * p1, Point_t * p2) { return(p1->t() < p2->t()); }
bool order2(Point_t * p1, Point_t * p2) { return(p1->u() < p2->u()); }
bool order3(Point_t * p1, Point_t * p2) { return(p1->v() < p2->v()); }

// And the corresponding sort functions :
void sort0(QList<Point_t *> l) { std::sort(l.begin(), l.end(), order0); }
void sort1(QList<Point_t *> l) { std::sort(l.begin(), l.end(), order1); }
void sort2(QList<Point_t *> l) { std::sort(l.begin(), l.end(), order2); }
void sort3(QList<Point_t *> l) { std::sort(l.begin(), l.end(), order3); }


//Then the global sort function :
void QRationalSurfaceIntersection::sort4D(QList<Point_t *> l, int i)
{
	if (i!=0 && i!=1 && i!=2 && i!=3) qCritical("Bad choice of integer for the variable in function sort");
	switch(i)
	{
		case 0: sort0(l);
		break;
		case 1: sort1(l);
		break;
		case 2: sort2(l);
		break;
		case 3: sort3(l);
		break;
		default:
		break;
	}
}

// Topology in the regular case
QList<Segment_t> QRationalSurfaceIntersection::topology(Node_t * no, int i)
{
	if (i!=0 && i!=1 && i!=2 && i!=3) qCritical("Bad choice of integer for the variable in function topology");
	QList<Segment_t> ls;// = QList<Segment_t>();
	QList<Point_t * > lp = no->cell->allIntersections();
	sort4D(lp,i);
	if (lp.size() & 1) qCritical("Presence of mutiple points on the border in the topology computation");

	for (int i = 0; i <= lp.size() - 2; i=i+2) 
	{
		ls << qMakePair(lp[i],lp[i+1]);
	 }
	return ls;
}

QList<Segment_t> QRationalSurfaceIntersection::getTopology4D(QImplicitCurve4D * c)
{
// Initialization of the cell root by computing all the intersection of c with the border of its bounding box.
	Cell_t * cell = new QCell4D(c->bb()) ;
	QSynapsInterface * iSynaps = dynamic_cast<QSynapsInterface *>(PluginManager::instance()->plugin("Synaps")) ;
	QList<QPoint4f *> lpts;
	int faceBorder[2];
	faceBorder[0] = 0; faceBorder[1] = 0;
	lpts = iSynaps->intersectionPoints4D(c, cell, faceBorder);
	cell->setSMinIntersections(lpts);
	
	faceBorder[0] = 0; faceBorder[1] = 1; 
	lpts = iSynaps->intersectionPoints4D(c, cell, faceBorder);
	cell->setSMaxIntersections(lpts);
	
	faceBorder[0] = 1; faceBorder[1] = 0; 
	lpts = iSynaps->intersectionPoints4D(c, cell, faceBorder);
	cell->setTMinIntersections(lpts);
	
	faceBorder[0] = 1; faceBorder[1] = 1; 
	lpts = iSynaps->intersectionPoints4D(c, cell, faceBorder);
	cell->setTMaxIntersections(lpts);
	
	faceBorder[0] = 2; faceBorder[1] = 0; 
	lpts = iSynaps->intersectionPoints4D(c, cell, faceBorder);
	cell->setUMinIntersections(lpts);
	
	faceBorder[0] = 2; faceBorder[1] = 1; 
	lpts = iSynaps->intersectionPoints4D(c, cell, faceBorder);
	cell->setUMaxIntersections(lpts);
	
	faceBorder[0] = 3; faceBorder[1] = 0; 
	lpts = iSynaps->intersectionPoints4D(c, cell, faceBorder);
	cell->setVMinIntersections(lpts);
	
	faceBorder[0] = 3; faceBorder[1] = 1; 
	lpts = iSynaps->intersectionPoints4D(c, cell, faceBorder);
	cell->setVMaxIntersections(lpts);
	// cout << __func__ << " - Initialisation..." << endl; getchar();
// Construction of an hexatree which root is constructed by using the bounding box of c.
    Hexatree_t hexatree ;
    Node_t * root = hexatree.GetRoot() ;
    root->SetCell(cell) ;

	// Construction of a list of nodes initialized by the root of the hexatree, it will be used to test the regularity criterion.
	QList<Node_t *> nodes ;
    nodes.push_back(root) ;    
    while(!nodes.empty()) 
	{// cout << __func__ << " nodes.size() = " << nodes.size() << endl ;
		Node_t * no = nodes.takeFirst() ; // in the beginning, it's the root of the hexatree, we use it to construct the hexatree.
		Cell_t * bb = no->cell ;
		
		if ( (bb->size() > sbd_epsilon) && (bb->size() < sbd_epsilon_max) ) // Condition to test the regularity
		{
			if (isRegular(c, no, 0))
				no->objects << topology(no, 0) ;
			else if (isRegular(c, no, 1))
				no->objects << topology(no, 1) ;
			else if (isRegular(c, no, 2))
				no->objects << topology(no, 2) ;
			else if (isRegular(c, no, 3))
				no->objects << topology(no, 3) ;
			else
			{
				subdivide(c, no); 
				nodes << no->LLLLchild << no->LLLRchild << no->LLRLchild << no->LLRRchild 
						<< no->LRLLchild << no->LRLRchild << no->LRRLchild << no->LRRRchild
						<< no->RLLLchild << no->RLLRchild << no->RLRLchild << no->RLRRchild
						<< no->RRLLchild << no->RRLRchild << no->RRRLchild << no->RRRRchild;
			}	
		}
		else // (bb->size() <= sbd_epsilon) || (bb->size() >= sbd_epsilon_max)
		{
			if (bb->size() <= sbd_epsilon)
			{
				qCritical("Beyond minimum subdivision size, generating dummy topology") ;
				no->objects << dummy(no) ;
			}
			else 
			{
				subdivide(c, no);
	    	// QList<Cell_t *> cells = subdivide(c, no) ;
			// The neighborhood is already set for the nodes, so, each functions using neighborhood relations have to use a node instead of a cell

			// no->LLLLchild = new Node_t(LLLL, no, cells[0]);
			// no->LLLRchild = new Node_t(LLLR, no, cells[1]);
			// no->LLRLchild = new Node_t(LLRL, no, cells[2]);
			// no->LLRRchild = new Node_t(LLRR, no, cells[3]);
			// no->LRLLchild = new Node_t(LRLL, no, cells[4]);
			// no->LRLRchild = new Node_t(LRLR, no, cells[5]);
			// no->LRRLchild = new Node_t(LRRL, no, cells[6]);
			// no->LRRRchild = new Node_t(LRRR, no, cells[7]);
			// no->RLLLchild = new Node_t(RLLL, no, cells[8]);
			// no->RLLRchild = new Node_t(RLLR, no, cells[9]);
			// no->RLRLchild = new Node_t(RLRL, no, cells[10]);
			// no->RLRRchild = new Node_t(RLRR, no, cells[11]);
			// no->RRLLchild = new Node_t(RRLL, no, cells[12]);
			// no->RRLRchild = new Node_t(RRLR, no, cells[13]);
			// no->RRRLchild = new Node_t(RRRL, no, cells[14]);
			// no->RRRRchild = new Node_t(RRRR, no, cells[15]); // Now we make this directly in the subdivide function.
			                            
				nodes << no->LLLLchild << no->LLLRchild << no->LLRLchild << no->LLRRchild 
						<< no->LRLLchild << no->LRLRchild << no->LRRLchild << no->LRRRchild
						<< no->RLLLchild << no->RLLRchild << no->RLRLchild << no->RLRRchild
						<< no->RRLLchild << no->RRLRchild << no->RRRLchild << no->RRRRchild;
			} 
    	}
	}
    return fusion(hexatree) ;
}

QList<Segment_t> QRationalSurfaceIntersection::dummy(Node_t * n)
{
	QList<Segment_t> l;
	Q_UNUSED(n) ;
    return l;
}

QList<Segment_t> QRationalSurfaceIntersection::fusion(Hexatree_t hexatree)
{
    return fusion(hexatree.GetRoot());
}

QList<Segment_t> QRationalSurfaceIntersection::fusion(Node_t * node)
{
	if (node->isLeaf()) return node->objects; // it's the topology in of the curve in the cell.
	else return	fusion(node->LLLLchild)
				<< fusion(node->LLLRchild)
				<< fusion(node->LLRLchild)
				<< fusion(node->LLRRchild)
				<< fusion(node->LRLLchild)
				<< fusion(node->LRLRchild)
				<< fusion(node->LRRLchild)
				<< fusion(node->LRRRchild)
				<< fusion(node->RLLLchild)
				<< fusion(node->RLLRchild)
				<< fusion(node->RLRLchild)
				<< fusion(node->RLRRchild)
				<< fusion(node->RRLLchild)
				<< fusion(node->RRLRchild)
				<< fusion(node->RRRLchild)
				<< fusion(node->RRRRchild);
}

// QList<Segment_t> QRationalSurfaceIntersection::fusion(Node_t * node)
// {
// 	if (node->isLeaf()) return node->objects; // it's the topology in of the curve in the cell.
// 	else return
// 				merge( 
// 						merge(
// 								merge(
// 										merge( fusion(node->LLLLchild), fusion(node->LLLRchild) ),
// 										merge( fusion(node->LLRLchild), fusion(node->LLRRchild) )),
// 								merge(
// 										merge( fusion(node->LRLLchild), fusion(node->LRLRchild) ),
// 										merge( fusion(node->LRRLchild), fusion(node->LRRRchild) ))),
// 
// 						merge(
// 								merge(
// 										merge( fusion(node->RLLLchild), fusion(node->RLLRchild) ),
// 										merge( fusion(node->RLRLchild), fusion(node->RLRRchild) )),
// 								merge(
// 										merge( fusion(node->RRLLchild), fusion(node->RRLRchild) ),
// 										merge( fusion(node->RRRLchild), fusion(node->RRRRchild) )))
// 					);
// }
// 
// QList<Segment_t> QRationalSurfaceIntersection::merge(QList<Segment_t> top1, QList<Segment_t> top2) // maybe, we could have a better function...
// {
// 	return top1 << top2;
// }

topology_t QRationalSurfaceIntersection::getTopology3D(QList<Segment_t> top4D) // transform the 4D topology to 3D topology.
{
	topology_t top;
	QSynapsInterface * iSynaps = dynamic_cast<QSynapsInterface *>(PluginManager::instance()->plugin("Synaps")) ;
	
	for (int i = 0; i < top4D.size(); ++i)
	{
		top << qMakePair(iSynaps->map2D3D(top4D[i].first->s(), top4D[i].first->t(), this->s1),
							iSynaps->map2D3D(top4D[i].second->s(), top4D[i].second->t(), this->s1)); 
	}
	return top;
}

void QRationalSurfaceIntersection::run(void) 
{
	// QSynapsInterface * iSynaps = dynamic_cast<QSynapsInterface *>(PluginManager::instance()->plugin("Synaps")) ;

	QCell4D * C4 = new QCell4D();
	Q_UNUSED(C4) ;
	// int faceBorder[2] = {1,1};
	// cout << __func__ << endl;
	// QList<QPoint4f *> l = iSynaps->intersectionPoints4D(curve, curve->bb(), faceBorder);
	// cout << __func__ << " : l.size() = " << l.size() << endl;
	// cout << l.first()->t() << endl;
	// createExamples();
	// QList<Segment_t> top4D = getTopology4D(curve);
	// getTopology3D(top4D);
	
	test_isBorderMonotonic4D();
	
}

} ; // namespace QIntersection ;
