/***************************************************************************
                          estimator.cpp  -  description
                             -------------------
    begin                : Thu Feb 27 2003
    copyright            : (C) 2003 by Silviu Simen
    email                : ssimen@vryad.com
 ***************************************************************************/

#include "estimator.h"
#include <math.h>
#include <float.h>
#include <stdio.h>

void CEstimator::Init (int grade)
{
  Done ();
  m_grade = grade;
  coefs = new float[m_grade + 1];
  speeds = new float[m_grade + 1];
  accs = new float[m_grade + 1];
  m_numfilled = 0;
  ZeroAll();
}

void CEstimator::ZeroAll ()
{
  m_numfilled = 0;
  for (int i = 0; i < m_grade + 1; i++)
  {
    coefs[i] = 0.0f;
    speeds[i] = 0.0f;
    accs[i] = 0.0f;
  }
  a = v = x = 0.0;
}

CEstimator::CEstimator ()
{
  m_numfilled = -1;
  m_grade = -1;
  coefs = NULL;
  speeds = NULL;
  accs = NULL;
}

CEstimator::~CEstimator ()
{
  Done ();
}

void CEstimator::Done ()
{
  if (coefs)
    delete coefs;
  if (speeds)
    delete speeds;
  if (accs)
    delete accs;
  coefs = NULL;
  speeds = NULL;
  accs = NULL;
  m_grade = -1;
  m_numfilled = -1;
}

float CEstimator::GetAverage (float *data, int size)
{
  int i;
  float f = 0.0;
  if (size < 1)
    return 0.0;
  for (i = 0; i < size; i++)
  {
    f += data[i];
  }
  f /= (float) size;
  return f;
}

float CEstimator::GetAverage ()
{
  return GetAverage (coefs, m_numfilled);
}

float CEstimator::GetStdDeviation (float *data, int size, float average)
{
  int i;
  float m_std_deviation_v = 0.0;
  if (size < 1)
    return 0.0;
  for (i = 0; i < size; i++)
  {
    m_std_deviation_v += (float) pow (data[i] - average, 2);
  }
  m_std_deviation_v /= (float) (size);
  m_std_deviation_v = (float) sqrt (m_std_deviation_v);
  return m_std_deviation_v;
}

float CEstimator::GetAverageWithStdDeviation (float *data, int size, float average, float stddev, int *numused)
{
  float new_avg = 0.0;
  int newsize = 0;
  int i;
  if (size < 1)
    return 0.0;
  for (i = 0; i < size; i++)
  {
    if (data[i] - average <= stddev)
    {
      new_avg += data[i];
      newsize++;
    }
  }
  if (newsize == 0)
    new_avg = average;
  else
    new_avg /= (float) newsize;
  *numused = newsize;
  return new_avg;
}

float CEstimator::GetFilteredAvg (float *stddev, int *numused)
{
  float avg;

  avg = GetAverage (coefs, m_numfilled);
  *stddev = GetStdDeviation (coefs, m_numfilled, avg);
  avg = GetAverageWithStdDeviation (coefs, m_numfilled, avg, *stddev, numused);

  return avg;
}

void CEstimator::AddSample (float value)
{
  int i;
  if (m_numfilled < m_grade)
  {
	for (i = m_numfilled + 1; i > 0; i--)
	coefs[i] = coefs[i - 1];
	coefs[0] = value;
	m_numfilled++;
  }
  else
  {
	for (i = m_grade; i > 0; i--)
	coefs[i] = coefs[i - 1];
	coefs[0] = value;
  }
}

float CEstimator::GetEstimate ()
{
  int i;
  a = 0.0;
  v = 0.0;
  x = 0.0;
  for (i = 1; i < m_numfilled; i++)
    accs[i - 1] = (speeds[i] - speeds[i - 1]);
  for (i = 1; i < m_numfilled + 1; i++)
    speeds[i - 1] = (coefs[i] - coefs[i - 1]);
  //v = speeds[0];
  v = GetAverage (speeds, m_numfilled - 1);
  //a = accs[0];
  a = GetAverage (accs, m_numfilled - 2);
  x = GetAverage (coefs, m_numfilled - 0);
  return (float) (a / 2.0 + v + x);
}

