/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QAxelScriptFactory
 * 2006-07-06
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# include <QAxel/QPoint3f.h>
# include <QAxel/QPointSet.h>
# include <QAxel/QPlane.h>
# include <QAxel/QRationalCurve.h>
# include <QAxel/QBSplineCurve.h>
# include <QAxel/QNurbsCurve.h>
# include <QAxel/QImplicitCurve2D.h>
# include <QAxel/QImplicitCurve3D.h>
# include <QAxel/QImplicitCurve4D.h>
# include <QAxel/QRationalSurface.h>
# include <QAxel/QBSplineSurface.h>
# include <QAxel/QNurbsSurface.h>
# include <QAxel/QImplicitSurface.h>

# include <QScript/QAxelScriptFactory.h>

using namespace std ;
using namespace QAxel ;

QAxelScriptFactory::QAxelScriptFactory()
{
    registerClass("Point3f", &QPoint3f::staticMetaObject) ;
    registerClass("PointSet", &QPointSet::staticMetaObject) ;
    registerClass("Plane", &QPlane::staticMetaObject) ;
    registerClass("RationalCurve", &QRationalCurve::staticMetaObject) ;
    registerClass("ImplicitCurve2D", &QImplicitCurve2D::staticMetaObject) ;
    registerClass("ImplicitCurve3D", &QImplicitCurve3D::staticMetaObject) ;
    registerClass("RationalSurface", &QRationalSurface::staticMetaObject) ;
    registerClass("ImplicitSurface", &QImplicitSurface::staticMetaObject) ;
}

QObject * QAxelScriptFactory::create(const QString &type, const QVariantList &l, QObject *)
{
    if (type == "Point3f") {
	float x = l[0].toDouble() ;
	float y = l[1].toDouble() ;
	float z = l[2].toDouble() ;
	return new QPoint3f(x, y, z) ;
    } else if(type == "Plane") {
	float a = l[0].toDouble() ;
	float b = l[1].toDouble() ;
	float c = l[2].toDouble() ;
	float d = l[3].toDouble() ;
	return new QPlane(a, b, c, d) ;
    } else if(type == "RationalCurve") {
	QString px = l[0].toString() ;
	QString py = l[1].toString() ;
	QString pz = l[2].toString() ;
	QString pw = l[3].toString() ;
	return new QRationalCurve(px, py, pz, pw) ;
    } else if(type == "ImplicitCurve2D") {
	QString equation = l[0].toString() ;
	float xmin = l[1].toDouble() ;
	float xmax = l[2].toDouble() ;
	float ymin = l[3].toDouble() ;
	float ymax = l[4].toDouble() ;
	return new QImplicitCurve2D(equation, xmin, xmax, ymin, ymax) ;
    } else if(type == "ImplicitCurve3D") {
	QString equation1 = l[0].toString() ;
	QString equation2 = l[1].toString() ;
	float xmin = l[2].toDouble() ;
	float xmax = l[3].toDouble() ;
	float ymin = l[4].toDouble() ;
	float ymax = l[5].toDouble() ;
	float zmin = l[6].toDouble() ;
	float zmax = l[7].toDouble() ;
	return new QImplicitCurve3D(equation1, equation2, xmin, xmax, ymin, ymax, zmin, zmax) ;
    } else if(type == "RationalSurface") {
	QString px = l[0].toString() ;
	QString py = l[1].toString() ;
	QString pz = l[2].toString() ;
	QString pw = l[3].toString() ;
	double umin = l[4].toDouble() ;
	double umax = l[5].toDouble() ;
	double vmin = l[6].toDouble() ;
	double vmax = l[7].toDouble() ;
	return new QRationalSurface(px, py, pz, pw, umin, umax, vmin, vmax) ;
    } else if(type == "ImplicitSurface") {
	QString eq = l[0].toString() ;
	double xmin = l[1].toDouble() ;
	double xmax = l[2].toDouble() ;
	double ymin = l[3].toDouble() ;
	double ymax = l[4].toDouble() ;
	double zmin = l[5].toDouble() ;
	double zmax = l[6].toDouble() ;
	return new QImplicitSurface(eq, xmin, xmax, ymin, ymax, zmin, zmax) ;
    }

    return 0;
}
