/*****************************************************************************
 * A x e l
 *****************************************************************************
 * QSislPlugin
 * 2006-05-13
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# ifndef QSISLPLUGIN_H
# define QSISLPLUGIN_H

# include <QGLViewer/qglviewer.h>
# include <QSisl/QSislInterface.h>

using namespace QAxel ;
using namespace qglviewer ;

class SISLCurve ;
class SISLSurf ;

/****************************************************************************/
/*!
  \class QSislPlugin QsislPlugin.h
  \brief The class QSislPlugin implements the \link QSislInterface \endlink
  as a glue to the Sisl library.

  More ...
*/
/****************************************************************************/
class QSislPlugin : public QObject, public QSislInterface
{
    Q_OBJECT
    Q_INTERFACES(QSislInterface QAxel::QPluginInterface)
	
public:
    QString name() const ;
    QStringList features() const ;
    
    /*! @name Creation */
    void create(QBSplineCurve * curve) ;
    void create(QBSplineSurface * surface) ;

    /*! @name IOStream */    
    QBSplineCurve * readCurve(QFile * file) ;
    QBSplineSurface * readSurface(QFile * file) ;
    
    /*! @name Closest points */
    QPoint3f * closestPoint(QBSplineCurve * curve, QPoint3f * p) ;
    
    /*! @name Approximation */
    QBSplineCurve * approximateBSplineCurve(QList<QPoint3f *> points, int , int , int, int) ;

    QBSplineSurface * approximateBSplineSurface(QList<QPoint3f *> points, int , int, int , int, int) ;

    /*! @name Joining */
    QBSplineCurve * join(QBSplineCurve * c1, QBSplineCurve * c2) ;
    
    /*! @name Interpolation */
    QBSplineCurve * interpolate(QList<QPoint3f *> points) ;
    // QBSplineSurface * interpolate(QList<QPoint3f *> points) ;
    
    /*! @name Blending */
    QBSplineCurve * blend(QBSplineCurve * curve1, QBSplineCurve * curve2) ;

    /*! @name Offset */
    QBSplineCurve * offset(QBSplineCurve * curve, Vec direction, double distance) ;
    
    /*! @name Intersection */
    QList<QPoint3f *> intersection(QBSplineCurve * curve1, QBSplineCurve * c2) ;
    QList<QPoint3f *> intersection(QBSplineCurve * curve, QCell * cell) ;
    QList<QPoint3f *> intersection(QBSplineCurve * curve, QPair<QPoint3f *, QPoint3f *> line) ;
    QList<QPoint3f *> intersection(QBSplineCurve * curve, QPlane * plane) ;
    
    /*! @name Bounding */
    QBoundingBox * bounding(QBSplineCurve   * curve) ;
    QBoundingBox * bounding(QBSplineSurface * surface) ;

private:
    QMap<QAxelObject *, SISLCurve *> curves ;
    QMap<QAxelObject *, SISLSurf  *> surfaces ;
} ;

#endif
