/*****************************************************************************
 * A x e l
 *****************************************************************************
 * IStreamAxl
 * 2006-05-24
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/
                     
# include <iostream>

# include <QString>                 
# include <QFile>
# include <QLabel>
# include <QStringList>                 
# include <QProgressDialog>
# include <QApplication>
# include <QMessageBox>
# include <QRegExp>
# include <QStatusBar>
# include <QProgressBar>
# include <QWindowsXPStyle>
# include <QTextStream>

# include <QDomDocument>
# include <QDomNodeList>
# include <QDomElement>

# include <QAxel/QVertex.h>               
# include <QAxel/QEdge.h>
# include <QAxel/QFace.h>
# include <QAxel/QMesh.h>
# include <QAxel/QPoint3f.h>
# include <QAxel/QPointSet.h>
# include <QAxel/QPlane.h>
# include <QAxel/QSphere.h>
# include <QAxel/QCube.h>
# include <QAxel/QCylinder.h>
# include <QAxel/QCurve.h>
# include <QAxel/QSurface.h>

# include <QGui/Commands.h>
# include <QGui/MainWindow.h>
# include <QGui/ScriptingDock.h>

# include <QStream/IStream/IStreamAxl.h>
# include <QStream/IStream/IStreamVect.h>
# include <QStream/IStream/IStreamOff.h>

using namespace std ;
using namespace qglviewer ;

namespace QStream {

IStreamAxl::IStreamAxl(QString s, MainWindow * mw) 
{
    filename = s ;
    mainwindow = mw ;
    
    domdocument = new QDomDocument("axl") ;
    QFile * file = new QFile(filename) ;
    
    if( !file->open(QIODevice::ReadOnly | QIODevice::Text) )  {
	QMessageBox::information(0, "Axel","Error of reading","&Ok",0) ;
	return ;
    }
	
    if( !domdocument->setContent(file)) {
	QMessageBox::information(0, "Axel", filename + " is not a valid Axel file","&Ok",0) ;
	return ;
    }
	
    file->close() ;
}

void IStreamAxl::run(void) 
{
    QString description = getDescription() ;
    
    progress = new QProgressDialog(description, 0, 0, 15) ;
    progress->setWindowTitle("Axel") ;
    
    getPoints() ;      progress->setValue(1) ;  qApp->processEvents();
    getPlanes() ;      progress->setValue(2) ;  qApp->processEvents();
    getCylinders() ;   progress->setValue(3) ;  qApp->processEvents();
    getCubes() ;       progress->setValue(4) ;  qApp->processEvents();
    getSpheres() ;     progress->setValue(5) ;  qApp->processEvents();
    getCurves() ;      progress->setValue(6) ;  qApp->processEvents();
    getSurfaces() ;    progress->setValue(7) ;  qApp->processEvents();
    getScript() ;      progress->setValue(8) ;  qApp->processEvents();
    getCamera() ;      progress->setValue(9) ;  qApp->processEvents();

    delete progress ;
}

QStringList IStreamAxl::getPolynomials(QDomElement e)
{
    QStringList l ;
    QDomNodeList nodelist = e.elementsByTagName("polynomial") ;
    for(unsigned int i = 0 ; i < nodelist.length() ; i++)  {
	QDomElement element = nodelist.item(i).toElement() ;
	l << element.text().simplified() ;
    }
    return l ;
}

QBoundingBox * IStreamAxl::getDomain(QDomElement e) 
{
    QBoundingBox * bb = NULL ;
    QDomNodeList nodelist = e.elementsByTagName("domain") ;
    if(nodelist.size() > 1) {
	QMessageBox::information(0, "Axel", "Only one domain is to be specified","&Ok",0) ;
    } else {
	QDomElement element = nodelist.item(0).toElement() ;
	QString text = element.text().simplified() ;
	QStringList bounds = text.split(QRegExp("\\s+")) ;
	double bbs[] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0} ; 
	for(int k = 0 ; k < bounds.size() ; k++) bbs[k] = bounds[k].toDouble() ;
	bb = new QBoundingBox(bbs[0], bbs[1], bbs[2], bbs[3], bbs[4], bbs[5]) ;
    }
    return bb ;
}

QPair<double, double> IStreamAxl::getGrid(QDomElement e)
{
    QPair<double, double> p ;
    QDomNodeList nodelist = e.elementsByTagName("grid") ;
    for(unsigned int i = 0 ; i < nodelist.length() ; i++)  {
	QDomElement element = nodelist.item(i).toElement() ;
	QString text = element.text().simplified() ;
	QStringList bounds = text.split(QRegExp("\\s+")) ;
	p.first = bounds[0].toDouble() ;
	p.second = bounds[1].toDouble() ;
    }
    return p ;
}

void IStreamAxl::getPiecewiseLinearCurves(QDomElement e)
{
    QBoundingBox * bb = getDomain(e) ;    
    Q_UNUSED(bb) ;
}

void IStreamAxl::getPiecewiseLinearSurfaces(QDomElement e)
{
    QBoundingBox * bb = getDomain(e) ;    
    Q_UNUSED(bb) ;
}

void IStreamAxl::getMeshCurves(QDomElement e)
{
    QDomNodeList nodelist = e.elementsByTagName("vect") ;
    for(unsigned int i = 0 ; i < nodelist.length() ; i++)  {
	QDomElement element = nodelist.item(i).toElement() ;
	
	QString fileName = "/tmp/tmp_axl.vect" ;
	
	QFile * f = new QFile(fileName) ;
	f->open(QIODevice::ReadWrite) ;
	QTextStream tmp(f) ;
	tmp << element.text().trimmed() ;
	f->close() ;
	
	QFile * file = new QFile(fileName) ;
	if (!file->open(QIODevice::ReadOnly | QIODevice::Text)) return;
	IStreamVect parser(file) ;
	QMesh * m = parser.run() ;
	mainwindow->viewer()->addObject(m) ;
	file->close() ;
    }
}

void IStreamAxl::getMeshSurfaces(QDomElement e)
{
    QDomNodeList nodelist = e.elementsByTagName("off") ;
    for(unsigned int i = 0 ; i < nodelist.length() ; i++)  {
	QDomElement element = nodelist.item(i).toElement() ;
	
	QString fileName = "/tmp/tmp_axl.off" ;
	
	QFile * f = new QFile(fileName) ;
	f->open(QIODevice::ReadWrite );
	QTextStream tmp(f) ;
	tmp << element.text().trimmed() ;
	f->close() ;
	
	QFile * file = new QFile(fileName) ;
	if (!file->open(QIODevice::ReadOnly | QIODevice::Text)) return;
	IStreamOff parser(file) ;
	QMesh * m = parser.run() ;
	mainwindow->viewer()->addObject(m) ;
	file->close() ;
    }
}

void IStreamAxl::getImplicitCurves(QDomElement e)
{
    QBoundingBox * bb = getDomain(e) ;
    QStringList polynomials = getPolynomials(e) ;
    if(polynomials.size() == 1) {
	QImplicitCurve2D * c = new QImplicitCurve2D(polynomials[0], bb) ;
	if(e.hasAttribute("name")) c->setName(e.attributeNode("name").value()) ;
	if(e.hasAttribute("color")) {
	    QString color = e.attributeNode("color").value() ;
	    QStringList colors = color.split(QRegExp("\\s+")) ;
	    int r = colors[0].toInt() ;
	    int g = colors[1].toInt() ;
	    int b = colors[2].toInt() ;
	    c->setRGB(r, g, b) ;
	}
	mainwindow->viewer()->addObject(c) ;
    } else if(polynomials.size() == 2) {
	QImplicitCurve3D * c = new QImplicitCurve3D(polynomials[0], polynomials[1], bb) ;
	if(e.hasAttribute("name")) c->setName(e.attributeNode("name").value()) ;
	if(e.hasAttribute("color")) {
	    QString color = e.attributeNode("color").value() ;
	    QStringList colors = color.split(QRegExp("\\s+")) ;
	    int r = colors[0].toInt() ;
	    int g = colors[1].toInt() ;
	    int b = colors[2].toInt() ;
	    c->setRGB(r, g, b) ;
	}
	mainwindow->viewer()->addObject(c) ;
    } else {
	QMessageBox::information(0, "Axel", "Too much polynomials for a curve","&Ok",0) ;
	return ;
    }
}

void IStreamAxl::getImplicitSurfaces(QDomElement e) 
{
    QBoundingBox * bb = getDomain(e) ;
    QStringList polynomials = getPolynomials(e) ;
    if(polynomials.size() != 1) {
	QMessageBox::information(0, "Axel", "Only one polynomial for an implicit surface","&Ok",0) ;
	return ;
    } 
    QImplicitSurface * s = new QImplicitSurface(polynomials[0], bb) ;
    if(e.hasAttribute("name")) s->setName(e.attributeNode("name").value()) ;
    if(e.hasAttribute("color")) {
	QString color = e.attributeNode("color").value() ;
	QStringList colors = color.split(QRegExp("\\s+")) ;
	int r = colors[0].toInt() ;
	int g = colors[1].toInt() ;
	int b = colors[2].toInt() ;
	if(colors.size() == 4) { int a = colors[3].toInt() ; s->setRGBA(r, g, b, a) ; }
	else s->setRGB(r, g, b) ;
    }
    mainwindow->viewer()->addObject(s) ;
}

void IStreamAxl::getRationalCurves(QDomElement e) 
{
    QBoundingBox * bb = getDomain(e) ;
    QStringList polynomials = getPolynomials(e) ;
    QRationalCurve * c = new QRationalCurve(polynomials[0], polynomials[1], polynomials[2], polynomials[3], bb->xmin(), bb->xmax()) ;
    if(e.hasAttribute("name")) c->setName(e.attributeNode("name").value()) ;
    if(e.hasAttribute("color")) {
	QString color = e.attributeNode("color").value() ;
	QStringList colors = color.split(QRegExp("\\s+")) ;
	int r = colors[0].toInt() ;
	int g = colors[1].toInt() ;
	int b = colors[2].toInt() ;
	c->setRGB(r, g, b) ;
    }
    mainwindow->viewer()->addObject(c) ;
}

void IStreamAxl::getParametricCurves(QDomElement e) 
{
    Q_UNUSED(e) ;
}

void IStreamAxl::getRationalSurfaces(QDomElement e)
{
    QBoundingBox * bb = getDomain(e) ;
    QStringList polynomials = getPolynomials(e) ;
    QPair<double, double> p = getGrid(e) ;
    if(p.first == 0) p.first = 50 ;
    if(p.second == 0) p.second = 50 ;
    QRationalSurface * ps = new QRationalSurface(polynomials[0], polynomials[1], polynomials[2], polynomials[3], bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax(), p.first, p.second) ;
    if(e.hasAttribute("name")) ps->setName(e.attributeNode("name").value()) ;
    if(e.hasAttribute("color")) {
	QString color = e.attributeNode("color").value() ;
	QStringList colors = color.split(QRegExp("\\s+")) ;
	int r = colors[0].toInt() ;
	int g = colors[1].toInt() ;
	int b = colors[2].toInt() ;
	if(colors.size() == 4) { int a = colors[3].toInt() ; ps->setRGBA(r, g, b, a) ; }
	else ps->setRGB(r, g, b) ;
    }
    mainwindow->viewer()->addObject(ps) ;
}

int IStreamAxl::getDimension(QDomElement e) 
{
    int d = 0 ;
    QDomNodeList nodelist = e.elementsByTagName("dimension") ;
    for(unsigned int i = 0 ; i < nodelist.length() ; i++)  {
	QDomElement element = nodelist.item(i).toElement() ;
	d = element.text().simplified().toInt() ;
    }
    return d ;
}

QList<int> IStreamAxl::getNumbers(QDomElement e) 
{
    QList<int> numbers ;
    QDomNodeList nodelist = e.elementsByTagName("number") ;
    for(unsigned int i = 0 ; i < nodelist.length() ; i++)  {
	QDomElement element = nodelist.item(i).toElement() ;
	QStringList list = element.text().simplified().split(QRegExp("\\s+")) ;
	foreach(QString s, list) numbers << s.toInt() ;
    }
    return numbers ;
}

QList<int> IStreamAxl::getOrders(QDomElement e) 
{
    QList<int> orders ;
    QDomNodeList nodelist = e.elementsByTagName("order") ;
    for(unsigned int i = 0 ; i < nodelist.length() ; i++)  {
	QDomElement element = nodelist.item(i).toElement() ;
	QStringList list = element.text().simplified().split(QRegExp("\\s+")) ;
	foreach(QString s, list) orders << s.toInt() ;
    }
    return orders ;
}

QList<double> IStreamAxl::getKnots(QDomElement e, int i) 
{
    QList<double> knots ;
    QDomNodeList nodelist = e.elementsByTagName("knots") ;
    QDomElement element = nodelist.item(i).toElement() ;
    QStringList list = element.text().simplified().split(QRegExp("\\s+")) ;
    foreach(QString s, list) knots << s.toDouble() ;
    return knots ;
}

QList<QPoint3f *> IStreamAxl::getControlPoints(QDomElement e) 
{
    QList<QPoint3f *> points ;
    QDomNodeList nodelist = e.elementsByTagName("points") ;
    for(unsigned int i = 0 ; i < nodelist.length() ; i++)  {
	QDomElement element = nodelist.item(i).toElement() ;
	QStringList list = element.text().simplified().split(QRegExp("\\s+")) ;
	
	for(QStringList::iterator itr = list.begin() ; itr != list.end() ; itr+=3) {
	    QPoint3f * p = new QPoint3f(itr->toFloat(), (itr+1)->toFloat(), (itr+2)->toFloat()) ;
	    p->setRGB(139, 35, 35) ;
	    points << p ;
	}
    }
    return points ;
}

void IStreamAxl::getBSplineCurves(QDomElement e) 
{
    int dimension = getDimension(e) ;
    QList<int> orders = getOrders(e) ;
    QList<int> numbers = getNumbers(e) ;
    QList<double> knots = getKnots(e, 0) ;
    QList<QPoint3f *> points = getControlPoints(e) ;
    
    QBSplineCurve * bsc = new QBSplineCurve(NULL) ;
    bsc->order = orders[0] ;
    bsc->number = numbers[0] ;
    bsc->idim = dimension ;
    double * knotsd = new double[knots.size()] ;
    for(int i = 0 ; i < knots.size() ; i++) knotsd[i] = knots[i] ;
    bsc->knots = knotsd ;
    
    foreach(QPoint3f * p, points) {
	bsc->push_point(p) ;
	p->setBlackListed(true) ;
    }
    
    if(e.hasAttribute("name")) bsc->setName(e.attributeNode("name").value()) ;
    if(e.hasAttribute("color")) {
	QString color = e.attributeNode("color").value() ;
	QStringList colors = color.split(QRegExp("\\s+")) ;
	int r = colors[0].toInt() ;
	int g = colors[1].toInt() ;
	int b = colors[2].toInt() ;
	bsc->setRGB(r, g, b) ;
    }
    bsc->compute() ;
    mainwindow->viewer()->addObject(bsc) ;
}

void IStreamAxl::getBSplineSurfaces(QDomElement e) 
{
    int dimension = getDimension(e) ;
    QList<int> orders = getOrders(e) ;
    QList<int> numbers = getNumbers(e) ;
    QList<double> knots1 = getKnots(e, 0) ;
    QList<double> knots2 = getKnots(e, 1) ;
    QList<QPoint3f *> points = getControlPoints(e) ;

    QBSplineSurface * bss = new QBSplineSurface(NULL) ;
    bss->order1 = orders[0] ;
    bss->order2 = orders[1] ;
    bss->number1 = numbers[0] ;
    bss->number2 = numbers[1] ;
    bss->idim = dimension ;
    double * knotsd1 = new double[knots1.size()] ;
    for(int i = 0 ; i < knots1.size() ; i++) knotsd1[i] = knots1[i] ;
    bss->knots1 = knotsd1 ;
    double * knotsd2 = new double[knots2.size()] ;
    for(int i = 0 ; i < knots2.size() ; i++) knotsd2[i] = knots2[i] ;
    bss->knots2 = knotsd2 ;
    foreach(QPoint3f * p, points) {
	bss->push_point(p) ;
	p->setBlackListed(true) ;
    }
	
    if(e.hasAttribute("name")) bss->setName(e.attributeNode("name").value()) ;
    if(e.hasAttribute("color")) {
	QString color = e.attributeNode("color").value() ;
	QStringList colors = color.split(QRegExp("\\s+")) ;
	int r = colors[0].toInt() ;
	int g = colors[1].toInt() ;
	int b = colors[2].toInt() ;
	if(colors.size() == 4) { int a = colors[3].toInt() ; bss->setRGBA(r, g, b, a) ; }
	else bss->setRGB(r, g, b) ;
    }
    bss->compute() ;
    mainwindow->viewer()->addObject(bss) ;
}

void IStreamAxl::getParametricSurfaces(QDomElement e) 
{
    Q_UNUSED(e) ;
    // QBoundingBox * bb = getDomain(e) ;
    // QStringList polynomials = getPolynomials(e) ;
    // QParametricSurface * ps = new QParametricSurface(polynomials[0], polynomials[1], polynomials[2], bb->xmin(), bb->xmax(), bb->ymin(), bb->ymax(), bb) ;
    // if(e.hasAttribute("name")) ps->setName(e.attributeNode("name").value()) ;
    // if(e.hasAttribute("color")) {
    // 	QString color = e.attributeNode("color").value() ;
    // 	QStringList colors = color.split(QRegExp("\\s+")) ;
    // 	int r = colors[0].toInt() ;
    // 	int g = colors[1].toInt() ;
    // 	int b = colors[2].toInt() ;
    // 	if(colors.size() == 4) { int a = colors[3].toInt() ; ps->setRGBA(r, g, b, a) ; }
    // 	else ps->setRGB(r, g, b) ;
    // }
    // mainwindow->viewer()->addObject(ps) ;
}

QString IStreamAxl::getDescription(void)
{
    QDomNodeList nodelist= domdocument->elementsByTagName("point") ;
    QDomElement element = nodelist.item(0).toElement() ;
    return element.text() ;
}

void IStreamAxl::getPoints(void)
{
    QDomNodeList nodelist= domdocument->elementsByTagName("point") ;
    for(unsigned int i = 0 ; i < nodelist.length() ; i++)  {
	QDomElement element = nodelist.item(i).toElement() ;
	QStringList coordinates = element.text().simplified().split(QRegExp("\\s+")) ;
	CreatePointCommand * command = new CreatePointCommand(mainwindow->viewer(), coordinates[0].toFloat(), coordinates[1].toFloat(), coordinates[2].toFloat()) ;
	// QPoint3f * p = new QPoint3f(coordinates[0].toFloat(), coordinates[1].toFloat(), coordinates[2].toFloat()) ;
	if(element.hasAttribute("name")) command->setName(element.attributeNode("name").value()) ;
	if(element.hasAttribute("color")) {
	    QString color = element.attributeNode("color").value() ;
	    QStringList colors = color.split(QRegExp("\\s+")) ;
	    int r = colors[0].toInt() ;
	    int g = colors[1].toInt() ;
	    int b = colors[2].toInt() ;
	    command->setColor(r,g,b) ;
	}
	// mainwindow->viewer()->addObject(p) ;
	mainwindow->stack()->push(command) ;
    }

    QDomNodeList nodelist2= domdocument->elementsByTagName("pointset") ;
    for(unsigned int i = 0 ; i < nodelist2.length() ; i++)  {
	bool colors = false ;
	QDomElement element = nodelist2.item(i).toElement() ;
	if(element.hasAttribute("color")) if(element.attributeNode("color").value() == "rgb") colors = true ;
	QStringList coordinates = element.text().simplified().split(QRegExp("\\s+")) ;
	if(colors) {
	    QPointSet * ps = new QPointSet(coordinates.size()/6, true) ;
	    for(int i=0 ; i<coordinates.size() ; i+=6) {
		float x = coordinates[i+0].toFloat() ;
		float y = coordinates[i+1].toFloat() ;
		float z = coordinates[i+2].toFloat() ;
		int r = coordinates[i+3].toInt() ;
		int g = coordinates[i+4].toInt() ;
		int b = coordinates[i+5].toInt() ;
		ps->push(x, y, z, r, g, b) ;
	    }
	    if(element.hasAttribute("name")) ps->setName(element.attributeNode("name").value()) ;
	    mainwindow->viewer()->addObject(ps) ;
	} else {
	    QPointSet * ps = new QPointSet(coordinates.size()/3, false) ;
	    for(int i=0 ; i<coordinates.size() ; i+=3) {
		float x = coordinates[i+0].toFloat() ;
		float y = coordinates[i+1].toFloat() ;
		float z = coordinates[i+2].toFloat() ;
		ps->push(x, y, z) ;
	    }
	    if(element.hasAttribute("name")) ps->setName(element.attributeNode("name").value()) ;
	    mainwindow->viewer()->addObject(ps) ;
	}
    }
}

void IStreamAxl::getPlanes(void) 
{	
    QDomNodeList nodelist= domdocument->elementsByTagName("plane") ;
    for(unsigned int i = 0 ; i < nodelist.length() ; i++)  {
	int size = 0 ; float a = 0, b = 0, c = 0, d = 0 ;
	QStringList polynomials ;
	QDomElement element = nodelist.item(i).toElement() ;
	QDomNodeList childs = nodelist.item(i).childNodes() ;
	for(unsigned int j = 0 ; j < childs.length() ; j++) {
	    QDomElement e = childs.item(j).toElement() ;
	    if(e.tagName() == "size" ) {
		size = e.text().toInt() ;
	    } else if(e.tagName() == "polynomial") {
		QString polynomial = e.text().simplified() ;
		QRegExp ra("(\\d*)(\\.\\d+)*(x)(\\^\\d+)*") ;
		QRegExp rb("(\\d*)(\\.\\d+)*(y)(\\^\\d+)*") ;
		QRegExp rc("(\\d*)(\\.\\d+)*(z)(\\^\\d+)*") ;
//		QRegExp rd("(\\d+)$") ;
		if(ra.indexIn(polynomial) > -1) { QString as = ra.cap(1) ; if(as == "") a = 1 ; else a = as.toFloat() ; }
		if(rb.indexIn(polynomial) > -1) { QString bs = rb.cap(1) ; if(bs == "") b = 1 ; else b = bs.toFloat() ; }
		if(rc.indexIn(polynomial) > -1) { QString cs = rc.cap(1) ; if(cs == "") c = 1 ; else c = cs.toFloat() ; }
//		if(rd.indexIn(polynomial) > -1) { QString ds = rd.cap(1) ; if(ds == "") d = 0 ; else d = ds.toFloat() ; }
	    }
	}
	QPlane * p = new QPlane(a, b, c, d, size) ;
	if(element.hasAttribute("name")) p->setName(element.attributeNode("name").value()) ;
	if(element.hasAttribute("color")) {
	    QString color = element.attributeNode("color").value() ;
	    QStringList colors = color.split(QRegExp("\\s+")) ;
	    int r = colors[0].toInt() ;
	    int g = colors[1].toInt() ;
	    int b = colors[2].toInt() ;
	    p->setRGB(r, g, b) ;
	}
	mainwindow->viewer()->addObject(p) ;
    }
}

void IStreamAxl::getSpheres(void)
{
    QDomNodeList nodelist= domdocument->elementsByTagName("sphere") ;
    for(unsigned int i = 0 ; i < nodelist.length() ; i++) {
	QDomElement element = nodelist.item(i).toElement() ;
	float x = 0, y = 0, z = 0, r = 0 ;
		
	QDomNodeList childs = nodelist.item(i).childNodes() ;
	for(unsigned int j = 0 ; j < childs.length() ; j++) {
	    QDomElement e = childs.item(j).toElement() ;
	    if(e.tagName() == "center" ) {
		QStringList coordinates = e.text().simplified().split(QRegExp("\\s+")) ;
		x = coordinates[0].toFloat() ;
		y = coordinates[1].toFloat() ;
		z = coordinates[2].toFloat() ;
	    } else if(e.tagName() == "radius") {
		QString radius = e.text().simplified() ;
		r = radius.toFloat() ;
	    }
			
	}
		
	QSphere * s = new QSphere(x, y, z, r) ;
	if(element.hasAttribute("name")) s->setName(element.attributeNode("name").value()) ;
	if(element.hasAttribute("color")) {
	    QString color = element.attributeNode("color").value() ;
	    QStringList colors = color.split(QRegExp("\\s+")) ;
	    int r = colors[0].toInt() ;
	    int g = colors[1].toInt() ;
	    int b = colors[2].toInt() ;
	    s->setRGB(r, g, b) ;
	}
	mainwindow->viewer()->addObject(s) ;
    }
}

void IStreamAxl::getCubes(void)
{
    QDomNodeList nodelist= domdocument->elementsByTagName("cube") ;
    for(unsigned int i = 0 ; i < nodelist.length() ; i++)  {
	QDomElement element = nodelist.item(i).toElement() ;
	float x = 0, y = 0, z = 0, s = 0 ;
		
	QDomNodeList childs = nodelist.item(i).childNodes() ;
	for(unsigned int j = 0 ; j < childs.length() ; j++) {
	    QDomElement e = childs.item(j).toElement() ;
		
	    if(e.tagName() == "center" ) {
		QStringList coordinates = e.text().simplified().split(QRegExp("\\s+")) ;
		x = coordinates[0].toFloat() ;
		y = coordinates[1].toFloat() ;
		z = coordinates[2].toFloat() ;
	    } else if(e.tagName() == "size") {
		QString radius = e.text().simplified() ;
		s = radius.toFloat() ;
	    }
	}
	QCube * c = new QCube(x, y, z, s) ;
	if(element.hasAttribute("name")) c->setName(element.attributeNode("name").value()) ;
	if(element.hasAttribute("color")) {
	    QString color = element.attributeNode("color").value() ;
	    QStringList colors = color.split(QRegExp("\\s+")) ;
	    int r = colors[0].toInt() ;
	    int g = colors[1].toInt() ;
	    int b = colors[2].toInt() ;
	    c->setRGB(r, g, b) ;
	}
	mainwindow->viewer()->addObject(c) ;
    }
}

void IStreamAxl::getCylinders(void)
{
    QDomNodeList nodelist= domdocument->elementsByTagName("cylinder") ;
    for(unsigned int i = 0 ; i < nodelist.length() ; i++)  {
	QPoint3f * from = NULL ;
	QPoint3f * to = NULL ;
	float radius = 0.01 ;
	QDomElement element = nodelist.item(i).toElement() ;
	QDomNodeList childs = nodelist.item(i).childNodes() ;
	for(unsigned int j = 0 ; j < childs.length() ; j++) {
	    QDomElement e = childs.item(j).toElement() ;
	    if(e.tagName() == "from" ) {
		QStringList coordinates = e.text().simplified().split(QRegExp("\\s+")) ;
		from = new QPoint3f(coordinates[0].toFloat(), coordinates[1].toFloat(), coordinates[2].toFloat()) ;
	    } else if(e.tagName() == "to" ) {
		QStringList coordinates = e.text().simplified().split(QRegExp("\\s+")) ;
		to = new QPoint3f(coordinates[0].toFloat(), coordinates[1].toFloat(), coordinates[2].toFloat()) ;
	    } else if(e.tagName() == "radius") {
		radius = e.text().simplified().toFloat() ;
	    }
	}
		
	QCylinder * c = new QCylinder(from, to, radius) ;
	if(element.hasAttribute("name")) c->setName(element.attributeNode("name").value()) ;
	if(element.hasAttribute("color")) {
	    QString color = element.attributeNode("color").value() ;
	    QStringList colors = color.split(QRegExp("\\s+")) ;
	    int r = colors[0].toInt() ;
	    int g = colors[1].toInt() ;
	    int b = colors[2].toInt() ;
	    c->setRGB(r, g, b) ;
	}
	mainwindow->viewer()->addObject(c) ;
    }
}

void IStreamAxl::getCurves(void) 
{
    QDomNodeList nodelist= domdocument->elementsByTagName("curve") ;
    for(unsigned int i = 0 ; i < nodelist.length() ; i++)  {
	QDomElement element = nodelist.item(i).toElement() ;
	if(element.hasAttribute("type")) {
	    QString type = element.attributeNode("type").value() ;
	    if(type == "implicit") {
		getImplicitCurves(element) ;
	    } else if(type == "parametric") {
		getParametricCurves(element) ;
	    } else if(type == "mesh") {
		getMeshCurves(element) ;
	    } else if(type == "rational") {
		getRationalCurves(element) ;
	    } else if(type == "bspline") {
		getBSplineCurves(element) ;
	    } else if(type == "linear") {
		getPiecewiseLinearCurves(element) ;
	    }
	}
    }
}
		
void IStreamAxl::getSurfaces(void) 
{
    QDomNodeList nodelist= domdocument->elementsByTagName("surface") ;
    for(unsigned int i = 0 ; i < nodelist.length() ; i++)  {
	QDomElement element = nodelist.item(i).toElement() ;
	if(element.hasAttribute("type")) {
	    QString type = element.attributeNode("type").value() ;
	    if(type == "implicit") {
		getImplicitSurfaces(element) ;
	    } else if(type == "parametric") {
		getParametricSurfaces(element) ;
	    } else if(type == "mesh") {
		getMeshSurfaces(element) ;
	    } else if(type == "rational") {
		getRationalSurfaces(element) ;
	    } else if(type == "bspline") {
		getBSplineSurfaces(element) ;
	    }
	}
    }
}

void IStreamAxl::getScript(void)
{
    QDomNodeList nodelist= domdocument->elementsByTagName("script") ;
    for(unsigned int i = 0 ; i < nodelist.length() ; i++)  {
	QDomElement element = nodelist.item(i).toElement() ;
	bool runScript = false ;
	if(element.hasAttribute("run")) {
	    QString runScriptString = element.attributeNode("run").value() ;
	    if(runScriptString == "true") runScript = true ;
	}
	QString script = element.text().trimmed() ;
	ScriptingDock * scriptingDock = mainwindow->scriptingDock() ;
	scriptingDock->setText(script) ;
	if(runScript) scriptingDock->run() ;
    }
}

void IStreamAxl::getCamera(void)
{
    QDomNodeList nodelist= domdocument->elementsByTagName("camera") ;
    for(unsigned int i = 0 ; i < nodelist.length() ; i++)  {
	QDomElement element = nodelist.item(i).toElement() ;
	mainwindow->viewer()->camera()->initFromDOMElement(element) ;
    }
}

} ;
