/*****************************************************************************
 * A x e l
 *****************************************************************************
 * IStreamOff
 * 2006-03-24
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/
                     
# include <iostream>
                 
# include <QFile>
# include <QLabel>
# include <QStringList>                 
# include <QProgressDialog>
# include <QApplication>

# include <QAxel/QVertex.h>               
# include <QAxel/QEdge.h>
# include <QAxel/QFace.h>
# include <QAxel/QMesh.h>

# include <QStream/IStream/IStreamOff.h>

using namespace std ;

namespace QStream {

IStreamOff::IStreamOff(QFile * f) : IStream(f), progress(0)
{
    dim = 3 ;
    nVertices = nEdges = nFaces = 0 ;
    textureCoordinates = normals = colors = homogeneous = ndim = false ;
    currentVertex = 0 ; currentFace = 0 ;
}

QMesh * IStreamOff::run(void)
{ 
    QMesh * m = new QMesh() ;  
    m->setMeshType(QMesh::Surface) ;
    int step = 0 ;
    while(!atEnd()) {                                     
      if(progress && progress->wasCanceled()) break;
	QString line = readLine().simplified() ;
	if(line.startsWith("#")) {
	    continue ;
	} else if(line.isEmpty()) {
	    continue ;
	} else if(step==0) {
	    if(line.startsWith("STCN4nOFF")) {
		textureCoordinates = true ; colors = true ; normals = true ; homogeneous = true ; ndim = true ;
		/* dim is to be parsed */
	    } else if (line.startsWith("CN4nOFF")) {
		colors = true ; normals = true ; homogeneous = true ; ndim = true ;
	    } else if (line.startsWith("N4nOFF")) {
		normals = true ; homogeneous = true ; ndim = true ;
	    } else if (line.startsWith("4nOFF")) {
		homogeneous = true ; ndim = true ;
	    } else if (line.startsWith("nOFF")) {
		ndim = true ;
	    } else if (line.startsWith("OFF")) {
		
	    }                    
	    step++ ;
	} else if(step==1) {  
	    QStringList numbers = line.split(QRegExp("\\s")) ;
	    nVertices = numbers.takeFirst().toInt() ;
	    nFaces = numbers.takeFirst().toInt() ;
	    if(numbers.size()>0) nEdges = numbers.takeFirst().toInt() ;  
	    
	    m->setNumberOfVertices(nVertices) ;
	    m->setNumberOfFaces(nFaces) ;
	    
	    progress = new QProgressDialog("Importing Object File Format", "Cancel", 0, nVertices+nFaces) ;
	    progress->setWindowTitle("Axel") ;
	    
	    step++ ;
	} else if(step==2) {
	    readVertex(line, m) ;
	    currentVertex++ ;
	    progress->setValue(currentVertex) ;
	    qApp->processEvents();
	    if(currentVertex == nVertices) {
		step++ ;
	    }
	} else if(step==3) {              
	    readFace(line, m) ;                
	    currentFace++ ;     
	    progress->setValue(currentVertex+currentFace) ; 
	    qApp->processEvents();
	    if(currentFace == nFaces) step++ ;
	}
    }           
    if(progress->wasCanceled()) 
    {
      delete m;
      return 0;
    }

    m->computeNormals() ;
    return m ;
}

void IStreamOff::readVertex(QString line, QMesh * m)
{  
    QStringList numbers = line.split(QRegExp("\\s")) ;
    
    float x = numbers.takeFirst().toFloat() ;
    float y = numbers.takeFirst().toFloat() ;
    float z = numbers.takeFirst().toFloat() ;                                   
    QVertex * v = new QVertex(currentVertex,x,y,z,m) ;        
	
    if(normals) {
	float nx = numbers.takeFirst().toFloat() ;
	float ny = numbers.takeFirst().toFloat() ;
	float nz = numbers.takeFirst().toFloat() ;
	v->setNormal(nx,ny,nz) ;   
    }
    if(colors)  {
	float r = numbers.takeFirst().toFloat() ;
	float g = numbers.takeFirst().toFloat() ;
	float b = numbers.takeFirst().toFloat() ;
	v->setColor(r,g,b) ;
    }
    if(textureCoordinates) {  
	float cu = numbers.takeFirst().toFloat() ;
	float cv = numbers.takeFirst().toFloat() ;
	v->setUVMapping(cu,cv) ;
    }
    m->push_vertex(v) ;
}

void IStreamOff::readFace(QString line, QMesh * m) 
{
    QStringList numbers = line.split(QRegExp("\\s")) ;  
	
    QFace * f = new QFace(currentFace, m) ;  
    int n = numbers.takeFirst().toInt() ;                          
	
    for(int i = 0 ; i < n ; i++)
	f->push_index(numbers.takeFirst().toInt()) ;
  
    if(!numbers.empty()) {
	float r = numbers.takeFirst().toFloat() ;
	float g = numbers.takeFirst().toFloat() ;
	float b = numbers.takeFirst().toFloat() ;
	if(!numbers.empty()) {
	    float a = numbers.takeFirst().toFloat() ;
	    f->setColor(r,g,b,a) ;
	} else {
	    f->setColor(r,g,b) ;
	}
    }    
	
    // Computing edges
    QEdge * e1 = new QEdge(f->indexes_[0], f->indexes_[1], m) ;
    QEdge * e2 = new QEdge(f->indexes_[1], f->indexes_[2], m) ;
    QEdge * e3 = new QEdge(f->indexes_[2], f->indexes_[0], m) ;
    m->push_edge(e1) ;
    m->push_edge(e2) ;
    m->push_edge(e3) ;
		                    
    m->push_face(f) ;
}

} ;
