/*****************************************************************************
 * A x e l
 *****************************************************************************
 * 0StreamYafray
 * 2007-02-06
 * Julien Wintz
 *****************************************************************************
 *               Copyright (C) 2006 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments :
 ****************************************************************************/

# include <iostream>

# include <QFile>
# include <QLabel>
# include <QStringList>
# include <QProgressDialog>
# include <QApplication>
# include <QMessageBox>
# include <QTextStream>
# include <QDomDocument>
# include <QDomNodeList>
# include <QDomElement>

# include <QGui/Viewer.h>
# include <QAxel/QRationalSurface.h>
# include <QAxel/QBSplineSurface.h>
# include <QAxel/QNurbsSurface.h>
# include <QAxel/QImplicitSurface.h>
# include <QAxel/QFace.h>
# include <QStream/OStream/OStreamYafray.h>

using namespace std ;
using namespace qglviewer ;

namespace QStream {

OStreamYafray::OStreamYafray(QString s, Viewer * v) 
{
    filename = s ;
    viewer = v ;
    
    domdocument = new QDomDocument() ;
}

void OStreamYafray::run(void)
{
    QList<QDomElement> objects ;
    
    QList<QAxelObject *> isurfaces = viewer->listObjectsFromType("ImplicitSurface");
    foreach(QAxelObject * o, isurfaces) {
	QImplicitSurface * s = dynamic_cast<QImplicitSurface *>(o) ;
//	objects.push_back(meshToYafray(s->name(), s->mesh())) ;
	Q_UNUSED(s) ;
    }
    
    QList<QAxelObject *> bsurfaces = viewer->listObjectsFromType("BSplineSurface");
    foreach(QAxelObject * o, bsurfaces) {
	QBSplineSurface * s = dynamic_cast<QBSplineSurface *>(o) ;
	int nbi ; int * indexes = NULL ;
	int nbv ; double * vertices = NULL ;
	s->getGlData(&nbi, indexes, &nbv, vertices) ;
	objects.push_back(meshToYafray(s->name(), indexes, nbi, vertices, nbv)) ;
    }
    
    QList<QAxelObject *> imeshes = viewer->listObjectsFromType("Mesh");
    foreach(QAxelObject * o, imeshes) {
	QMesh * m = dynamic_cast<QMesh *>(o) ;
	objects.push_back(meshToYafray(m->name(), m)) ;
    }
    
    buildDomTree(objects);
    save();
}

QDomElement OStreamYafray::meshToYafray(QString name, int * indices, int nindices, double * vertices, int nvertices)
{
    QDomElement transform = domdocument->createElement("transform");
    transform.setAttribute("m00","1.0");
    transform.setAttribute("m01","0.0");
    transform.setAttribute("m02","0.0");
    transform.setAttribute("m03","0.0");
    transform.setAttribute("m10","0.0");
    transform.setAttribute("m11","0.0");
    transform.setAttribute("m12","-1.0");
    transform.setAttribute("m13","0.0");
    transform.setAttribute("m20","0.0");
    transform.setAttribute("m21","1.0");
    transform.setAttribute("m22","0.0");
    transform.setAttribute("m23","0.0");
    transform.setAttribute("m30","0.0");
    transform.setAttribute("m31","0.0");
    transform.setAttribute("m32","0.0");
    transform.setAttribute("m33","1.0");
    // scene.appendChild(transform);
	
    QDomElement object = domdocument->createElement("object") ;
    object.setAttribute("name", name) ;
    object.setAttribute("shadow", "on") ;
    object.setAttribute("shader_name", "blender_default") ;
    transform.appendChild(object) ;
    
    QDomElement attributes = domdocument->createElement("attributes") ;
    QDomText attributesTag = domdocument->createTextNode("\n") ;
    attributes.appendChild(attributesTag) ;
    object.appendChild(attributes) ;
    
    QDomElement mesh = domdocument->createElement("mesh") ;
    mesh.setAttribute("autosmooth", "120") ;
    mesh.setAttribute("has_orco", "off") ;
    object.appendChild(mesh) ;
    
    cout << "nbv: " << nvertices << endl ;
    
    QDomElement points = domdocument->createElement("points") ;
    for(int i = 0 ; i < nvertices ; i+=3) {
	cout << i << endl ;
	cout << vertices[i+0] << endl ;
	cout << vertices[i+1] << endl ;
	cout << vertices[i+2] << endl ;
	QDomElement point = domdocument->createElement("p") ;
	point.setAttribute("x", num.setNum(vertices[i+0])) ;
	point.setAttribute("y", num.setNum(vertices[i+1])) ;
	point.setAttribute("z", num.setNum(vertices[i+2])) ;
	points.appendChild(point) ;
    }
    mesh.appendChild(points) ;
    
    cout << "ok so far" << endl ;
    
    cout << "nbi: " << nindices << endl ;
    
    QDomElement faces = domdocument->createElement("faces") ;
    for(int i = 0 ; i < nindices ; i+=3) {
	cout << i << endl ;
	cout << *(indices+i+0) << endl ;
	cout << *(indices+i+1) << endl ;
	cout << *(indices+i+2) << endl ;
	QDomElement face = domdocument->createElement("f") ;
	face.setAttribute("a", num.setNum(indices[i+0])) ;
	face.setAttribute("b", num.setNum(indices[i+1])) ;
	face.setAttribute("c", num.setNum(indices[i+2])) ;
	face.setAttribute("shader_name", "blender_default") ;
	faces.appendChild(face) ;
    }
    mesh.appendChild(faces) ;
    
    return transform ;	
}

QDomElement OStreamYafray::meshToYafray(QString name, QMesh * m)
{
    QDomElement transform = domdocument->createElement("transform");
    transform.setAttribute("m00","1.0");
    transform.setAttribute("m01","0.0");
    transform.setAttribute("m02","0.0");
    transform.setAttribute("m03","0.0");
    transform.setAttribute("m10","0.0");
    transform.setAttribute("m11","0.0");
    transform.setAttribute("m12","-1.0");
    transform.setAttribute("m13","0.0");
    transform.setAttribute("m20","0.0");
    transform.setAttribute("m21","1.0");
    transform.setAttribute("m22","0.0");
    transform.setAttribute("m23","0.0");
    transform.setAttribute("m30","0.0");
    transform.setAttribute("m31","0.0");
    transform.setAttribute("m32","0.0");
    transform.setAttribute("m33","1.0");
    // scene.appendChild(transform);
    
    QDomElement object = domdocument->createElement("object") ;
    object.setAttribute("name", name) ;
    object.setAttribute("shadow", "on") ;
    object.setAttribute("shader_name", "blender_default") ;
    transform.appendChild(object) ;
    
    QDomElement attributes = domdocument->createElement("attributes") ;
    QDomText attributesTag = domdocument->createTextNode("\n") ;
    attributes.appendChild(attributesTag) ;
    object.appendChild(attributes) ;
    
    QDomElement mesh = domdocument->createElement("mesh") ;
    mesh.setAttribute("autosmooth", "0.1") ;
    mesh.setAttribute("has_orco", "off") ;
    object.appendChild(mesh) ;
    
    QDomElement points = domdocument->createElement("points") ;
    // QDomText coordinates = domdocument->createTextNode(meshToXmlPoints(mesh));
    for(int i = 0 ; i < m->nvertices_ ; i++) {
	QDomElement point = domdocument->createElement("p") ;
	point.setAttribute("x", num.setNum(m->glvertices_[3*i+0])) ;
	point.setAttribute("y", num.setNum(m->glvertices_[3*i+1])) ;
	point.setAttribute("z", num.setNum(m->glvertices_[3*i+2])) ;
	points.appendChild(point) ;
    }
    // points.appendChild(coordinates);
    mesh.appendChild(points) ;
    
    QDomElement faces = domdocument->createElement("faces") ;
    // QDomText indexes = domdocument->createTextNode(meshToXmlFaces(mesh));
    for(int i = 0 ; i < m->faces_.size() ; i++) {
	QDomElement face = domdocument->createElement("f") ;
	face.setAttribute("a", num.setNum(m->faces_[i]->indexes_[0])) ;
	face.setAttribute("b", num.setNum(m->faces_[i]->indexes_[1])) ;
	face.setAttribute("c", num.setNum(m->faces_[i]->indexes_[2])) ;
	face.setAttribute("shader_name", "blender_default") ;
	faces.appendChild(face) ;
    }
    // points.appendChild(indexes);
    mesh.appendChild(faces) ;
    
    return transform ;	
}

void OStreamYafray::buildDomTree(QList<QDomElement> objects)
{
    QDomElement scene = domdocument->createElement("scene");
    domdocument->appendChild(scene);
    
    QDomElement shader = domdocument->createElement("shader");
    shader.setAttribute("type","blendershader");
    shader.setAttribute("name","blender_default");
    scene.appendChild(shader);
    
    QDomElement shaderAttributes = domdocument->createElement("attributes");
    shader.appendChild(shaderAttributes);
    
    QDomElement shaderColor = domdocument->createElement("color");
    shaderColor.setAttribute("r","0.8") ;
    shaderColor.setAttribute("g","0.8") ;
    shaderColor.setAttribute("b","0.8") ;
    shaderAttributes.appendChild(shaderColor) ;
    
    QDomElement shaderSpecularColor = domdocument->createElement("specular_color");
    shaderSpecularColor.setAttribute("r","1.0") ;
    shaderSpecularColor.setAttribute("g","1.0") ;
    shaderSpecularColor.setAttribute("b","1.0") ;
    shaderAttributes.appendChild(shaderSpecularColor) ;
    
    QDomElement shaderMirrorColor = domdocument->createElement("mirror_color");
    shaderMirrorColor.setAttribute("r","1.0") ;
    shaderMirrorColor.setAttribute("g","1.0") ;
    shaderMirrorColor.setAttribute("b","1.0") ;
    shaderAttributes.appendChild(shaderMirrorColor) ;
    
    QDomElement shaderDiffuseReflect = domdocument->createElement("diffuse_reflect");
    shaderDiffuseReflect.setAttribute("value","0.8") ;
    shaderAttributes.appendChild(shaderDiffuseReflect) ;
    
    QDomElement shaderSpecularAmount = domdocument->createElement("specular_amount");
    shaderSpecularAmount.setAttribute("value","0.5") ;
    shaderAttributes.appendChild(shaderSpecularAmount) ;
    
    QDomElement shaderAlpha = domdocument->createElement("alpha");
    shaderAlpha.setAttribute("value","1.0") ;
    shaderAttributes.appendChild(shaderAlpha) ;
    
    QDomElement shaderEmit = domdocument->createElement("emit");
    shaderEmit.setAttribute("value","0.0") ;
    shaderAttributes.appendChild(shaderEmit) ;
    
    QDomElement shaderMatmodes = domdocument->createElement("matmodes");
    shaderMatmodes.setAttribute("value","traceable shadow") ;
    shaderAttributes.appendChild(shaderMatmodes) ;
    
    QDomElement shaderDiffuseBrdf = domdocument->createElement("diffuse_brdf");
    shaderDiffuseBrdf.setAttribute("value","lambert") ;
    shaderAttributes.appendChild(shaderDiffuseBrdf) ;
    
    QDomElement shaderSpecularBrdf = domdocument->createElement("specular_brdf");
    shaderSpecularBrdf.setAttribute("value","blender_cooktorr") ;
    shaderAttributes.appendChild(shaderSpecularBrdf) ;
    
    QDomElement shaderHard = domdocument->createElement("hard");
    shaderHard.setAttribute("value","50") ;
    shaderAttributes.appendChild(shaderHard) ;
    
    for(int i = 0 ; i < objects.size() ; i++)
	scene.appendChild(objects[i]) ;
    
    QDomElement light = domdocument->createElement("light");
    light.setAttribute("type",           "pointlight") ;
    light.setAttribute("glow_intensity", "0.00000000") ;
    light.setAttribute("glow_offset",    "0.00000000") ;
    light.setAttribute("glow_type",      "0"         ) ;
    light.setAttribute("name",           "LAMP1"     ) ;
    light.setAttribute("power",          "20.0000000") ;
    light.setAttribute("cast_shadows",   "off"       ) ;
    scene.appendChild(light);
    
    QDomElement lightFrom = domdocument->createElement("from") ;
    lightFrom.setAttribute("x", "5.0") ;
    lightFrom.setAttribute("y", "0.5") ;
    lightFrom.setAttribute("z", "3.5") ;
    light.appendChild(lightFrom) ;
    
    QDomElement lightColor = domdocument->createElement("color") ;
    lightColor.setAttribute("r", "1.0") ;
    lightColor.setAttribute("g", "1.0") ;
    lightColor.setAttribute("b", "1.0") ;
    light.appendChild(lightColor) ;
    
    QDomElement background = domdocument->createElement("background") ;
    background.setAttribute("type", "constant") ;
    background.setAttribute("name", "world_background") ;
    scene.appendChild(background) ;
    
    QDomElement backgroundColor = domdocument->createElement("color") ;
    backgroundColor.setAttribute("r", "1.0") ;
    backgroundColor.setAttribute("g", "1.0") ;
    backgroundColor.setAttribute("b", "1.0") ;
    background.appendChild(backgroundColor) ;
    
    QDomElement camera = domdocument->createElement("camera") ;
    camera.setAttribute("name", "MAINCAM") ;
    camera.setAttribute("type", "perspective") ;
    camera.setAttribute("resx", "800") ;
    camera.setAttribute("resy", "600") ;
    camera.setAttribute("focal", "1.093750") ;
    camera.setAttribute("aspect_ratio", "1.000000") ;
    camera.setAttribute("dof_distance", "0.000000") ;
    camera.setAttribute("aperture", "0.000000") ;
    camera.setAttribute("use_qmc", "on") ;
    camera.setAttribute("bokeh_type", "disk1") ;
    camera.setAttribute("bokeh_bias", "uniform") ;
    camera.setAttribute("bokeh_rotation", "0.000000") ;
    scene.appendChild(camera) ;
    
    QDomElement cameraFrom = domdocument->createElement("from") ;
    cameraFrom.setAttribute("x", num.setNum(viewer->camera()->position().x)) ;
    cameraFrom.setAttribute("y", num.setNum(viewer->camera()->position().y)) ;
    cameraFrom.setAttribute("z", num.setNum(viewer->camera()->position().z)) ;
    camera.appendChild(cameraFrom) ;
    
    QDomElement cameraTo = domdocument->createElement("to") ;
    cameraTo.setAttribute("x", num.setNum(viewer->camera()->viewDirection().x)) ;
    cameraTo.setAttribute("y", num.setNum(viewer->camera()->viewDirection().y)) ;
    cameraTo.setAttribute("z", num.setNum(viewer->camera()->viewDirection().z)) ;
    camera.appendChild(cameraTo) ;
    
    QDomElement cameraUp = domdocument->createElement("up") ;
    cameraUp.setAttribute("x", num.setNum(viewer->camera()->upVector().x)) ;
    cameraUp.setAttribute("y", num.setNum(viewer->camera()->upVector().y)) ;
    cameraUp.setAttribute("z", num.setNum(viewer->camera()->upVector().z)) ;
    camera.appendChild(cameraUp) ;
    
    QDomElement render = domdocument->createElement("render") ;
    render.setAttribute("camera_name", "MAINCAM") ;
    render.setAttribute("raydepth", "5") ;
    render.setAttribute("gamma", "1.000000") ;
    render.setAttribute("exposure", "0.000000") ;
    render.setAttribute("AA_passes", "2") ;
    render.setAttribute("AA_minsamples", "4") ;
    render.setAttribute("AA_pixelwidth", "1.5") ;
    render.setAttribute("AA_threshold", "0.5") ;
    render.setAttribute("bias", "0.001000") ;
    render.setAttribute("clamp_rgb", "on") ;
    render.setAttribute("background_name", "world_background") ;
    scene.appendChild(render) ;
    
    QDomElement renderOutfile = domdocument->createElement("outfile") ;
    renderOutfile.setAttribute("value", "/tmp/axel_render.tga") ;
    render.appendChild(renderOutfile) ;
}

void OStreamYafray::save()
{
    QFile file(filename);
    if( !file.open(QIODevice::WriteOnly | QIODevice::Text) )  {
	QMessageBox::information(0, "Axel","Error of reading","&Ok",0) ;
	return ;
    }
    
    QTextStream ts( &file );
    ts << domdocument->toString();
    file.close();
}

} ;
